/*******************************************************************************
 
 
 	    FILE:	DBMSScheduleFactory.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete ScheduleFactory for Database (DBMS)
				extends the abstract Class CourseScheduleFactory
 				Factory to create three instances of HolidayReader, CourseReader
 				and ScheduleWriter that target dealing with DBMS format and 
 				contains a processSchedule() method specific to handling 
 				processing DBMS.
		
 	RELATION:	DBMSScheduleFactory is a concrete factory that extends 
 				CourseScheduleFactory, it is created by ScheduleDriver.main() 
 				using refection via the Reflection class 
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

public class DBMSScheduleFactory extends CourseScheduleFactory {
	
	private String	pServer;							// name of the server
	private String	pDatabase;							// server with database
	private String	pLogin;								// user login
	private String	pPassword;							// user password
	
	private static DBMSHolidayReader   pObjHolidayReader;		// obj ref.
	private static DBMSCourseReader    pObjCourseReader;		// obj ref.
	private static DBMSScheduleWriter  pObjScheduleWriter;		// obj ref.
	
	/***************************************************************************
    *
    *	Constructor: DBMSScheduleFactory 
	*
	*	@param		mServer		name of the server
	*	@param		mDatabase	server where database lives
	*	@param		mLogin		user login
	*	@param		mPassword	user password
    */
	public DBMSScheduleFactory(String mServer, String mDatabase, 
								String mLogin, String mPassword) {
	
		super();
		
		// set DBMS info
		pServer		= mServer;
		pDatabase	= mDatabase;
		pLogin		= mLogin;
		pPassword	= mPassword;
		
		// create core objects, passing them to DBMS information
		pObjHolidayReader  = new  DBMSHolidayReader(mServer, mDatabase,
												    mLogin, mPassword);
		pObjCourseReader   = new   DBMSCourseReader(mServer, mDatabase,
												    mLogin, mPassword);
		pObjScheduleWriter = new DBMSScheduleWriter(mServer, mDatabase, 
													mLogin, mPassword);
		
	}
	
	
	/***************************************************************************
    *
    *	processSchedule
	*
    * 		Actively processes the users schedule via DBMS, using
	*		the DBMS versions of HolidayReader, CourseReader and ScheduleWriter.
    *
	*	Note: Method Declaration in CourseScheduleFactory
    */
	public void processSchedule() throws CriticalSDError {
	
		if ( ! connectToDBMS() ) {
			throw new CriticalSDError("CRITICAL ERROR: DBMS CAN'T CONNECT TO SERVER "
										 + pServer + "\n");
		}
		
		// read holiday file
		pHolidayData = pObjHolidayReader.readHolidays(pServer);
		pErrors		+= pObjHolidayReader.getHolidayReadErrors();
		
		// tell the courses reader what the holidays are
		pObjCourseReader.setHolidays(pHolidayData);
		
		// read courses file
		pCoursesData = pObjCourseReader.readCourses(pServer);
		pErrors		+= pObjCourseReader.getCourseReadErrors();
		
		// print result
		pObjScheduleWriter.writeSchedule(pHolidayData, pCoursesData, pErrors);
		
	}
	
	
	/***************************************************************************
    *
    *	cannotConnectToDBMS
	*
    * 		STUB method for testing.
    *
    */
	private boolean connectToDBMS() {
		// STUB
		return true;
	}
	
}