/*******************************************************************************
 
 
 	    FILE:	DBMSScheduleWriter.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete ScheduleWriter for Relational Database Management System (DBMS)
				extends the abstract Class ScheduleWriter	
				Specifically handles output of schedule in DBMS format.
				
 	RELATION:	DBMSScheduleWriter is a concrete factory that extends ScheduleWriter,
 				it is created by DBMSScheduleFactory().
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;


public class DBMSScheduleWriter extends ScheduleWriter {

	private String	pServer;				// name of the server
	private String	pDatabase;				// server where database lives
	private String	pLogin;					// user login
	private String	pPassword;				// user password
	
	
	/***************************************************************************
    *
    *	Constructor: DBMSScheduleWriter 
	*
	*	@param		mServer		name of the server
	*	@param		mDatabase	server where database lives
	*	@param		mLogin		user login
	*	@param		mPassword	user password
    */
	public DBMSScheduleWriter(String mServer, String mDatabase,
								String mLogin, String mPassword) {
		super();
		
		// set DBMS info
		pServer		= mServer;
		pDatabase	= mDatabase;
		pLogin		= mLogin;
		pPassword	= mPassword;
	}
	
	
	/***************************************************************************
    *
    *	writeSchedule (STUB)
	*
    * 		Specifically handles output for of schedule in DBMS format.
    *
	*	Note: Method Declaration in ScheduleWriter
    */
	protected void writeSchedule(ScheduleData mHolidayData,
									ArrayList mCoursesData,
									   String pErrors) throws CriticalSDError {
			
		// print any read errors encountered
		if ( ! pErrors.equals("") )
			System.out.println("NONFATAL ERRORS:\n" + pErrors);
			
		System.out.println("Information needed for RDBMS output:");
		System.out.println("    Server = \"" + pServer + "\"");
		System.out.println("  Database = \"" + pDatabase + "\"");
		System.out.println("     Login = \"" + pLogin    + "\"");
		System.out.println("  Password = \"" + pPassword + "\"");
		System.out.println();
	}
}