/*******************************************************************************
 
 
 	    FILE:	HolidayReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Abstract Factory to create HolidayReaders which target various platforms.
			
 	RELATION:	HolidayReader is a base abstract factory that is extended by
 				SFHolidayReader, XMLHolidayReader, and DBMSHolidayReader classes 
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.asn3;

import java.util.*;

public abstract class HolidayReader {
	
	static public Calendar	pCal;						// calendar obj

	protected ScheduleData		pObjHolidayData;
	protected String			pHolidayReadErrors;		// list of all read 
														// errors thrown by parser.
	
	
	/***************************************************************************
    *
    *	Constructor: HolidayReader 
	*
	*		Initializes fields, notably the pHolidayReadErrors string for 
	*		nonfatal errors
    */
	public HolidayReader() {

		// ensure calendar has value
		pCal = new GregorianCalendar();
	
		// initialize holiday data
		pObjHolidayData		= new ScheduleData("", "Holidays");
		pHolidayReadErrors	= "";		
	}
	
	
	/***************************************************************************
    *
    *	Empty method declarations
	*
    */
    protected abstract ScheduleData readHolidays(String mString) 
    										throws CriticalSDError;
	protected abstract void parseLine(String mLine) 
											throws ReadFileError;	
	
	
	/***************************************************************************
    *
    *	parseStrToDate
  	*
  	*		Converts a string of form "mm/dd/yy" into a Date object.
  	*		Unsucessful attempts result in null.
  	*
  	*	@param		The date string in neeed of conversion.
  	*	@return		A Date object, or null
    */
	protected Date parseStrToDate(String mDateStr) {
	
		Date mReturn = null;								// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				// month [mMonth] is 0 to 11
				pCal = new GregorianCalendar(mYear, mMonth - 1, mDay);		
				mReturn = pCal.getTime();
			}
		}
	
		return mReturn;
	}
	
	
	/***************************************************************************
    *
    *	stringToInt
    *
    *		Converts a string to an integer.
  	*
  	*	@param		The string to convert.
  	*	@return		An integer representation of the given string.
    */
	protected static int stringToInt(String mString) {
		return Integer.parseInt(mString.trim());
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for the pHolidayReadErrors String
	*
    *	@return		all read errors encountered by this FileReader instance.
    */
	public String getHolidayReadErrors() {
		return pHolidayReadErrors;
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for the pHolidays ArrayList of Dates
	*
    *	@return		Iterator to pHolidays.
    */
	public Iterator getHolidayDates() {
		return pObjHolidayData.getDates().iterator();
	}
}