/*******************************************************************************
 
 
 	    FILE:	ScheduleData.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Convienent object for storing and outputting schedule information,
 				primarily Dates.  Is instantiated by both Course Readers and Holiday Readers
	
 	RELATION:	ScheduleData objects are created by the HolidayReader() constructor and
 				CourseReader.buildCourse()
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.asn3;

import java.util.*;
import java.io.*;


public class ScheduleData {
	
	private String    pCourseNumber;			// void if excluded dates
	private String    pCourseTitle;				// = "HOLIDAYS" if holiday dates
	private ArrayList pDatesList;				// list of dates


	/***************************************************************************
    *
    *	Constructor: ScheduleData 
    *
    *		Constructs a ScheduleData object for storing and printing data.
    *
    *		Note:	Both Holidays and Courses also stored in ScheduleData objects.
    */
	public ScheduleData(String mCourseNumber, String mCourseTitle) {
		
		pCourseNumber = mCourseNumber;
		pCourseTitle  = mCourseTitle;
		pDatesList    = new ArrayList();
	}
	

	/***************************************************************************
    *
    *	getDates		[Called by Class CourseReader.setHolidays() ]
    *
    *		Acessor function to tell setHolidays() in Class CourseReader the
    *		list of excluded holidays.
	*
	*	@return		pDatesList.
    */
	public ArrayList getDates() {
		return pDatesList;
	}
	
	
	/***************************************************************************
    *
    *	getDatesIterator
    *
    *		Acessor function to return pDatesList iterator.
	*
	*	@return		Iterator to pDatesList.
    */
	public Iterator getDatesIterator() {
		return pDatesList.iterator();
	}
	
	
	/***************************************************************************
    *
    *	addDate 
    *
    *		Adds the Date [mDate] to list pDatesList.
	*
	*	@param		Date to be added to this ScheduleData.
    */
	public void addDate(Date mDate) {
		
		if (mDate != null)						// check for valid date
			pDatesList.add(mDate);				// add
	}
	
	
	/***************************************************************************
    * 
    *	fixDateString
    *	
    *		The toString of Date is tokenized and formatted ("fixed"), so not to print
    *		HH:MM:SS and EST values
    *
    *
    *	@param		Date to read toString from
    *	@return		the resulting string in form "Mon Jan 05, 2004"
    */
	public String fixDateString(Date mDate) {
	
		String mReturn = "";
		
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
		
															// tokens
		mReturn += mTokenDate.nextToken() + " ";			// DAY
		mReturn += mTokenDate.nextToken() + " ";			// MONTH
		mReturn += mTokenDate.nextToken() + ", ";			// DATE
		mTokenDate.nextToken();								// HH:MM:SS
		mTokenDate.nextToken();								// EST
		mReturn += mTokenDate.nextToken() + " ";			// YEAR
		
		return mReturn;
	}
	
	
	/***************************************************************************
    * 
    *	Override the toString() method of java.lang.Object for printing.
    *	
    *		Creates a nice display string of schedule data dates.
    *
    */
	public String toString() {
	
		String mReturn = "\n" + pCourseNumber + pCourseTitle + "\n\n";
		
		int itemCount = 1;
		for (Iterator e = pDatesList.iterator(); e.hasNext(); )
			mReturn += itemCount++ + ": " + 
								fixDateString( (Date) e.next()) + "\n";
		
		return mReturn;
	}
}

