/*******************************************************************************
 
 
 	    FILE:	ScheduleDriver.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	   USAGE:	java cscie247.asn3.ScheduleDriver [configfile]
 
 	 PURPOSE:	Initiates the process of creating a schedule.  Contains main().
 				Reads arguments from a config file, then acts upon those 
 				arguments via the Reflection class.  The processSchedule() 
 				method is invoked, which processes given arguments to create a 
 				schedule targeted for various platforms, namely Sf, XML or DBMS.

 	RELATION:	ScheduleDriver contains main(). ScheduleDriver creates 
 				Reflection class and is responsiible for creating (via reflection)
 				the SFScheduleFactory, XMLScheduleFactory or DBMSScheduleFactory,
 				depending on the needs of the user.
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn3;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class ScheduleDriver {
	
	/***************************************************************************
    *
    *	Constructor: ScheduleDriver
    *
    */
	public ScheduleDriver() {
	}
	
	
	/***************************************************************************
    *
    *	readConfigFile
    *
	*		Reads the config file [mFile] and converts its contents into 
	*		an ArrayList
    *
	*	@param mFile	name of the config file to be read.
    *	@return			An ArrayList of parsed Strings, to be used as args.
    */
	public static ArrayList readConfigFile(String mFile) throws Exception, 
													FileNotFoundException {
		
		ArrayList mReturn = new ArrayList();
		
		// open file for reading
		FileInputStream   mFIS = new FileInputStream(mFile);
		InputStreamReader mISR = new InputStreamReader(mFIS);
		BufferedReader    mBR  = new BufferedReader(mISR);
		
		String mLine = mBR.readLine();				// get first char (or EOF)
		while (mLine != null) {
		
			mLine = mLine.trim();
			if (mLine.length() > 0)
				mReturn.add(mLine);

			mLine = mBR.readLine();					// get next line
		}
		
		mFIS.close();								// close reader
            
		return mReturn;
	}


	/***************************************************************************
    *
    *	Main
    *
    *	  Main expects one argument containing the config file name.
	*		The first line in the config file is used as the
	*		 Class name to be instantiated via reflection, subsequent
	*		 lines are passed to the constructor of that Class as args.
	*		There is a Reflection class to handle the complexies of reflection.
    *
    *		All fatal error Exceptions printed from main()
    */	
	static public void main(String args[]) {
		
		if (args.length == 1) {
		
			ArrayList mArgsLst = new ArrayList();	// default to empty
			
			try {
				mArgsLst = readConfigFile(args[0]);	// get args from config file
			} catch (FileNotFoundException e) {
				System.out.println("CRITICAL ERROR: CONFIG FILE \"" 
									+ args[0] + "\" NOT FOUND");
			} catch (Exception e) {
				System.out.println("CRITICAL ERROR: UNABLE TO LOAD FROM \"" 
									+ args[0] + "\"");
				System.out.println("DUE TO: " + e);
			}
			
			Iterator mItr = mArgsLst.iterator();
			if ( mItr.hasNext() ) {
			
				// remove the first argument in mArgsLst to use as the Class name
				String mClassName = (String) mItr.next();
				mItr.remove();
				
				try {
					
					// build appropriate factory using Reflection class
					Reflection objScheduleFactory = 
										new Reflection(mClassName, mArgsLst);
					
					// invoke processSchedule() method using Reflection class
					// processSchedule() requires no args, so use mEmptyArgs
					ArrayList mEmptyArgs = new ArrayList();				
					objScheduleFactory.invokeMethod("processSchedule", mEmptyArgs);
					
				
				} catch (InvocationTargetException e) {
				
					// CriticalSDError Exception caught by Reflection Class
					System.out.println(e.getCause().getMessage());
					
				} catch (ClassNotFoundException e) {
				
					// Config file using an invalid Class name
					System.out.println("CLASS NOT FOUND: " + e.getMessage() 
											+ "  CHECK YOUR CONFIG FILE.");
				
				} catch (Exception e) {
				
					// Errors not caused by user [programming errors].
					if (e instanceof NoSuchMethodException  ||
						e instanceof IllegalAccessException ||
						e instanceof IllegalArgumentException ||
						e instanceof InstantiationException ||
						e instanceof SecurityException) {
						
						// Reflection Exception, most likely NoSuchMethodException
						System.out.println("REFLECTION ERROR: " + e);
						
					} else {
						
						// All other Exceptions
						//e.printStackTrace();
						System.out.println("UNUSUAL ERROR: " + e);
					}
				}
			} else
				System.out.println("CRITICAL ERROR: BAD CONFIG FILE");
		} else
			System.out.println("CRITICAL ERROR: 1 parameter required. " 
										+ "ScheduleDriver [configFile]");
    }
}