/*******************************************************************************
 
 
 	    FILE:	DistressDriver.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Assignment Four, 

 
 	 PURPOSE:	Used to test drive and simulate the application.
                Objects are fabricated and manipulated within Distress, then
                their values checked using DistressWriter
				
ASSOCIATIONS:	Entry point to running and testing Distress
                Instantiates Distress and DistressWriter
 	
	MODIFIED:	November 30th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn4;

import java.util.*;
import cscie247.distress.*;

public class DistressDriver {
	
    /**																		   |
    *
    *	DistressDriver Constructor.
    *
    */
	public DistressDriver() {
    }
    
    
    /**																		   |
    *
    *	Creates a set of objects matching the three scenarios from Assignment 2
    *
    */    
    public static Distress fabricateObjects() {
    
        Distress mObjDistress = Distress.getDistress();
		
        User mUser;
        Property mProperty;
        PropertyProfile mPProfile;
        
        // create a set of objects matching the three scenarios
        
        // fabricate Scenario 1.
		try {
        
			// Snopes registers as a Realtor
            mUser = Distress.register (TypeUser.REALTOR,
												"J. Snopes",
												"12345 Some Street",
												"Cambridge, MA 02138",
												"jsnopes",
												"themansion",
												"jsnopes@faulkner.net",
												"Visa",
												"2345234987634",
												"Yoknopatawpha Realty");
        
            // make Snopes's SingleFamily property
            // note I have set mBaths to 1.5.  mBaths was not specified on the Instance Diag.
            mProperty = new SingleFamily(mUser,
                                                    "A 2341 Jason St",
                                                    "Arlington, MA 02474",
                                                    6,
                                                    1.5,
                                                    2,
                                                    yyyyToDate("1945"),
                                                    TypeParking.OFFSTREET,
                                                    500000.00,  
                                                    1500,
                                                    "Hardwood floors, finished basement",
                                                    "A gem in a quiet neighborhood.  Buy it now!",
                                                    1,
                                                    5445);
 
        
            // Snopes adds his SingleFamily property
            try {
                mUser.add(mProperty);
            } catch (ErrDistress e) { System.out.println(e.getMessage()); }
		} catch (ErrRegistration e) {
			System.out.println(e.getMessage());
		} catch (ErrDistress e) {
            System.out.println(e.getMessage());
        }
        
        
        // fabricate Scenario 2.
        
		try {
        
			// Hank Hook of Captain Capital Mortgage registers as a lender
			mUser = mObjDistress.register (TypeUser.LENDER,
										   "Hank Hook",
										   "67890 Other Street",
										   "Cambridge, MA 02138",
										   "hankhook",
										   "ccm_hook",
										   "hhook@ccmortgage.com",
										   "Mastercard",
										   "8767498765437",
										   "Captain Capital Mortgage");
			
			MortgageOffering mMortgage = new MortgageOffering(mUser,
															  mmddyyyyToDate("4/4/2002"),
															  TypeMortgage.FIXEDRATE, 
															  120,
															  6.3,
															  5.0,
															  1.2);
                                                                
            // Hank adds a mortgage offering
            try {
                mUser.add(mMortgage);
            } catch (ErrDistress e) { System.out.println(e.getMessage()); }
                
            
            mMortgage = new MortgageOffering(mUser,
                                                mmddyyyyToDate("11/5/2001"),
                                                TypeMortgage.VARIABLE, 
                                                220,
                                                6.2,
                                                5.0,
                                                1.3);
            // Hank adds a mortgage offering
            try {
                mUser.add(mMortgage);
            } catch (ErrDistress e) { System.out.println(e.getMessage()); }
            
		} catch (ErrRegistration e) {
			System.out.println(e.getMessage());
		} catch (ErrDistress e) {
            System.out.println(e.getMessage());
        }
			
            
        // fabricate Scenario 3.
		try {
            
			// Ma and Pa Kettle registers as a cunsumer
			mUser = mObjDistress.register (TypeUser.CONSUMER,
										   "Ma & Pa Kettle",
										   "1400 Settle Street",
										   "Cambridge, MA 02138",
										   "ma_pa_kettle",
										   "ma_and_pa",
										   "ma_pa@hotmail.com",
										   "Visa",
										   "345789897631",
										   TypePurchasingTF.WITHINTWOMONTHS);
            
            
			// construct empty search criteria
			SearchCriteria mSearch = new SearchCriteria();
			// add search criteria looking for property
			mSearch.addObject("cityStateZip", "Arlington, MA 02474");
			mSearch.addObject("propertyType", TypeProperty.SINGLEFAMILY);
			mSearch.addInteger("parking", 2, 4);
			mSearch.addDouble("listingPrice", 300000, 600000);
			// add search criteria looking for mortgage
			mSearch.addObject("mortgageType", TypeMortgage.FIXEDRATE);
			mSearch.addDouble("interestRate", 0, 6.3);
			// NOTE: This PropertyProfile search is not used in main()
            //		main creates its own Search, sending it directly, 
            //		as outlined by the spec.
			
            mPProfile = new PropertyProfile(mUser, mSearch);
															  
															  
			// Ma & Pa add their search criteria
			try {
				mUser.add(mPProfile);
			} catch (ErrDistress e) { System.out.println(e.getMessage()); }
			
		} catch (ErrRegistration e) {
			System.out.println(e.getMessage());
		} catch (ErrDistress e) {
            System.out.println(e.getMessage());
        }
        
        return mObjDistress;
    }
    
	
	/**																		   |
  	*
  	*	Converts a string of form "mm/dd/yy" into a Date object.
  	*	Unsucessful attempts result in null.
  	*
  	*	@param String	The date string in neeed of conversion.
  	*
  	*	@return			A Date object, or null
    */
	public static Date mmddyyyyToDate(String mDateStr) {

		Date mReturn = null;							// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				// month is 0 to 11
				Calendar mCal = new GregorianCalendar(mYear, mMonth - 1, mDay);		
				mReturn = mCal.getTime();
			}
		}
	
		return mReturn;
	}
	
	
	/**																		   |
  	*
  	*	Converts a string of form "yyyy" into a Date object.
  	*
  	*	@param String	The date string in neeed of conversion.
  	*
  	*	@return			A Date object, or null
    */
	public static Date yyyyToDate(String mDateStr) {
		int mYear  = stringToInt(mDateStr);
		Calendar mCal = new GregorianCalendar(mYear, 1, 1);	
		return mCal.getTime();
	}
	
	
	/**																		   |
    *
    *	Convert a string to an int.
  	*
  	*	@param	String		String to convert.
  	*	@return				int representation of given string.
    */
	private static int stringToInt(String mString) {
		return Integer.parseInt(mString.trim());
	}
	
	
    /**																		   |
    * 
    *	Main.
    *	
    *	As outlined in spec.
    *	  1. Invoikes fabricateObjects(), which returns a refernce to  
    *		 the Distress object.
    *	  2. Creates an instace of DistressWriter, a text output class that 
    *		 "dumps" the set of objects to the screen, and invoke its dumpAll() 
    *		 method so to display fabricated objects.
    *	  3. Create a set of name-value pairs in a search object sufficient for 
	*		 finding the Arlington property listed by J. Snopes and  
    *		 invoke Distress.findProperty().  Use DistressWriter to print  
    *		 Property objects found
    *	  4. Remove User with id of "jsnopes" by invoking remove(), then run
    *		 DistressWriter.dumpAll() to show that that object and none of its 
    *		 child objects show up again
    *
	*	@param	String	Args sent to main by user
    */
    public static void main(String[] argv) {
    
		Distress mObjDistress		= fabricateObjects();

		DistressWriter mObjWriter   = new DistressWriter(mObjDistress);
		
		DistressWriter.dumpAll();
		
		// create name-value "pairs" in a SearchCriteria object
        // sufficient for finding the Arlington property listed by J. Snopes
		SearchCriteria mSearch = new SearchCriteria();
		mSearch.addObject("cityStateZip", "Arlington, MA 02474");
		mSearch.addObject("propertyType", TypeProperty.SINGLEFAMILY);
		mSearch.addInteger("parking", 2, 4);
		mSearch.addDouble("listingPrice", 300000, 600000);
		
		// print search result for properties
		try {
			System.out.println(">>>>> Property MATCHES\n");
			ArrayList mSearchResult = Distress.findProperty(mSearch);
            
            Iterator mItr = mSearchResult.iterator();
            while(mItr.hasNext())
                DistressWriter.prtProperty( (Property) mItr.next());
            
            
		} catch (Exception e) { System.out.println(e.getMessage()); }
		
		try {
			// Invoke Distress.remove with id of "jsnope."
			User mUser = Distress.getUser("jsnopes");
			Distress.remove(mUser);
		} catch (Exception e) { System.out.println(e.getMessage()); }
		
		DistressWriter.dumpAll();
		
	}
}