/*******************************************************************************
 
 
 	    FILE:	Consumer.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four, 

 
 	 PURPOSE:	This represents someone who registers to search for a property,
                presumably to buy.  This extends the class User, and adds only
                one bit of information, the time-frame in which the Consumer
                might consider buying.
				
ASSOCIATIONS:	A Consumer can have zero or one PropertyProfile objects.
                [Note, a more liberal system might allow a user to have
                multiple names PropertyProfiles, such as "MyCambridgeDreamHome",
                so long as the user is willing to pay for them.]
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;


public class Consumer extends User {

	private PropertyProfile	 pPropertyProfile;		// ...0 or 1
    private TypePurchasingTF pPurchasingTimeFrame;	// Values can represent 
                                                    //  "within two months", 
                                                    //  "three-six months", 
                                                    //  "just looking"
    
    /**																		   |
    *
    *	Consumer Constructor.
    *
    */

	public Consumer(TypeUser mUserType,
					String 	 mName,
                    String	 mAddress,
                    String	 mCityStateZip,
                    String	 mLoginName,
                    String	 mPin, 
                    String	 mEmail, 
                    String	 mCredotCardCo, 
                    String	 mCreditCardNumber,
                    TypePurchasingTF mPurchasingTimeFrame) {
                    
        super(mUserType, mName, mAddress, mCityStateZip, mLoginName, 
					mPin, mEmail, mCredotCardCo, mCreditCardNumber);
        
		pPropertyProfile	 = null;
        pPurchasingTimeFrame = mPurchasingTimeFrame;
    }
    
    
	/**																		   |
    * 
    *	Returns an iterator to the pProperties list
	*
    *	@return	Iterator		Iterator to pProperties.
    */
    public Iterator list() {
		// create a list so to pass an Iterator
		ArrayList mList = new ArrayList();
		
		if (pPropertyProfile != null) {
			mList.add(pPropertyProfile);
		}
		
        return mList.iterator();
    }
    

	/**																		   |
    * 
    *	Adds the Property instance mProperty to the pProperties list,
    *	 if it is not already.
	*
    *	@param	Property		Property instance to be added to pUsers list.
    */
    public void add(Object mObject) {
		
		pPropertyProfile = (PropertyProfile) mObject;
    }
    
    
	/**																		   |
    * 
    *	Removes the Property instance mProperty from the pProperties list,
    *	 if found.
	*
    *	@param	Property	Property instance to be removed from pUsers list.
    */
    public void remove(Object mObject) {
		pPropertyProfile = null;
    }
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	
	public PropertyProfile getPropertyProfile() {
		return pPropertyProfile;
	}
	public void setPropertyProfile(PropertyProfile mPropertyProfile) {
		 pPropertyProfile = mPropertyProfile;
	}
	
	public TypePurchasingTF  getPurchasingTimeFrame() {
        return pPurchasingTimeFrame;
    }
	public void setPurchasingTimeFrame(TypePurchasingTF mPurchasingTimeFrame) {
        pPurchasingTimeFrame = mPurchasingTimeFrame;
    }
}