/*******************************************************************************
 
 
 	    FILE:	Lender.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four, 

 
 	 PURPOSE:	This represents an individual lender who has one or more
                mortgage offerings to advertise.  Class Lender extends class
                User, and adds the name of the company the lender is affiliated
                with, if any.
                
				
ASSOCIATIONS:	A Lender owns a collection of MortgageOfferings. 
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;


public class Lender extends User {

    private String		pCompanyName;		// The name of the company for whom
                                            //  the lender works, if any.
                                            // can be null.
    private ArrayList	pMortgageOfferings;	// List of MortgageOffering
    										//  instances owned by this realtor
    
    /**																		   |
    *
    *	Lender Constructor.
    *
    */
	public Lender(TypeUser mUserType,
				  String   mName,
				  String   mAddress,
				  String   mCityStateZip,
				  String   mLoginName,
				  String   mPin, 
				  String   mEmail, 
				  String   mCredotCardCo, 
				  String   mCreditCardNumber,
				  String   mCompanyName) {
                   
        super(mUserType, mName, mAddress, mCityStateZip, mLoginName, mPin, 
        			mEmail, mCredotCardCo, mCreditCardNumber);
        
        pCompanyName       = mCompanyName;
        pMortgageOfferings = new ArrayList();			// init list
    }

    
	/**																		   |
    * 
    *	Returns an iterator to the pMortgageOfferings list
	*
    *	@return	Iterator		Iterator to pMortgageOfferings.
    */
    public Iterator list() {
        return  pMortgageOfferings.iterator();
    }
    

	/**																		   |
    * 
    *	Adds the MortgageOffering instance mMortgageOffering to the 
    *	 pMortgageOfferings list, if it is not already.
	*
    *	@param	MortgageOffering		MortgageOffering instance to be added to
    *									 pUsers list.
    */
    public void add(Object mObject) throws ErrDistress {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null)
            pMortgageOfferings.add(mObject);
        else
			throw new ErrDistress("CAN'T ADD ITEM. MortgageOffering \"" + 
								  ((MortgageOffering) mObject).getOfferingID() +
								  "\" is already in " + this.getName() +
								  "'s User List.");
    }
    
    
	/**																		   |
    * 
    *	Removes the MortgageOffering instance mMortgageOffering from the 
    *	 pMortgageOfferings list, if found.
	*
    *	@param	MortgageOffering	MortgageOffering instance to be removed from
    *								 pUsers list.
    */
    public void remove(Object mObject) throws ErrDistress {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null)
			throw new ErrDistress("CAN'T REMOVE ITEM. MortgageOffering \"" + 
								  ((MortgageOffering) mObject).getOfferingID() +
								  "\" is not in " + this.getName() +
								  "'s User List.");
        else
            mItr.remove();
    }
    
    
	/**																		   |
    * 
    *	Searches through the pMortgageOfferings list looking for a MortgageOff   
    *	instance with the same reference as mMortgageOff.  If found, an Iterator  
    *	is returned resting on said list item (so it may be removed easily), 
    *	otherwise a null is returned.
	*
    *	@return	Iterator	points to item mUser in pUsers. if mUser can't be
    *						found then returns null
    */
	private Iterator checkFor(Object mMortgageOffering) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mMortgageOffering == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }
    
	
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public String  getCompanyName() {
        return pCompanyName;
    }
	public void setCompanyName(String mCompanyName) {
        pCompanyName = mCompanyName;
    }
    
	public ArrayList  getMortgageOfferings() {
        return pMortgageOfferings;
    }
	public void setMortgageOfferings(ArrayList mMortgageOfferings) {
        pMortgageOfferings = mMortgageOfferings;
    }
}