/*******************************************************************************
 
 
 	    FILE:	MultiFamily.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four

 
 	 PURPOSE:	This extends the class Property and describes a house with
                multiple rentable units.  Some of the attributes inherited
                from Property (such as numberOfRooms) have the total value
                across all its units.
				
ASSOCIATIONS:	A MultiFamily object owns one or more MFUnits
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;

public class MultiFamily extends Property {

    private int		  pNumberOfUnits;		// number of MFUnits
    private ArrayList pMFUnits;				// list of MFUnits on property
	
    /**																		   |
    *
    *	MultiFamily Constructor.
    *
    */
	public MultiFamily(User			mObjContainer,
                       String		mAddress,
                       String		mCityStateZip,
                       int			mRooms,
                       double		mBaths,
                       int			mParking,
                       Date			mYearBuilt,
                       TypeParking	mParkingType,
                       double		mListingPrice,
                       int			mLivingSpace,
                       String		mSpecialFeatures,
                       String		mDescription,
                       int			mNumberOfFloors) {
        
		
		
        super(TypeProperty.MULTIFAMILY, mObjContainer, mAddress, mCityStateZip,  
              mRooms, mBaths, mParking, mYearBuilt, mParkingType, mListingPrice,
              mLivingSpace, mSpecialFeatures, mDescription, mNumberOfFloors);

		pMFUnits		= new ArrayList();
		updateNumUnits();							// Units must be added
													// using add();
													
    }
    
	/**																		   |
    *
    *	This method takes the name-value SearchCriteria and determines if the
    *	Property object matches them.  It ignores any names that it cannot 
    *	understand, and throws an exception if the value associated with a name
    *	does not belong to an appropriate type.  This returns false if the 
    *	Property object doesn't match any of the values given.
    *	match() throws an exception if the property and city are not given.
    *	Note: the search criteria generally describe bounds on an attribute
    *	(e.g., one criterion might be that the minimum number of rooms is 6).
    *
    *	Note: Overrides match in Property
    */
    public boolean match(SearchCriteria mCriteria) throws ErrDistress {
		
        if ( ! mCriteria.getHasPropertyTypeAndCity() )
            throw new ErrDistress("SearchCriteria does not have both"
                                    + "property type and city");
		
        boolean mReturn = true;						// assume match
        
		Iterator	mItr = mCriteria.list();
		SearchItem	mSearchItem;
		
		while(mItr.hasNext() && mReturn == true) {
			mSearchItem = (SearchItem) mItr.next();
		
            mReturn = compare(mSearchItem.getVarName(),
                              mSearchItem.getMin(),
                              mSearchItem.getMax());
		}
        
		return mReturn;
	}
      
	
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
		
		boolean mReturn = true;						// assume match
		
		if (mVarName.equals("numberOfUnits")) {
			if (! testInteger(pNumberOfUnits, mMin, mMax) )
				mReturn = false;
		}
        
        if ( ! super.compare(mVarName, mMin, mMax) ) {
            mReturn = false;
        }
        
         // compare for each MF unit?
         
		return mReturn;
	}
	
	
	/**																		   |
    * 
    *	Returns an iterator to the pMFUnits list
	*
    *	@return	Iterator		Iterator to pMFUnits.
    */
    public Iterator list() {
        return  pMFUnits.iterator();
    }
    

	/**																		   |
    * 
    *	Adds the Property instance mProperty to the pMFUnits list,
    *	 if it is not already.
	*
    *	@param	Property		Property instance to be added to pUsers list.
    */
    public void add(Object mObject) throws ErrDistress {
        
        if ( getUnit( ((MFUnit) mObject).getUnitNumber() ) != null )
            throw new ErrDistress("CAN'T ADD ITEM." + 
                        "An MFUnit with the same unit number already exists.");
        
        Iterator mItr = checkFor(mObject);
         
        if (mItr == null)
            pMFUnits.add(mObject);
        else
            throw new ErrDistress("CAN'T ADD ITEM. MFUnit is already in " +
						this.getPropertyID() + " MultiFamily Property.");
						
		updateNumUnits();
    }
    
    
	/**																		   |
    * 
    *	Removes the Property instance mProperty from the pMFUnits list,
    *	 if found.
	*
    *	@param	Property	Property instance to be removed from pUsers list.
    */
    public void remove(Object mObject) throws ErrDistress {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null)
			throw new ErrDistress("CAN'T REMOVE MFUnit. It is not in " +
						this.getPropertyID() + " MultiFamily Property.");
        else
            mItr.remove();
			
		updateNumUnits();
    }
    
	
	/**																		   |
    * 
    *	Returns the number of units in this MultiFamily by counting the number
    *	of objects in pMFUnits
    *
    *	Note: MultiFamily should always have at least 2 units
    *	
    */
	private void updateNumUnits() {
		pNumberOfUnits = pMFUnits.size();			// update number of units
													// should have 2 or more.
	}
	
    
	/**																		   |
    * 
    *	Returns a MFUnit with specified pUnitNumber
	*
    *	@param	int		MFUnit number being looked for
    *	@return	MFUnit	MFUnit with unitNumber matching parameter
    */
    private MFUnit getUnit(int mUnitNumber) {
        MFUnit mReturn = null;
        
        Iterator mItr = list();
        MFUnit mMFUnit;
        
        while(mItr.hasNext() && mReturn == null) {
            mMFUnit = (MFUnit) mItr.next();
            if ( mMFUnit.getUnitNumber() == mUnitNumber )
                mReturn = mMFUnit;
        }
        
        return mReturn;
    }
    
    
	/**																		   |
    * 
    *	Searches through the pMFUnits list looking for a Properties instance  
    *	with the same reference as mProperties.  If found, an Iterator is 
    *	returned resting on said list item (so it may be removed easily), 
    *	otherwise a null is returned.
	*
    *	@return	Iterator	points to item mUser in pUsers. if mUser can't be
    *						found then returns null
    */
	private Iterator checkFor(Object mObject) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mObject == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }
  
    
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public ArrayList getMFUnits() {
		return pMFUnits;											// read only
	}
	
	public int  getNumberOfUnits() {
        return pNumberOfUnits; 						// read only
                                                    // use add() and remove()
                                                    // to change pNumberOfUnits
    }
}