/*******************************************************************************
 
 
 	    FILE:	Property.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four

 
 	 PURPOSE:	This is a compendium of information about a particular Property.
				
ASSOCIATIONS:	This is owned by a Realtor, and it should be possible to find
                the Realtor from the Property object.
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;


public class Property {
	
	private static int pPropertyIDCounter = 0;
	
	private TypeProperty pPropertyType;
	private User		 pObjContainer;		// reference to Container object
    private String		 pPropertyID;		// Assigned by system, read only
                                            //  (called 'tracking number' in 
                                            //  the assignment statement.
    private String		 pAddress;			// number and street, unit number
                                            //  if needed
    private String		 pCityStateZip;		// Town, State 5-digit zipcode
    private int			 pRooms;			// Bedrooms, kitchen, living room,
                                            //  dining room, den, other
                                            // non-bathroom/non-closet
    private double		 pBaths;			// Have to allow 1.5 baths
    private int			 pParking;			// Number of spaces available
    private	Date		 pYearBuilt;		// Month/Day/Year
    private	TypeParking	 pParkingType;		// Off-street, not available, garage
    private double		 pListingPrice;		// Dollars and cents
    private int			 pLivingSpace;		// Integral number of square feet in
                                            //  the house
    private	String		 pSpecialFeatures;	// Wax poetic in 25 words or less
    private	String		 pDescription;		// even more lyrical in 256 chars
    private	Date		 pDateListed;		// 4-digit year
    private	int			 pNumberOfFloors;	// Number of floors in property

    /**																		   |
    *
    *	Property Constructor.
    *
    */
	public Property(TypeProperty mPropertyType,
					User		 mObjContainer,
                    String		 mAddress,
                    String		 mCityStateZip,
                    int			 mRooms,
                    double		 mBaths,
                    int			 mParking,
                    Date		 mYearBuilt,
                    TypeParking	 mParkingType,
                    double		 mListingPrice,
                    int			 mLivingSpace,
                    String		 mSpecialFeatures,
                    String		 mDescription,
                    int			 mNumberOfFloors) {
    
		pPropertyType	 = mPropertyType;
		pObjContainer	 = mObjContainer;
        pPropertyID		 = generateId();
        pAddress		 = mAddress;
        pCityStateZip	 = mCityStateZip;
        pRooms			 = mRooms;
        pBaths			 = mBaths;    // ENSURE VALUE whole or .5  not .3333 !
        pParking		 = mParking;
        pYearBuilt		 = mYearBuilt;
        pParkingType	 = mParkingType;
        pListingPrice	 = mListingPrice;
        pLivingSpace	 = mLivingSpace;
        pSpecialFeatures = mSpecialFeatures;
        pDescription	 = mDescription;
        pNumberOfFloors	 = mNumberOfFloors;
        
		/// Get when item was listed (now).
		GregorianCalendar mCalendar = new GregorianCalendar();
		pDateListed		 = mCalendar.getTime();    	// today	
    }
    
    
	/**																		   |
    * 
    *	generate a unique ID for Property objects
	*
    *	@return	String		ID in string format
    */
	private static String generateId() {
		
		pPropertyIDCounter++;
		
		String mID = "" + pPropertyIDCounter;
		
		for (int i = 6 - mID.length(); i > 0; i--)
			mID = "0" + mID;
		
		mID = "Prop" + mID;
		
		return (mID);
	}
	
    
	/**																		   |
    *
    *	the following small "test" methods are called by compare() to preform
    *	actual comparisons or range test of ints, doubles, strings, Dates
    *	and enumerated types
    *
    */
	protected boolean testInteger(int mVar, Object mMin, Object mMax) {
	
		Integer mTest = new Integer(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );											  
		return ( (testMin == +1 && testMax == -1) || 
				   testMin == 0 || testMax == 0);
	}
	protected boolean testDouble(double mVar, Object mMin, Object mMax) {
	
		Double mTest = new Double(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );											  
		return ( (testMin == +1 && testMax == -1) || 
				   testMin == 0 || testMax == 0);
	}
	protected boolean testString(String mVar, Object mVal) {
		return ( mVar.compareTo(mVal) == 0 );
	}
	protected boolean testDate(Date mTestDate, Object mMin, Object mMax) {
		// typecast objects
		Date mDateStart = (Date) mMin;
		Date mDateEnd   = (Date) mMax;
		
		return (mTestDate.after(mDateStart) && mTestDate.before(mDateStart));
	}
	protected boolean testEnum(Object mVar, Object mVal) {
		return ( mVar == mVal );
	}
	
    
	/**																		   |
    *
    *	This method takes the name-value SearchCriteria and determines if the
    *	Property object matches them.  It ignores any names that it cannot 
    *	understand, and throws an exception if the value associated with a name
    *	does not belong to an appropriate type.  This returns false if the 
    *	Property object doesn't match any of the values given.
    *	match() throws an exception if the property and city are not given.
    *	Note: the search criteria generally describe bounds on an attribute
    *	(e.g., one criterion might be that the minimum number of rooms is 6).
    *
    *	Note: May be overwritten by MultiFamily, SingleFamily or Condominium
    */
    public boolean match(SearchCriteria mCriteria) throws ErrDistress {
		
        if ( ! mCriteria.getHasPropertyTypeAndCity() )
            throw new ErrDistress("SearchCriteria does not have both"
                                    + "property type and city");
        
        boolean mReturn = true;						// assume match
        
		Iterator	mItr = mCriteria.list();
		SearchItem	mSearchItem;
		
		while(mItr.hasNext() && mReturn == true) {
			mSearchItem = (SearchItem) mItr.next();
		
            mReturn = compare(mSearchItem.getVarName(),
                              mSearchItem.getMin(),
                              mSearchItem.getMax());
		}
			
        return mReturn;
    }
    
    
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
        
        boolean mReturn = true;						// assume match
        
        if (mVarName.equals("propertyType")) {
            if (! testEnum(pPropertyType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("address")) {
            if (! testString(pAddress, mMin) )
                mReturn = false;
        } else if (mVarName.equals("cityStateZip")) {
            if (! testString(pCityStateZip, mMin) )
                mReturn = false;
        } else if (mVarName.equals("rooms")) {
            if (! testInteger(pRooms, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("baths")) {
            if (! testDouble(pBaths, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parking")) {
            if (! testInteger(pParking, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("yearBuilt")) {
            if (! testDate(pYearBuilt, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parkingType")) {
            if (! testEnum(pParkingType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("listingPrice")) {
            if (! testDouble(pListingPrice, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("livingSpace")) {
            if (! testInteger(pLivingSpace, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("specialFeatures")) {
            if (! testString(pSpecialFeatures, mMin) )
                mReturn = false;
        } else if (mVarName.equals("description")) {
            if (! testString(pDescription, mMin) )
                mReturn = false;
        } else if (mVarName.equals("numberOfFloors")) {
            if (! testInteger(pNumberOfFloors, mMin, mMax) )
                mReturn = false;
        }
        
        return mReturn;
    }
    
    
    
	/**																		   |
    * 
    *	Obtains the email of the owner of this listing.
	*
    *	@return	String		the email of the owner who listed this item
    */
    private String obtainEmail() {
        return pObjContainer.getEmail();
    }
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
    
	// avoids instanceof and allows quick dynamic referencing
	public TypeProperty getPropertyType() {							// read only
		return pPropertyType;
	}
	
	public User getObjContainer() {							
		return pObjContainer;										// read only
                            // reference to the Realtor who listed this Property
	}
	
	public String getPropertyID() {
        return pPropertyID;											// read only
    }
    
	public String getAddress() {
        return pAddress;
    }
	public void setAddress(String mAddress) {
        pAddress = mAddress;
    }
    
	public String getCityStateZip() {
        return pCityStateZip;
    }
	public void setCityStateZip(String mCityStateZip) {
        pCityStateZip = mCityStateZip;
    }
    
	public int getRooms() {
        return pRooms;
    }
	public void setRooms(int mRooms) {
        pRooms = mRooms;
    }
    
	public double getBaths() {
        return pBaths;
    }
	public void setBaths(double mBaths) {
        pBaths = mBaths;
    }
    
	public int getParking() {
        return pParking;
    }
	public void setParking(int mParking) {
        pParking = mParking;
    }
    
	public Date getYearBuilt() {
        return pYearBuilt;
    }
	public void setYearBuilt(Date mYearBuilt) {
        pYearBuilt = mYearBuilt;
    }
    
	public TypeParking getParkingType() {
        return pParkingType;
    }
	public void setParkingType(TypeParking mParkingType) {
        pParkingType = mParkingType;
    }
    
	public double getListingPrice() {
        return pListingPrice;
    }
	public void setListingPrice(double mListingPrice) {
        pListingPrice = mListingPrice;
    }
    
	public int getLivingSpace() {
        return pLivingSpace;
    }
	public void setLivingSpace(int mLivingSpace) {
        pLivingSpace = mLivingSpace;
    }
    
	public String getSpecialFeatures() {
        return pSpecialFeatures;
    }
	public void setSpecialFeatures(String mSpecialFeatures) {
        pSpecialFeatures = mSpecialFeatures;
    }
    
	public String getDescription() {
        return pDescription;
    }
	public void setDescription(String mDescription) {
        pDescription = mDescription;
    }
    
	public Date getDateListed() {
        return pDateListed;
    }
	public void setDateListed(Date mDateListed) {
        pDateListed = mDateListed;
    }
    
	public int getNumberOfFloors() {
        return pNumberOfFloors;
    }
	public void setNumberOfFloors(int mNumberOfFloors) {
        pNumberOfFloors = mNumberOfFloors;
    } 
 }