/*******************************************************************************
 
 
 	    FILE:	Realtor.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four, 

 
 	 PURPOSE:	This represents an individual realtor who has properties for
                which they represent the seller.  Class Realtor extends class
                User, and adds the name of the agency, if any.
				
ASSOCIATIONS:	A Realtor owns a collection of PropertyListings.
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;


public class Realtor extends User {

    private String	  pAgency;				// The agency for which the Realtor 
                                            //  works.  This can be null
    
    private ArrayList pProperties;			// list of Properties listed 
    										//  by this Realtor
    
    
    /**																		   |
    *
    *	Realtor constructor.
    *
    */
	public Realtor(TypeUser mUserType,
				   String	mName,
                   String	mAddress,
                   String	mCityStateZip,
                   String	mLoginName,
                   String	mPin, 
                   String	mEmail, 
                   String	mCredotCardCo, 
                   String	mCreditCardNumber,
                   String	mAgency) {
                   
        super(mUserType, mName, mAddress, mCityStateZip, mLoginName, 
				mPin, mEmail, mCredotCardCo, mCreditCardNumber);
        
        pAgency     = mAgency;
        pProperties = new ArrayList();		// init list
    }

    
	/**																		   |
    * 
    *	Returns an iterator to the pProperties list
	*
    *	@return	Iterator		Iterator to pProperties.
    */
    public Iterator list() {
        return  pProperties.iterator();
    }
    

	/**																		   |
    * 
    *	Adds the Property instance mProperty to the pProperties list,
    *	 if it is not already.
	*
    *	@param	Property		Property instance to be added to pUsers list.
    */
    public void add(Object mObject) throws ErrDistress {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null) {
        
            System.out.println("GOT HERE A \n");
            // if a Multifamily, ensure that it has at least two UFUnits.
            Property mProperty = (Property) mObject;
            
            if (mProperty.getPropertyType() == TypeProperty.MULTIFAMILY ) {
                MultiFamily mMFamily = (MultiFamily) mObject;
                System.out.println("GOT HERE B \n");
                if ( mMFamily.getNumberOfUnits() < 2)
                    throw new ErrDistress("CAN'T ADD ITEM. MultiFamily \"" + 
                                ((Property) mObject).getPropertyID() + 
                                "\" has fewer than 2 MFUnits");
            }
            
            pProperties.add(mObject);
        } else
            throw new ErrDistress("CAN'T ADD ITEM. Property \"" + 
								  ((Property) mObject).getPropertyID() +
								  "\" is already in " + this.getName() +
								  "'s User List.");
    }
    
    
	/**																		   |
    * 
    *	Removes the Property instance mProperty from the pProperties list,
    *	 if found.
	*
    *	@param	Property	Property instance to be removed from pUsers list.
    */
    public void remove(Object mObject) throws ErrDistress {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null)
			throw new ErrDistress("CAN'T REMOVE ITEM. Property \"" + 
								  ((Property) mObject).getPropertyID() +
								  "\" is not in " + this.getName() +
								  "'s User List.");
        else
            mItr.remove();
    }
    
    
	/**																		   |
    * 
    *	Searches through the pProperties list looking for a Properties instance  
    *	with the same reference as mProperties.  If found, an Iterator is 
    *	returned resting on said list item (so it may be removed easily), 
    *	otherwise a null is returned.
	*
    *	@return	Iterator	points to item mUser in pUsers. if mUser can't be
    *						found then returns null
    */
	private Iterator checkFor(Object mObject) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mObject == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }
    
        
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	// avoids instanceof and allows quick dynamic referencing
	public TypeUser getUserType() {							// read only
		return TypeUser.REALTOR;
	}
	                     
	public String getAgency() {
        return pAgency;
    }
	public void setAgency(String mAgency) {
        pAgency = mAgency;
    }
    
    
	public ArrayList getProperties() {
        return pProperties;
    }
	public void setProperties(ArrayList mProperties) {
        pProperties = mProperties;
    }
}