/*******************************************************************************
 
 
 	    FILE:	SearchCriteria.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	Class to emulate enumeration for 
                enum UserType = {CONSUMER, REALTOR, LENDER};
				
ASSOCIATIONS:	Holds an array of SearchItems that specify the exact criteria
                being searched for by a Consumer User
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5;

import java.util.*;

public class SearchCriteria implements java.io.Serializable {
    
    private ArrayList fSearchItems;
	private boolean fHasPropertyType = false;
	private boolean fHasCityStateZip = false;
    
    /**																		   |
    *
    *	SearchCriteria Constructor.
    *
    */
	public SearchCriteria() {
		fSearchItems = new ArrayList();
    }
    
    
	/**																		   |
    * 
    *	Returns an iterator to the list of search items
	*
    *	@return		returns an Iterator to the list of search items
    */
	public Iterator list() {
		return  fSearchItems.iterator();
	}
	
    

	/**																		   |
    * 
    *	Following functions add SearchItem objects (as comparisons or ranges) 
    *		to SearchCriteria  
	*
    */
	public void addInteger(String mVarName, int mVal) {
		Integer nVal = new Integer(mVal);
		
		fSearchItems.add( new SearchItem(mVarName, nVal, nVal) );
	}
	public void addInteger(String mVarName, int mMin, int mMax) {
		Integer nMin = new Integer(mMin);
		Integer nMax = new Integer(mMax);
		
		fSearchItems.add( new SearchItem(mVarName, nMin, nMax) );
	}
	public void addDouble(String mVarName, double mVal) {
		Double nVal  = new Double(mVal);
		
		fSearchItems.add( new SearchItem(mVarName, nVal, nVal) );
	}
	public void addDouble(String mVarName, double mMin, double mMax) {
		Double nMin  = new Double(mMin);
		Double nMax  = new Double(mMax);
		
		fSearchItems.add( new SearchItem(mVarName, nMin, nMax) );
	}
	
    
    
	/**																		   |
    * 
    *	Ensures that the Max setting is present as an Object for comparisions
	*
    *	@param	String		Name of var being tested for
    *	@param	Object		Comparison value
    */
	public void addObject(String mVarName, Object mVal) {
		addObject(mVarName, mVal, (Object) null);
	}
    
    
    
	/**																		   |
    * 
    *	This adds the SearchItem object to the SearchCriteria's [fSearchItems]
	*	list. Assigns fHasPropertyType and fHasCityStateZip booleans when found
    *
    *	@param	String		Name of var being tested for
    *	@param	Object		Min of range
    *	@param	Object		Max of range
    */
	public void addObject(String mVarName, Object mMin, Object mMax) {
		
		if ( mVarName.equals("propertyType") )
			fHasPropertyType = true;
		
		if ( mVarName.equals("cityStateZip") )
			fHasCityStateZip = true;
		
		fSearchItems.add( new SearchItem(mVarName, mMin, mMax) );
	}
	
    
	/**																		   |
    * 
    *	returns if this SearchCriteria object has both a property type and city
    *	  the match() for Property calls this, and throws an exception if false
    *	  the match() for MortgageOffering does not require such a test.
	*
    *	@return	boolean		true if they exist
    */
	public boolean getHasPropertyTypeAndCity() {
		return (fHasPropertyType && fHasCityStateZip);
	}
	
	/* 																		   |
	*			Accessors & Mutators
	*/
	public ArrayList getSearchItems() {								// read only
		return fSearchItems;
	}
	public boolean getHasPropertyType() {							// read only
		return fHasPropertyType;
	}
	public boolean getHasCityStateZip() {							// read only
		return fHasCityStateZip;
	}
}