/*******************************************************************************
 
 
 	    FILE:	ConsumerImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	This represents someone who registers to search for a property,
                presumably to buy.  This extends the class User, and adds only
                one bit of information, the time-frame in which the Consumer
                might consider buying.
				
ASSOCIATIONS:	A Consumer can have zero or one PropertyProfile objects.
                [Note, a more liberal system might allow a user to have
                multiple names PropertyProfiles, such as "MyCambridgeDreamHome",
                so long as the user is willing to pay for them.]
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public class ConsumerImpl extends UserImpl implements Consumer, java.io.Serializable {

	private PropertyProfile	 fPropertyProfile;		// ...0 or 1
    private TypePurchasingTF fPurchasingTimeFrame;	// Values can represent 
                                                    //  "within two months", 
                                                    //  "three-six months", 
                                                    //  "just looking"
    
    /**																		   |
    *
    *	ConsumerImpl Constructor.
    *
    */

	public ConsumerImpl(TypeUser mUserType,
					String 	 mName,
                    String	 mAddress,
                    String	 mCityStateZip,
                    String	 mLoginName,
                    String	 mPin, 
                    String	 mEmail, 
                    String	 mCredotCardCo, 
                    String	 mCreditCardNumber,
                    TypePurchasingTF mPurchasingTimeFrame) throws java.rmi.RemoteException  {
                    
        super(mUserType, mName, mAddress, mCityStateZip, mLoginName, 
					mPin, mEmail, mCredotCardCo, mCreditCardNumber);
        
		fPropertyProfile	 = null;
        fPurchasingTimeFrame = mPurchasingTimeFrame;
    }
    
    
	/**																		   |
    * 
    *	Returns an iterator to the pProperties list
	*
    *	@return	Iterator		Iterator to pProperties.
    */
    public Iterator list() {
		// create a list so to pass an Iterator
		ArrayList mList = new ArrayList();
		
		if (fPropertyProfile != null) {
			mList.add(fPropertyProfile);
		}
		
        return mList.iterator();
    }
    

	/**																		   |
    * 
    *	Adds the Property instance mProperty to the pProperties list,
    *	 if it is not already.
	*
    *	@param	Property		Property instance to be added to pUsers list.
    */
    public void add(Object mObject) throws java.rmi.RemoteException {
		
        if (mObject instanceof PropertyProfile)
            fPropertyProfile = (PropertyProfile) mObject;
        else if (mObject instanceof SearchCriteria) {
            fPropertyProfile = new PropertyProfileImpl(this, (SearchCriteria) mObject);
        }
    }
    
    
	/**																		   |
    * 
    *	Removes the Property instance mProperty from the pProperties list,
    *	 if found.
	*
    *	@param	Property	Property instance to be removed from pUsers list.
    */
    public void remove(Object mObject) {
		fPropertyProfile = null;
    }
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	
	public PropertyProfile getPropertyProfile() {
		return fPropertyProfile;
	}
	public void setPropertyProfile(PropertyProfile mPropertyProfile) {
		 fPropertyProfile = mPropertyProfile;
	}
	
	public TypePurchasingTF  getPurchasingTimeFrame() {
        return fPurchasingTimeFrame;
    }
	public void setPurchasingTimeFrame(TypePurchasingTF mPurchasingTimeFrame) {
        fPurchasingTimeFrame = mPurchasingTimeFrame;
    }
}