/*******************************************************************************
 
 
 	    FILE:	DistressServer.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Assignment Five

       USAGE:	java DistressServer hostName portNumber
 
 	 PURPOSE:	Used to test drive and simulate the application.
                Objects are fabricated and manipulated within Distress, then
                their values checked using DistressWriter
				
ASSOCIATIONS:	Entry point to running and testing Distress
                Instantiates Distress and DistressWriter
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.*;
import java.util.*;


public class DistressServer {
    
    
    ///																		   |
    //
    //	DistressServer Constructor.
    //
    //
	public DistressServer() {
    }
    
    
	///																		   |
  	//
  	//	Converts a string of form "mm/dd/yy" into a Date object.
  	//	Unsucessful attempts result in null.
  	//
  	//	@param String	The date string in neeed of conversion.
  	//
  	//	@return			A Date object, or null
    //
	public static Date mmddyyyyToDate(String mDateStr) {

		Date mReturn = null;							// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				// month is 0 to 11
				Calendar mCal = new GregorianCalendar(mYear, mMonth - 1, mDay);		
				mReturn = mCal.getTime();
			}
		}
	
		return mReturn;
	}
	
	
	///																		   |
  	//
  	//	Converts a string of form "yyyy" into a Date object.
  	//
  	//	@param String	The date string in neeed of conversion.
  	//
  	//	@return			A Date object, or null
    //
	public static Date yyyyToDate(String mDateStr) {
		int mYear  = stringToInt(mDateStr);
		Calendar mCal = new GregorianCalendar(mYear, 1, 1);	
		return mCal.getTime();
	}
	
	
	///																		   |
    //
    //	Convert a string to an int.
  	//
  	//	@param	String		String to convert.
  	//	@return				int representation of given string.
    //
	private static int stringToInt(String mString) {
		return Integer.parseInt(mString.trim());
	}
    

	///																		   |
    //
    //	Creates a set of objects matching the three scenarios from Assignment 2
    //
    //  
    public static void fabricateObjects(Distress mDistress) 
                            throws java.rmi.RemoteException {
		
        User mUser;
        Property mProperty;
        PropertyProfile mPProfile;
        
        // fabricate Scenario 1.
		try {
        
			// Snopes registers as a Realtor
            mUser = mDistress.register (TypeUser.REALTOR,
												"J. Snopes",
												"12345 Some Street",
												"Cambridge, MA 02138",
												"jsnopes",
												"themansion",
												"jsnopes@faulkner.net",
												"Visa",
												"2345234987634",
												"Yoknopatawpha Realty");
        
            // make Snopes's SingleFamily property
            // note I have set mBaths to 1.5.  mBaths was not specified on the Instance Diag.
            mProperty = new SingleFamilyImpl(mUser,
                                                    "A 2341 Jason St",
                                                    "Arlington, MA 02474",
                                                    6,
                                                    1.5,
                                                    2,
                                                    yyyyToDate("1945"),
                                                    TypeParking.OFFSTREET,
                                                    500000.00,  
                                                    1500,
                                                    "Hardwood floors, finished basement",
                                                    "A gem in a quiet neighborhood.  Buy it now!",
                                                    1,
                                                    5445);
 
        
            // Snopes adds his SingleFamily property
            try {
                mUser.add(mProperty);
            } catch (ErrDistress e) { System.out.println(e.getMessage()); }
             
		} catch (ErrRegistration e) {
			System.out.println(e.getMessage());
		} catch (ErrDistress e) {
            System.out.println(e.getMessage());
        }
        
        
        // fabricate Scenario 3.
        
		try {
            
			// Ma and Pa Kettle registers as a cunsumer
			mUser = mDistress.register(TypeUser.CONSUMER,
										   "Ma & Pa Kettle",
										   "1400 Settle Street",
										   "Cambridge, MA 02138",
										   "ma_pa_kettle",
										   "ma_and_pa",
										   "ma_pa@hotmail.com",
										   "Visa",
										   "345789897631",
										   TypePurchasingTF.WITHINTWOMONTHS);
			
		} catch (ErrRegistration e) {
			System.out.println(e.getMessage());
		} catch (ErrDistress e) {
            System.out.println(e.getMessage());
        }
    }
    
    
    ///																		   |
    // 
    //	Read args passed to main(), and trigger Server to be opened.
    //
    //
    public static void main(String[] argv) {
    
        System.out.println("---------------------------+");
        System.out.println("            START SERVER   |");
        System.out.println("---------------------------+");
        
        
        // defaults
        String   mHostName   = "localhost";
        int      mPort       = 1099;
        String   mObjectName = "DISTRESSService";
        Registry mRegistry   = null;
        
        if(argv.length != 2) {
            System.out.println("Usage: java DistressServer hostname portNumber");
            System.out.println("Using defaults, hostName = \"localhost\"  portNumber = 1099");
        } else {
            
            // get host name
            mHostName = argv[0];
            
            // get the port
            try {
                mPort = Integer.parseInt(argv[1]);
            } catch (NumberFormatException e) {
                System.out.println("Usage: java DistressServer hostname portNumber");
                System.out.println("Could not read port number.");
                System.exit(1);
            }
        }
        
        try {
            //locate the registery
            mRegistry = LocateRegistry.getRegistry(mHostName, mPort);
            if(mRegistry == null) {
                System.out.println("getRegistry returned null");
                System.exit(1);
            }
        } catch (java.rmi.RemoteException e) {
            System.out.println("Remote exception trying to locate registry");
            System.out.println("ERR:" + e.getMessage());
        }
        
        Distress mObjDistress = null;
        
        // Create and install a security manager
        System.setSecurityManager(new RMISecurityManager());

        try {
        
            // Create instance of remote object
            mObjDistress = DistressImpl.getDistress();
            fabricateObjects(mObjDistress);
            
            // Register distress remote object
            // port defaults to 1099 if unspecified
            System.out.println("Registering Distress object on host " + mHostName + 
                               ", port " + mPort + 
                               ", under name " + mObjectName);
            mRegistry.rebind(mObjectName, mObjDistress);
            // Naming.rebind("//localhost/DISTRESSService", mObjDistress);
            System.out.println("DISTRESSService bound in registry");
            
            DistressWriter mObjWriter = new DistressWriter(mObjDistress);
            mObjWriter.dumpAll();
        
        } catch (java.rmi.RemoteException e) {
            System.out.println("ERR_REMOTE: " + e.getMessage());
            e.printStackTrace();
        } catch (Exception e) { 
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
    }
}
