/*******************************************************************************
 
 
 	    FILE:	RealtorImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	This represents an individual realtor who has properties for
                which they represent the seller.  Class Realtor extends class
                User, and adds the name of the agency, if any.
				
ASSOCIATIONS:	A Realtor owns a collection of PropertyListings.
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public class RealtorImpl extends UserImpl implements Realtor, java.io.Serializable {

    private String	  fAgency;				// The agency for which the Realtor 
                                            //  works.  This can be null
    
    private ArrayList fProperties;			// list of Properties listed 
    										//  by this Realtor
    
    
    /**																		   |
    *
    *	RealtorImpl constructor.
    *
    */
	public RealtorImpl(TypeUser mUserType,
				   String	mName,
                   String	mAddress,
                   String	mCityStateZip,
                   String	mLoginName,
                   String	mPin, 
                   String	mEmail, 
                   String	mCredotCardCo, 
                   String	mCreditCardNumber,
                   String	mAgency) throws java.rmi.RemoteException {
                   
        super(mUserType, mName, mAddress, mCityStateZip, mLoginName, 
				mPin, mEmail, mCredotCardCo, mCreditCardNumber);
        
        fAgency     = mAgency;
        fProperties = new ArrayList();		// init list
    }

    
	/**																		   |
    * 
    *	Returns an iterator to the fProperties list
	*
    *	@return	Iterator		Iterator to fProperties.
    */
    public Iterator list() {
        return  fProperties.iterator();
    }
    
// !!!! java.rmi.RemoteException
	/**																		   |
    * 
    *	Adds the Property instance mProperty to the fProperties list,
    *	 if it is not already.
	*
    *	@param	Property		Property instance to be added to pUsers list.
    */
    public void add(Object mObject) throws ErrDistress, java.rmi.RemoteException {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null) {
            fProperties.add(mObject);
            
        } else {
        
            try {
                throw new ErrDistress("CAN'T ADD ITEM. Property \"" + 
                                    ((Property) mObject).getPropertyID() +
                                    "\" is already in " + this.getName() +
                                    "'s User List.");
            } catch (java.rmi.RemoteException e) {
                System.out.println("ERR_REMOTE: " + e.getMessage());
                e.printStackTrace();
            }
        }
    }
    
// !!! java.rmi.RemoteException
	/**																		   |
    * 
    *	Removes the Property instance mProperty from the fProperties list,
    *	 if found.
	*
    *	@param	Property	Property instance to be removed from pUsers list.
    */
    public void remove(Object mObject) throws ErrDistress, java.rmi.RemoteException {
		
        Iterator mItr = checkFor(mObject);
        
        if (mItr == null) {
            try {
                throw new ErrDistress("CAN'T REMOVE ITEM. Property \"" + 
                                    ((Property) mObject).getPropertyID() +
                                    "\" is not in " + this.getName() +
                                    "'s User List.");
            } catch (java.rmi.RemoteException e) {
                System.out.println("ERR_REMOTE: " + e.getMessage());
                e.printStackTrace();
            }
            
        } else
            mItr.remove();
    }
    
    
	/**																		   |
    * 
    *	Searches through the fProperties list looking for a Properties instance  
    *	with the same reference as mProperties.  If found, an Iterator is 
    *	returned resting on said list item (so it may be removed easily), 
    *	otherwise a null is returned.
	*
    *	@return	Iterator	points to item mUser in pUsers. if mUser can't be
    *						found then returns null
    */
	private Iterator checkFor(Object mObject) {
    
        boolean mFoundIt = false;
        Iterator mItr = list();
		while(mItr.hasNext() && ! mFoundIt)
            if (mObject == mItr.next())
                mFoundIt = true;
                
        if (! mFoundIt)
            mItr = null;
        
        return mItr;
    }
    
        
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	// avoids instanceof and allows quick dynamic referencing
	public TypeUser getUserType() {							// read only
		return TypeUser.REALTOR;
	}
	                     
	public String getAgency() {
        return fAgency;
    }
	public void setAgency(String mAgency) {
        fAgency = mAgency;
    }
    
    
	public ArrayList getProperties() {
        return fProperties;
    }
	public void setProperties(ArrayList mProperties) {
        fProperties = mProperties;
    }
}