/*******************************************************************************
 
 
 	    FILE:	ConsumerScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	ConsumerScreen.  The infoPanel has a welcome message for the
                consumer who just logged in.  The workingPanel shows a list of
                search attributes, and allows the user to fill out some criteria
                for searching.  When the user clicks on the "Search" button in
                the commandPanel, the next screen is the SearchResultsScreen.
                Clicking on "Close" logs the user out, closes the ConsumerScreen,
                and returns to the MainDistressScreen
				
ASSOCIATIONS:	Opened from MainDistressScreen.
				"Search" btn leads to SearchResultsScreen
				"Close" btn returns to MainDistressScreen
				Is a child of DistressScreen
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class ConsumerScreen extends DistressScreen {
    
    public static ConsumerScreen thisWindow = null;
    public static Consumer		 user = null;
    
    private JTextField cityFld;
    private JTextField stateFld;
    private JTextField zipFld;
    private JTextField searchLowFld;
	private JTextField searchHighFld;

    private JComboBox  miscCriteriaMenu;
    private JLabel	   miscCriteriaMenuLow;
        
    private JButton searchBtn;
    private JButton closeBtn;


	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow();     
	}
    
    
    public ConsumerScreen() {
        super("Search Criteria");
    }
    
	/**																		   |
    *	Can be called if user has already been set
    */
    public static void openWindow() {
        openWindow(user);
    }
    
	/**																		   |
    * 
    *	Static method that goes about opening this window and adding 
    *	a window listener to conclude Distress if this window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    *	
    *	@param	Consumer		The Consumer object being viewed
    */
    public static void openWindow(Consumer mUser) {
    
        user = mUser;
        
        // instance of this window
        thisWindow = new ConsumerScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        thisWindow.setSize(400, 280);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
    }
    
    
   	/**	
    *
    *	static method to hide this window and dispose of it.
    *
    */
    private static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }

				
	/**																		   |
    *
    *	Sets up the information panel (template pattern)
	*   The infoPanel has a welcome message for the
	*   consumer who just logged in.
    *
    *	@return	JPanel		JPanel using a BorderLayout
    */
    public JPanel setUpInfoPanel() {
    
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new BorderLayout());
        
        // load vars from Distress (if loaded)
        String mUserName = "";
        if (user != null) {
            try {
                mUserName = user.getName();
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
        
        // setup welcome Consumer screen
        mReturn.add(new JLabel("Welcome "+ mUserName + " [ Consumer ]",
                            JLabel.CENTER), BorderLayout.NORTH);
        mReturn.add(new JLabel("Set your search criteria and press 'Search'",
                            JLabel.CENTER),BorderLayout.CENTER);
        mReturn.add(new JLabel("Note: You MUST specifiy a City, State, and Zip",
                            JLabel.CENTER),BorderLayout.SOUTH);
        
        return mReturn;
    }


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
	*   The workingPanel shows a list of search attributes,
	*   and allows the user to fill out some criteria for searching.
    *
    *	@return	JPanel		JPanel using a GridBagLayout
    */
    public JPanel setUpWorkingPanel() {
    
        GridBagLayout gbLayout = new GridBagLayout();
        JPanel mReturn = new JPanel();
        mReturn.setLayout(gbLayout);
        
        cityFld        = new JTextField("Arlington");
        stateFld       = new JTextField("MA");
        zipFld         = new JTextField("02474");
        searchLowFld   = new JTextField();
        searchHighFld  = new JTextField();
        
        // create parkingTypeMenu popup menu
        String [] criteriaList = {  "< criteria not set >", 
                                    "Listing Price (highest)",
                                    "Rooms (highest)",
                                    "Baths (highest)",
                                    "Parking Spaces (highest)",
                                    "Living Space (highest)",
                                    "Number of Floors (highest)",
                                    "Lot Size (highest)" };
                                    
        miscCriteriaMenu = new JComboBox( criteriaList );
        miscCriteriaMenu.setSelectedIndex(1);  // set to Listing Price
        miscCriteriaMenuLow = new JLabel( "   Listing Price (lowest)" );
        
        
        String[] mLabels = { "City:",
                             "State:",
                             "Zip Code:",
                             "Criteria (highest):",
                             "Criteria (lowest):" };
        
        JTextField[] mFields = { cityFld,
                                stateFld, 
                                  zipFld, 
                           searchHighFld, 
                            searchLowFld };
        
        
        int row = 0;
        addComponent(mReturn, new Label(" "), gbLayout, 0, row, 2, 1);	    // a 'spacer' row

        int mItemCount = mLabels.length;
        for (row = 1; row <= mItemCount; row++) {

            addComponent(mReturn, new Label(" "), gbLayout, 0, row, 1, 1);  // a spacer cell
            
            if (mLabels[row-1].equals("Criteria (highest):"))
                addComponent(mReturn, miscCriteriaMenu, gbLayout, 1, row, 2, 1);
            else if (mLabels[row-1].equals("Criteria (lowest):"))
                addComponent(mReturn, miscCriteriaMenuLow, gbLayout, 1, row, 2, 1);
            else
                addComponent(mReturn, new JLabel(mLabels[row-1]), gbLayout, 1, row, 2, 1);
            
            addComponent(mReturn, mFields[row-1], gbLayout, 3, row, 3, 1);
        }
                        
        addComponent(mReturn, new JLabel(" "), gbLayout, 6, row-1, 1, 1);   // a spacer cell
        addComponent(mReturn, new Label(" "), gbLayout, 0, row, 3, 1);		// a spacer cell

        // KeyListener class that enables or disables Login button
        KeyListener mKeyListener = new KeyListener() {
            // must deff keyPressed & keyReleased, though not used
            public void keyPressed(KeyEvent e) {}
            public void keyTyped(KeyEvent e) {}
            public void keyReleased(KeyEvent e) {
            
                // check if low price is empty and high price not empty
                if (! searchHighFld.getText().equals("") &&
                       searchLowFld.getText().equals(""))
                    searchLowFld.setText("0");	// put value into lowprice fld
                    
                if  (cityFld.getText().equals("") ||
                    stateFld.getText().equals("") ||
                      zipFld.getText().equals(""))
                    searchBtn.setEnabled(false);
                else
                    searchBtn.setEnabled(true);
            }
        };
        
        // Actionlistener for miscCriteriaMenu to change miscCriteriaMenuLow text
        miscCriteriaMenu.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                String [] criteriaList = {  "< criteria not set >", 
                                    "Listing Price (lowest)",
                                    "Rooms (lowest)",
                                    "Baths (lowest)",
                                    "Parking Spaces (lowest)",
                                    "Living Space (lowest)",
                                    "Number of Floors (lowest)",
                                    "Lot Size (lowest)" };
                int mSel = miscCriteriaMenu.getSelectedIndex();
                miscCriteriaMenuLow.setText("   " + 
                        criteriaList[mSel]);
                if (mSel == 0)						// if criteria not set
                    searchBtn.setEnabled(true);		// enable searchBtn
            }
        });
        
        cityFld.addKeyListener(mKeyListener);
        stateFld.addKeyListener(mKeyListener);
        zipFld.addKeyListener(mKeyListener);
        searchLowFld.addKeyListener(mKeyListener);
        searchHighFld.addKeyListener(mKeyListener);
        searchLowFld.addKeyListener(mKeyListener);
        searchHighFld.addKeyListener(mKeyListener);
        
        return mReturn;
    }

	    
	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*   two buttons with a spacer between them.
    *   When the user clicks on the "Search" button in
	*   the commandPanel, the next screen is the SearchResultsScreen.
	*   Clicking on "Close" logs the user out, closes the ConsumerScreen,
	*   and returns to the MainDistressScreen
	*
    *	@return	JPanel		JPanel using a default layout
    */
    public JPanel setUpCommandPanel() {

        /*
        * Set up the JButton for Close
        */
        closeBtn = new JButton("Close");
        closeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeWindow();
                MainDistressScreen.openWindow();
            }
        });
        
        /*
        * Set up the JButton for Search
        */
        searchBtn  = new JButton("Search");
        searchBtn.setEnabled(false);
        searchBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            
                // create search criteria
                SearchCriteria mSearch = new SearchCriteria();
                mSearch.addObject("cityStateZip",
                                 cityFld.getText().trim() + ", " +
                                stateFld.getText().trim() + " " +
                                  zipFld.getText().trim());
                mSearch.addObject("propertyType", TypeProperty.SINGLEFAMILY);
                
                // cleanup low and high values!
                double lowDble, highDble;
                String lowStr  = searchLowFld.getText().trim();
                String highStr = searchHighFld.getText().trim();
                if (lowStr.equals(""))
                    lowStr = "0";
                if (highStr.equals(""))
                    highStr = "0";
                try {
                  	lowDble = stringToDouble(lowStr);
                } catch (java.lang.NumberFormatException err) {
                    alert("Bad low search input:  " + searchLowFld.getText());
                    lowDble = 0;			// ensure valid low
                }
                try {
                	highDble = stringToDouble(highStr);
                }  catch (java.lang.NumberFormatException err) {
                    alert("Bad high search input:  " + searchHighFld.getText());
                    highDble = 0;
                }
                int lowInt  = (int) Math.round(lowDble);		// create int versions
                int highInt = (int) Math.round(highDble);
                
                if (lowDble > highDble)
                    alert("Your low search is greater than your high");
                else {
            
                    int menuSel = miscCriteriaMenu.getSelectedIndex();
                    
                    switch (menuSel) {
                        case 1:
                            mSearch.addDouble("listingPrice", lowDble, highDble);
                            break;
                        case 2:
                            mSearch.addInteger("rooms", lowInt, highInt);
                            break;
                        case 3:
                            mSearch.addDouble("baths", lowDble, highDble);
                            break;
                        case 4:
                            mSearch.addInteger("parking", lowInt, highInt);
                        case 5:
                            mSearch.addInteger("livingSpace", lowInt, highInt);
                            break;
                        case 6:
                            mSearch.addInteger("numberOfFloors", lowInt, highInt);
                            break;
                        case 7:
                            mSearch.addInteger("lotSize", lowInt, highInt);
                            break;
                    }

                    closeWindow();
                    SearchResultsScreen.openWindow(mSearch);
                }
            }
        });
        
        JPanel mReturn = new JPanel();
        
        mReturn.add(searchBtn);
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(closeBtn); 
        
        return mReturn;
    }
}