/*******************************************************************************
 
 
 	    FILE:	DistressGUIClient.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project

 
       USAGE:	java DistressGUIClient hostName portNumber
                java cscie247.asn5.client.DistressGUIClient localhost 1099
       
 	 PURPOSE:	The GUI version of the Distress Client, looks up Distress via
				RMI and opens the MainDistressScreen, passing to it Distress
				
ASSOCIATIONS:	Entry point to running and testing Distress Client
                Uses RMI lookup to access Distress
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					mDistress = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.util.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class DistressGUIClient {

    public static void main(String argv[])	{
                
        // defaults
        String   mHostName   = "localhost";
        int      mPort       = 1099;
        String   mObjectName = "DISTRESSService";
        
        if(argv.length != 2) {
            System.out.println("Using defaults, hostname = \"localhost\"  portNumber = 1099");
        } else {
            
            // get host name
            mHostName = argv[0];
            
            // get the port
            try {
                mPort = Integer.parseInt(argv[1]);
            } catch (NumberFormatException e) {
                System.out.println("Usage: java DistressServer hostname portNumber");
                System.out.println("Could not read port number.");
                System.exit(1);
            }
        }
        
        //
        // get distress
        Distress mDistress = null;
        
        try {
            // client first gets a reference to DISTRESSService
            //    from the Server's registry.
            mDistress = (Distress) Naming.lookup("//" + mHostName + ":" + mPort + "/" + mObjectName);
            
        } catch (java.rmi.NotBoundException e) {
            System.out.println("ERR:" + e.getMessage());
            e.printStackTrace();
        } catch (java.net.MalformedURLException e) {
            System.out.println("ERR:" + e.getMessage());
            e.printStackTrace();
        } catch (java.rmi.RemoteException e) {
            System.out.println("ERR:" + e.getMessage());
            e.printStackTrace();
        }
        
        MainDistressScreen.setDistress(mDistress);
        MainDistressScreen.openWindow();        
	}

	/**																		   |
    *
    *	Methods to print a Distress object to the screen.
    *	toString() is not used here, so as to keep a separation between
    *	the concerns of the object data, and the platform specific requirements
    *	of displaying information.
    *
    *   these methods are identical to those used by DistressWriter.java
    *   EXCEPT that when comparing enumerated types the .getIndex() method
    *   is used.
    */
	public static void prtUser(User mUser) throws java.rmi.RemoteException {
    
        // detect what type of User is to be printed,
        //   and redirect appropriately
        if (mUser.getUserType().getIndex() == TypeUser.CONSUMER.getIndex())
            prtConsumer((Consumer) mUser);
        else if (mUser.getUserType().getIndex() == TypeUser.REALTOR.getIndex())
            prtRealtor((Realtor) mUser);
	}
	public static void prtConsumer(Consumer mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Consumer");
		// System.out.println("    pPropertyProfile = " + mObj.getPropertyProfile());
		System.out.println("pPurchasingTimeFrame = " + mObj.getPurchasingTimeFrame());
		prtUserAbst((User) mObj);
	}
	public static void prtRealtor(Realtor mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Realtor");
		System.out.println("             pAgency = " + mObj.getAgency());
		// System.out.println("         pProperties = " + mObj.getProperties());
		prtUserAbst((User) mObj);
	}
    public static void prtUserAbst(User mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: User << abstract >>");
		System.out.println("           pUserType = " + mObj.getUserType());
		System.out.println("               pName = " + mObj.getName());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("          pLoginName = " + mObj.getLoginName());
		System.out.println("                pPin = " + mObj.getPin());
		System.out.println("              pEmail = " + mObj.getEmail());
		System.out.println("       pCredotCardCo = " + mObj.getCredotCardCo());
		System.out.println("   pCreditCardNumber = " + mObj.getCreditCardNumber());
		System.out.println();
	}
    public static void prtProperty(Property mProperty) throws java.rmi.RemoteException {
    
        // detect what type of Property is to be printed, 
        //   and redirect appropriately
        if (mProperty.getPropertyType().getIndex() == TypeProperty.SINGLEFAMILY.getIndex() )
            prtSingleFamily((SingleFamily) mProperty);
        else if (mProperty.getPropertyType().getIndex() == TypeProperty.CONDOMINIUM.getIndex() )
            prtCondominium((Condominium) mProperty);
    }
	public static void prtPropertyAbst(Property mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Property << abstract >>");
		System.out.println("       pPropertyType = " + mObj.getPropertyType());
		// System.out.println("       pObjContainer = " + mObj.getObjContainer());
		System.out.println("         pPropertyID = " + mObj.getPropertyID());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("              pRooms = " + mObj.getRooms());
		System.out.println("              pBaths = " + mObj.getBaths());
		System.out.println("            pParking = " + mObj.getParking());
        System.out.println("          pYearBuilt = " + justYear(mObj.getYearBuilt()));
		System.out.println("        pParkingType = " + mObj.getParkingType());
		System.out.println("       pListingPrice = " + mObj.getListingPrice());
		System.out.println("        pLivingSpace = " + mObj.getLivingSpace());
		System.out.println("    pSpecialFeatures = " + mObj.getSpecialFeatures());
		System.out.println("        pDescription = " + mObj.getDescription());
		System.out.println("         pDateListed = " + dateFormat(mObj.getDateListed()));
		System.out.println("     pNumberOfFloors = " + mObj.getNumberOfFloors());
		System.out.println();
	}
	public static void prtSingleFamily(SingleFamily mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: SingleFamily");
		System.out.println("            pLotSize = " + mObj.getLotSize());
		prtPropertyAbst((Property) mObj);
		
	}
	public static void prtCondominium(Condominium mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Condominium");
		System.out.println("           pCondoFee = " + mObj.getCondoFee());
		System.out.println("      pNumberOfUnits = " + mObj.getNumberOfUnits());
		prtPropertyAbst((Property) mObj);
	}
	public static void prtPropertyProfile(PropertyProfile mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: PropertyProfile");
		// System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("             pSearch = " + mObj.getSearchCritera());
        
        // print its SearchCriteria too.
        SearchCriteria mSearchCriteria = mObj.getSearchCritera();
        prtSearchCriteria(mSearchCriteria);
        
        // ...and print SearchItems in SearchCriteria also.
        Iterator mItr = mSearchCriteria.getSearchItems().iterator();
        while(mItr.hasNext())
            prtSearchItem( (SearchItem) mItr.next() );
                
        System.out.println();
	}
	public static void prtSearchCriteria(SearchCriteria mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchCritera");
        System.out.println("        pSearchItems = " + mObj.getSearchItems());
        System.out.println("    pHasPropertyType = " + mObj.getHasPropertyType());
        System.out.println("    pHasCityStateZip = " + mObj.getHasCityStateZip());
    }
    public static void prtSearchItem(SearchItem mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchItem");
        System.out.println("            pVarName = " + mObj.getVarName());
        System.out.println("                pMin = " + mObj.getMin());
        System.out.println("                pMax = " + mObj.getMax());
    }
    
	/**
    *
    *	The toString of Date is tokenized and formatted ("fixed"), so not to print
    *	HH:MM:SS and EST values.
    *
    *	@param	Date		Date to read toString() from
    *	@return	String		the resulting string in form "Mon Jan 05, 2004"
    */
	public static String dateFormat(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
		String mReturn = mTokenDate.nextToken() + " " +				// DAY
                         mTokenDate.nextToken() + " " +				// MONTH
                         mTokenDate.nextToken() + ", ";				// DATE
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		mReturn += mTokenDate.nextToken() + " ";					// YEAR
		
		return mReturn;
	}
    
	/**
    *
    *	Extract the year from the Date toString.
    *
    *	@param	Date		Date to read toString() from
    *	@return	String		just the year
    */
	public static String justYear(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
        mTokenDate.nextToken();										// DAY		(skip)
        mTokenDate.nextToken();										// MONTH	(skip)
        mTokenDate.nextToken();										// DATE		(skip)
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		String mReturn = mTokenDate.nextToken();					// YEAR
		
		return mReturn;
	}
}