/*******************************************************************************
 
 
 	    FILE:	SearchResultsScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	SearchResultsScreen.  The infoPanel shows a summary of the 
				search criteria (non-editable).  The workingPanel allows a  
				Consumer to peruse a list of  properties that match the search
				criteria. The commandPanel has a 'View' and a 'Close' button.
				When the user selects an item in the list and clicks the 'View'
				button, the next screen is the SingleFamilyViewScreen.
				Clicking the 'Close' button closes this screen and returns the
				user to the ConsumerScreen.
				
ASSOCIATIONS:	Opened by ConsumerScreen.
				"View" btn opens SingleFamilyViewScreen
				"Close" btn opens ConsumerScreen
				Child of abstract class DistressScreen
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class SearchResultsScreen extends DistressScreen {
    
	public static SearchResultsScreen	thisWindow = null;
    public static SearchCriteria		search;
    public static ArrayList				searchResult = new ArrayList();
    
    private JList				list;
    private DefaultListModel	listModel;
    
    int	    fLastSelected = 0;
	private ArrayList			properties;
    
	private JButton viewBtn;
	private JButton closeBtn;

	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow();     
	}
    
	public SearchResultsScreen() {
        super("Search Results");
	}
    
    // returning to this window from ViewSingFamEdit
    public static void openWindow() {
        openWindow(search);
    }
    
    
	/**																		   |
    * 
    *	Static method that goes about opening the window and adding 
    *	a window listener to concluds Distress if the window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    *	
    *	@param	SingleFamily		The SearchCriteria object being viewed/edited
    */
	public static void openWindow(SearchCriteria mSearch) {
        
        search = mSearch;
        
        if (distress != null) {
            try {
                searchResult = distress.findProperty(mSearch);
            } catch (DistressException e) {
                alert(e.getMessage());
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }

        // instance of this window
        thisWindow = new SearchResultsScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        thisWindow.setSize(400, 200);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
        
        if (searchResult.size() == 0)
            info("Search yielded no results.");
    }
    
   	/**	
    *																   |
    *	static method to hide this window and dispose of it.
    *
    */
    private static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }


	/**																		   |
    *
    *	Sets up the information panel (template pattern)
    *   The infoPanel shows a summary of the search criteria (non-editable).
	*
    *	@return	JPanel		JPanel using a BorderLayout
    */
	public JPanel setUpInfoPanel() {
        
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new BorderLayout());
        
        mReturn.add(new JLabel("Select the search result you wish to view.",
                JLabel.CENTER), BorderLayout.NORTH);

        
        return mReturn;
	}


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
    *   The workingPanel allows a Consumer to peruse a list of  properties that
	*   match the search criteria.
	*
    *	@return	JPanel		JPanel using a default layout
    */
	public JPanel setUpWorkingPanel() {
        JPanel mReturn = new JPanel();
        
        try {
        
            //Create the list and put it in a scroll pane
            listModel = new DefaultListModel();
            
            Iterator mItr = searchResult.iterator();
            Property mObj;
            while(mItr.hasNext()) {
                mObj = (Property) mItr.next();
                listModel.addElement( mObj.getPropertyID() + "  " +
                                    dateFormatmmddyyyy(mObj.getDateListed()) + "  " +
                                    mObj.getAddress() + "  " + 
                                    "$" + mObj.getListingPrice());
            }
            
            list = new JList(listModel);
            list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            list.setSelectedIndex(0);
            list.addListSelectionListener(new SelListener());
            JScrollPane mListScrollPane = new JScrollPane(list);
            
            mReturn.add(mListScrollPane);
        
        } catch (java.rmi.RemoteException e) {
            alert(e.getMessage());
        }
        
        return mReturn;
	}
    
   
	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*   The commandPanel has a 'View' and a 'Close' button.
	*   When the user selects an item in the list and clicks the 'View'
	*   button, the next screen is the SingleFamilyViewScreen.
	*   Clicking the 'Close' button closes this screen and returns the
	*   user to the ConsumerScreen.
    *
    *	@return	JPanel		JPanel using a default layout
    */
	public JPanel setUpCommandPanel() {
    
        /*
        *	Set up the JButton for Close
        */
        closeBtn = new JButton("Close");
        closeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeWindow();
                ConsumerScreen.openWindow();
            }
        });
        
        /*
        *	Set up the JButton for View
        */
        viewBtn  = new JButton("View");

        if (searchResult.size() == 0)
            viewBtn.setEnabled(false);
            
        viewBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SearchResultsScreen.closeWindow();
                SingleFamilyViewScreen.openWindow( (SingleFamily) searchResult.get(fLastSelected));
            }
        });
        
        JPanel mReturn = new JPanel();
        
        mReturn.add(viewBtn);
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(closeBtn);
        
        return mReturn;
	}
    
	/**																		   |
    *
    *	SelListener listenes for changes to the text field and updates the
	*   "View" button accordingly
    */
    private class SelListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            
            if (e.getValueIsAdjusting() == false) {
                int mSelected = list.getSelectedIndex();
                
                try {

                    //Selection, update text field.
                    Property mPropertySelected = (Property) searchResult.get(mSelected);
                    
                    // only enable edit button for SINGLEFAMILY properties
                    if (mPropertySelected.getPropertyType().toString().equals("SINGLEFAMILY"))
                        viewBtn.setEnabled(true);
                    else 
                        viewBtn.setEnabled(false);
                    
                    fLastSelected = mSelected;
                } catch (java.rmi.RemoteException em) {
                    alert(em.getMessage());
                } catch (Exception em) {
                    alert(em.getMessage());
                }
            }
        }
    }
}