/*******************************************************************************
 
 
 	    FILE:	SingleFamilyEditScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	SingleFamilyEditScreen.  The infoPanel shows all the non-editable
				attributes of the property.  The workingPanel allows a realtor
				to view and edit all the editable  attributes of the property.
				Clicking on 'Save' changes the objects on the server. 
				Clicking on 'Close' closes the screen (whether or not the user has
				saved any edits)  and returns to the RealtorScreen. 
				
ASSOCIATIONS:	Opened by RealtorScreen.
				"Save" btn changes objects on the server
				"Close" btn returns to the RealtorScreen
				Child of abstract class DistressScreen
				
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class SingleFamilyEditScreen extends DistressScreen {
    
    public static SingleFamilyEditScreen thisWindow = null;
    public static SingleFamily singleFamily;

    private JComboBox parkingTypeMenu;
    
    private JTextField dateListedFld;
	private JTextField addressFld;
    private JTextField cityFld;
	private JTextField stateFld;
	private JTextField zipFld;
    private JTextField yearBuiltFld;
    private JTextField roomsFld;
    private JTextField bathsFld;
    private JTextField parkingFld;
	private JTextField listingPriceFld;
	private JTextField livingSpaceFld;
	private JTextField specialFeaturesFld;
	private JTextField descriptionFld;
	private JTextField numberOfFloorsFld;
    private JTextField lotSizeFld;

    private JButton saveBtn;
    private JButton closeBtn;

	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow(null);     
	}
    
	
    public SingleFamilyEditScreen() {
        super("Edit Single Family");
    }
    
	
	/**																		   |
    * 
    *	Static method that goes about opening this window and adding 
    *	a window listener to conclude Distress if this window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    *	
    *	@param	SingleFamily		The SingleFamily object being viewed/edited
    */
    public static void openWindow(SingleFamily mSingleFamily) {
    
        singleFamily = mSingleFamily;
        
        // instance of this window
        thisWindow = new SingleFamilyEditScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        thisWindow.setSize(400, 544);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
    }
    
    
   	/**																		   |	
    *																   
    *	static method to hide this window and dispose of it.
    *
    */
    private static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }
            

	/**																		   |
    *
    *	Sets up the information panel (template pattern)
    *
    *	@return	JPanel		JPanel using a GridLayout
    */
    public JPanel setUpInfoPanel() {
    
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new GridLayout(0, 2, 2, 2));

        // initialize values to "" (useful if Distress not loaded)
        String mPropertyType = "";
        String mPropertyID   = "";
        
        if (singleFamily != null) {
            try {
                // load properties of singleFamily into vars
                mPropertyType = singleFamily.getPropertyType().toString();
                mPropertyID   = singleFamily.getPropertyID();
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
                             
        // array of labels
        String[] mLabels = { "",
                             "Property Type:", 
                             "Property ID:",
                             "" };
        
        // array of corresponding values
        String[] mValues = { "",
                             mPropertyType, 
                             mPropertyID,
                             "" };
        
        
        // place labels and values into grid layout
        int mItemCount = mLabels.length;
        for (int i = 0; i < mItemCount; i++) {
            mReturn.add(new JLabel(LEFTTXTPADDING + mLabels[i]));
            mReturn.add(new JLabel(mValues[i]));
        }
        
        return mReturn;
    }


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
    *
    *	@return	JPanel		JPanel using a GridLayout
    */
    public JPanel setUpWorkingPanel() {
    
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new GridLayout(0, 2, 4, 4));
        
        // create edit fields
        dateListedFld		= new JTextField();
        addressFld			= new JTextField();
        cityFld				= new JTextField();
        stateFld			= new JTextField();
        zipFld				= new JTextField();
        yearBuiltFld		= new JTextField();
        roomsFld			= new JTextField();
        bathsFld			= new JTextField();
        parkingFld			= new JTextField();
        listingPriceFld		= new JTextField();
        livingSpaceFld		= new JTextField();
        specialFeaturesFld	= new JTextField();
        descriptionFld		= new JTextField();
        numberOfFloorsFld   = new JTextField();
        lotSizeFld			= new JTextField();

        // create parkingTypeMenu popup menu
        String [] ParkingTypesList = { "Off-street",
                                       "Not available",
                                       "Garage" };
        parkingTypeMenu = new JComboBox( ParkingTypesList );
        
        if (singleFamily != null) {
            try {
            
                /// Get time now for dateListed
                GregorianCalendar mCalendar = new GregorianCalendar();
        
                // set the parkingType menu
                int mParkingType = singleFamily.getParkingType().getIndex();
                parkingTypeMenu.setSelectedIndex(mParkingType);
        
                 dateListedFld.setText(dateFormatmmddyyyy(mCalendar.getTime()));
                    addressFld.setText(     singleFamily.getAddress());
                       cityFld.setText( getCity(singleFamily.getCityStateZip()));
                      stateFld.setText(getState(singleFamily.getCityStateZip()));
                        zipFld.setText(  getZip(singleFamily.getCityStateZip()));
                  yearBuiltFld.setText(dateFormatmmddyyyy(singleFamily.getYearBuilt()));
                      roomsFld.setText("" + singleFamily.getRooms());
                      bathsFld.setText("" + singleFamily.getBaths());
                    parkingFld.setText("" + singleFamily.getParking());
               listingPriceFld.setText("" + singleFamily.getListingPrice());
                livingSpaceFld.setText("" + singleFamily.getLivingSpace());
            specialFeaturesFld.setText(     singleFamily.getSpecialFeatures());
                descriptionFld.setText(     singleFamily.getDescription());
             numberOfFloorsFld.setText("" + singleFamily.getNumberOfFloors());
                    lotSizeFld.setText("" + singleFamily.getLotSize());
                    
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
                             
                             
        String[] mLabels = { "Date Listed:",
                                "Address:",
                                "City:",
                                "State:",
                                "Zip:",
                                "Year Built:",
                                "Listing Price:",
                                "Rooms:", 
                                "Baths:",
                                "Parking Type:",
                                "Parking:",
                                "Living Space:",
                                "Special Features:",
                                "Description:",
                                "Number of Floors:",
                                "Lot Size:" };

        JTextField[] mFields = { dateListedFld,
                                 addressFld,
                                 cityFld,
                                 stateFld,
                                 zipFld,
                                 yearBuiltFld,
                                 listingPriceFld,
                                 roomsFld, 
                                 bathsFld,
                                 null,
                                 parkingFld,
                                 livingSpaceFld,
                                 specialFeaturesFld,
                                 descriptionFld,
                                 numberOfFloorsFld,
                                 lotSizeFld };
        
        int mItemCount = mLabels.length;
        for (int i = 0; i < mItemCount; i++) {
            mReturn.add(new JLabel(LEFTTXTPADDING + mLabels[i]));
            
            if (mLabels[i].equals("Parking Type:"))
                mReturn.add(parkingTypeMenu);
            else
                mReturn.add(mFields[i]);
        }
        
        // KeyListener class that enables or disables Login button
        KeyListener mKeyListener = new KeyListener() {
            // must deff keyPressed & keyReleased, though not used
            public void keyPressed(KeyEvent e) {}
            public void keyTyped(KeyEvent e) {}
            public void keyReleased(KeyEvent e) {
                saveBtn.setEnabled(true);
            }
        };
        
        // Actionlistener for parkingTypeMenu popup menu to enable Login button
        parkingTypeMenu.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                saveBtn.setEnabled(true);
            }
        });
        
        dateListedFld.addKeyListener(mKeyListener);
        addressFld.addKeyListener(mKeyListener);
        cityFld.addKeyListener(mKeyListener);
        stateFld.addKeyListener(mKeyListener);
        zipFld.addKeyListener(mKeyListener);
        yearBuiltFld.addKeyListener(mKeyListener);
        listingPriceFld.addKeyListener(mKeyListener);
        roomsFld.addKeyListener(mKeyListener);
        bathsFld.addKeyListener(mKeyListener);
        parkingFld.addKeyListener(mKeyListener);
        livingSpaceFld.addKeyListener(mKeyListener);
        specialFeaturesFld.addKeyListener(mKeyListener);
        descriptionFld.addKeyListener(mKeyListener);
        numberOfFloorsFld.addKeyListener(mKeyListener);
        lotSizeFld.addKeyListener(mKeyListener);
        
        return mReturn;
    }


	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*		two buttons with a spacer between them
    *
    *	@return	JPanel		JPanel using a default layout
    */
    public JPanel setUpCommandPanel() {

        /*
        *   Set up the JButton for Close
        */
        closeBtn = new JButton("Close");
        closeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeWindow();
                RealtorScreen.openWindow();
            }
        });
        
        /*
        *   Set up the JButton for Save
        */
        saveBtn  = new JButton("Save");
        saveBtn.setEnabled(false);
        saveBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                
                try {
                
                    // try setting the yearBuilt & DateListed first, as these are 
                    // most likely to throw a DateException or NumberFormatException
                    Date mYearBuilt  = mmddyyyyToDate(yearBuiltFld.getText().trim());
                    Date mDateListed = mmddyyyyToDate(dateListedFld.getText().trim());
                    singleFamily.setYearBuilt(mYearBuilt);
                    singleFamily.setDateListed(mDateListed);
                    
                    // save values for this Property
                    singleFamily.setAddress(addressFld.getText().trim());
                    singleFamily.setCityStateZip( cityFld.getText().trim() + ", " +
                                                 stateFld.getText().trim() + " " +
                                                   zipFld.getText().trim());
                    singleFamily.setRooms(stringToInt(roomsFld.getText()));
                    singleFamily.setBaths(stringToDouble(bathsFld.getText()));
                    
                    int mParkingType = parkingTypeMenu.getSelectedIndex();
                    if (mParkingType == 0)
                        singleFamily.setParkingType(TypeParking.OFFSTREET);
                    else if (mParkingType == 1) 
                        singleFamily.setParkingType(TypeParking.NOTAVAIL);
                    else if (mParkingType == 2)
                        singleFamily.setParkingType(TypeParking.GARAGE);
                        
                    singleFamily.setParking(stringToInt(parkingFld.getText()));
                    singleFamily.setListingPrice(stringToDouble(listingPriceFld.getText()));
                    singleFamily.setLivingSpace(stringToInt(livingSpaceFld.getText()));
                    singleFamily.setSpecialFeatures(specialFeaturesFld.getText().trim());
                    singleFamily.setDescription(descriptionFld.getText().trim());
                    singleFamily.setNumberOfFloors(stringToInt(numberOfFloorsFld.getText()));
                    singleFamily.setLotSize(stringToInt(lotSizeFld.getText()));
                    
                    // Conclude save.
                    info("Save was successful. ");				// tell about save
                    SingleFamilyEditScreen.closeWindow();		// close the window
                    RealtorScreen.openWindow();					// open Realtor screen
                    
                } catch (java.rmi.RemoteException err) {
                    alert("Can't save: " + err.getMessage());
                } catch (java.lang.NumberFormatException err) {
                    alert("Can't save due to bad input:  " + err.getMessage());
                } catch (DateException err) {
                    alert("Can't save:  " + err.getMessage());
                }
            }
        });
        
        JPanel mReturn = new JPanel();
        
        mReturn.add(saveBtn);
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(closeBtn); 
        
        return mReturn;
    }
}