/*******************************************************************************
 
 
 	    FILE:	UserImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Final Project

 
 	 PURPOSE:	User is an abstract class that defines all the common attributes
                of the known system users.
				
ASSOCIATIONS:	Owned by Distress object, no direct association back to that.
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.server;

import cscie247.distress.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public abstract class UserImpl extends UnicastRemoteObject
                                        implements User, java.io.Serializable {
    
	private TypeUser	userType;
    private String		name;				// Full name of user 
                                            // (Should match name on creditcard)
    private String		address;			// Number and street of user
                                            // (Should match the billing 
                                            //  address for the credit card)
    private String		cityStateZip;		// City, State 5-digit zip
    private String		loginName;			// This is the user's id
    private String		pin;				// Any 8-character string will do
    private String		email;				// somename@someaddress
    private String		creditCardCo;		// We take Visa, MasterCard, 
                                            // and Amex
    private String		creditCardNumber;	// up to 25 digits, no hyphens


    /**																		   |
    *
    *	UserImpl Constructor.
    *
    */
	public UserImpl(TypeUser mUserType,
				String	 mName,
                String	 mAddress,
                String	 mCityStateZip,
                String	 mLoginName,
                String	 mPin, 
                String	 mEmail, 
                String	 mCredotCardCo, 
                String	 mCreditCardNumber) throws java.rmi.RemoteException {
		
		userType			= mUserType;
        name				= mName;
        address				= mAddress;
        cityStateZip		= mCityStateZip;
        loginName			= mLoginName;
        pin					= mPin;
        email				= mEmail;
        creditCardCo		= mCredotCardCo;
        creditCardNumber	= mCreditCardNumber;
    }
	
	
	/**																		   |
    * 
    *	Empty method declarations
    *	
    */
    public abstract Iterator list();
	public abstract void add(Object mObject) throws DistressException, java.rmi.RemoteException;
	public abstract void remove(Object mObject) throws DistressException, java.rmi.RemoteException;
	

	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public TypeUser getUserType() {									// read only
		return userType; 									// avoids instanceof
                                             // allows quick dynamic referencing
	}
	
	public String getName() {
        return name;
    }
	public void setName(String mName) {
        name = mName;
    }
    
	public String getAddress() {
        return address;
    }
	public void setAddress(String mAddress) {
        address = mAddress;
    }
    
	public String getCityStateZip() {
        return cityStateZip;
    }
	public void setCityStateZip(String mCityStateZip) {
        cityStateZip = mCityStateZip;
    }
    
	public String getLoginName() {
        return loginName;
    }
	public void setLoginName(String mLoginName) {
        loginName = mLoginName;
    }
    
	public String getPin() {
        return pin;
    }
	public void setPin(String mPin) {
        pin = mPin;
    }
    
	public String getEmail() {
        return email;
    }
	public void setEmail(String mEmail) {
        email = mEmail;
    }
    
	public String getCredotCardCo() {
        return creditCardCo;
    }
	public void setCredotCardCo(String mCredotCardCo) {
        creditCardCo = mCredotCardCo;
    }
    
	public String getCreditCardNumber() {
        return creditCardNumber;
    }
	public void setCreditCardNumber(String mCreditCardNumber) {
        creditCardNumber = mCreditCardNumber;
    }
}