/**
*
*	   File:	FileReader.java
* 
*	 Author:	Russell Lowke
*
*	  Class:	CSCIE-247, Assignment One, Oct 2nd 2003.
*
*	Purpose:	Handles opening, reading, and closing a file.
*  
*	  Usage:	java ScheduleDriver [fileHolidays] [fileCoursesTable]
*
*				My variable conventions:
*					p 	stands for property of object
*					m 	stands for "my" and means local to a method
*					Obj stands for an object from this package
*
*/

package cscie247.asn1;

import java.util.*;
import java.io.*;


public class FileReader {

    private	HolidayParser	pObjHolidayParser;
	private CourseParser	pObjCourseParser;
	
	private CourseData		pObjHolidayData;
	private ArrayList		pCoursesList;

	/**
    *
    *	Constructor: FileReader 
    *
    *	  The file reader reads two types of files
    *	  [fileHolidays] and [fileCoursesTable].  There is a
    *	  parser object for each of these types, HolidayParser 
	*	  and CourseParser respectively.
	*
    */
	public FileReader() {
	
		// create parser objects
        pObjHolidayParser		= new HolidayParser();
        pObjCourseParser		= new CourseParser();
		
		// initialize corresponding holiday and courses data types
		pObjHolidayData			= new CourseData("", "Holidays");
		pCoursesList			= new ArrayList();
	}
	
	
	/**
    *
    *	readHolidays
    *
    *	  Read the [fileHolidays] file and return CourseData
    *
    *
    *	  return	A CourseData type [pObjHolidayData] that contains holiday information
	*
    */
    public CourseData readHolidays(String mFile) {
        
        readFile(mFile, true);
        
        return pObjHolidayData;
    }
    
    
	/**
    *
    *	readCourses
    *
    *	  Read the [fileCoursesTable] file and return ArrayList of CourseDatas
    *
    *	  return	 An ArrayList [pCoursesList] of CourseData items for each course.
	*
    */
    public ArrayList readCourses(String mFile) {

		// make the holiday Date list accessible to the CourseParser
        pObjCourseParser.setHolidays(pObjHolidayData.getDates());
        
        readFile(mFile, false);
        
        return pCoursesList;
    }
	
	
	/**
    *
    *	readFile
    *
    *	  Open a file and read line by line, giving
    *	  each line to a parser.
	*
    *	  param		mFile					Name of file to read.
	*	  param		mUseHolidayParser		true if holiday parser is to be used
	*
    */
	public void readFile(String mFile, boolean mUseHolidayParser) {
	
        try {
			
			// open file for reading
			FileInputStream   mFIS = new FileInputStream(mFile);
			InputStreamReader mISR = new InputStreamReader(mFIS);
			BufferedReader    mBR  = new BufferedReader(mISR);
			
			String mLine = mBR.readLine();									// get first char (or EOF)
			while (mLine != null) {
			
				if (mUseHolidayParser)
					pObjHolidayParser.parseLine(mLine, pObjHolidayData);	// parse to HolidayParser
				else
					pObjCourseParser.parseLine(mLine, pCoursesList);		// parse to CourseParser
				
				mLine = mBR.readLine();										// get next line
			}
			
			mFIS.close();													// close reader
            
		} catch (FileNotFoundException e) {
			System.out.println("File " + mFile + " not found.");
		} catch (Exception e) {
			System.out.println("Unable to load data from " + mFile + ".");
		}
	}
}

