/**
*
*	   File:	HolidayParser.java
* 
*	 Author:	Russell Lowke
*
*	  Class:	CSCIE-247, Assignment One, Oct 2nd 2003.
*
*	Purpose:	Parses lines of data from FileReader into a list of "holiday" Dates.
*
*	  Notes:	If parser finds non-whitespace junk then an exception is thrown.
*
*				Tab Width = 4
*
*				My variable conventions:
*					p 	stands for property of object
*					m 	stands for "my" and means local to a method
*					Obj stands for an object from this package
*
*/

 
package cscie247.asn1;

import java.util.*;
import java.io.*;


public class HolidayParser {

	static private Calendar  pCal;											// calendar obj
	
	/**
    *
    *	Constructor: HolidayParser 
    *
    *
    *	  This constructor ensures that there is a GregorianCalendar
    *	  available for use.
	*
    */
	public HolidayParser() {
		pCal = new GregorianCalendar();										// ensure calendar has value
	}
	
	
	/**
    *
    *	parseLine		(Called by FileReader)
  	*
  	*	  Uses parseStrToDate() to convert strings recieved from FileReader [mLine] into Date
  	*	  types. If sucessful those Dates are appended to a referenced CourseData object.
	*
	*	  Note:	If parser finds non-whitespace junk then an exception is thrown.
	*
	*
	*	  param mLine			String of content passed from FileReader.
	*	  param mObjCourseData	CourseData object being append to.
	*
    */
	public void parseLine(String mLine, CourseData mObjCourseData) {
        
        try {
			mLine = mLine.trim();											// remove whitespace/junk
			
			if (mLine.length() > 0) {
			
				Date mDate = parseStrToDate(mLine);							// try to parse string into a Date
				
				if (mDate == null)
					throw new Exception(mLine);
				
				mObjCourseData.addDate(mDate);								// add holiday date
			}
        }
        
        catch(Exception e) {
        	System.out.println("BAD DATA IN HOLIDAYS: " + e.getMessage());
        }
    }
	

	/**
    *
    *	parseStrToDate
  	*
  	*	  Converts a string of form "mm/dd/yy" into a Date object.
  	*	  Unsucessful attempts result in null.
  	*
  	*
  	*	  param mDateStr	The date string in neeed of conversion.
  	*
  	*	  return			A Date object, or null
	*
    */
	public Date parseStrToDate(String mDateStr) {
	
		Date mReturn = null;												// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				pCal = new GregorianCalendar(mYear, mMonth - 1, mDay);		// month is 0 to 11
				mReturn = pCal.getTime();
			}
		}
	
		return mReturn;
	}
	
	
	/**
    *
    *	stringToInt
    *
    *	  Converts a string to an integer.
  	*
  	*
  	*	  param mString		The string to convert.
  	*
  	*	  return			An integer representation of the given string.
	*
    */
	private static int stringToInt(String mString) {
		return Integer.parseInt(mString.trim());
	}
}