/**
*
*	   File:	CourseData.java
* 
*	 Author:	Russell Lowke
*
*	  Class:	CSCIE-247, Assignment One, Oct 2nd 2003.
*
*	Purpose:	Convienent object for storing and printing course or holiday dates.
*
*				My variable conventions:
*					p 	stands for property of object
*					m 	stands for "my" and means local to a method
*					Obj stands for an object from this package
*
*/

package cscie247.asn3;

import java.util.*;
import java.io.*;


public class CourseData {
	
	private String    pCourseNumber;								// void if excluded dates
	private String    pCourseTitle;									// "HOLIDAYS" if excluded dates
	private ArrayList pDatesList;									// list of dates

	/**
    *
    *	Constructor: CourseData 
    *
    *	  Constructs a CourseData object for storing and printing course data.
    *
    *	  Note:	Holidays are also stored in a CourseData object, for ease of printing.
	*
    */
	public CourseData(String mCourseNumber, String mCourseTitle) {
		
		pCourseNumber = mCourseNumber;
		pCourseTitle  = mCourseTitle;
		pDatesList    = new ArrayList();
	}
	

	/**
    *
    *	getDates		(Called by FileReader)
    *
    *
    *	  Getter function to tell readCourses() in FileReader the
    *	  list of excluded holidays.
	*
	*
	*	  return		pDatesList.
	*
    */
	public ArrayList getDates() {
		return pDatesList;
	}
	
	
	/**
    *
    *	addDate 
    *
    *
    *	  Adds the Date [mDate] to list pDatesList.
	*
	*
	*	  param		mDate	Date to be added to this CourseData.
    */
	public void addDate(Date mDate) {
		
		if (mDate != null)												// check for valid date
			pDatesList.add(mDate);										// add
	}
	
	
	/**
    * 
    *	fixDateString
    *	
    *	  The toString of Date is tokenized and formatted ("fixed"), so not to print
    *	  HH:MM:SS and EST values
    *
    *
    *	  param		mDate	Date to read toString from
    *
    *	  return			the resulting string in form "Mon Jan 05, 2004"
    *
    */
	public String fixDateString(Date mDate) {
	
		String mReturn = "";
		
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
		
																	// tokens
		mReturn += mTokenDate.nextToken() + " ";					// DAY
		mReturn += mTokenDate.nextToken() + " ";					// MONTH
		mReturn += mTokenDate.nextToken() + ", ";					// DATE
		mTokenDate.nextToken();										// HH:MM:SS
		mTokenDate.nextToken();										// EST
		mReturn += mTokenDate.nextToken() + " ";					// YEAR
		
		return mReturn;
	}
	
	
	/**
    * 
    *	Override the toString() method of java.lang.Object for printing.
    *	
    *	  Creates a nice display print of course and its dates.
    *
    *	  Note:	As requested, an iterator is specifically used to traverse
    *			the collection of dates.
    *
    */
	public String toString() {
	
		String mReturn = "\n" + pCourseNumber + pCourseTitle + "\n\n";
		
		int itemCount = 1;
		for (Iterator e = pDatesList.iterator(); e.hasNext(); )
			mReturn += itemCount++ + ": " + fixDateString( (Date) e.next()) + "\n";
		
		return mReturn;
	}
}

