/**
*
*	   File:	ScheduleDriver.java
* 
*	 Author:	Russell Lowke
*
*	  Class:	CSCIE-247, Assignment One, Oct 2nd 2003.
*
*	Purpose:	Initiates the process of reading a holiday list [fileHolidays] and
*				a table of courses [fileCoursesTable], with the intent of generating a
*				comprehesive list of dates on which student should attend class, accounting
*				for all holiday exceptions.
*  
*	  Usage:	java ScheduleDriver [fileHolidays] [fileCoursesTable]
*
*	  Notes:	Data for holidays is stored in a CourseData objects.
*
*				Tab Width = 4
*
*			 	Variable conventions:
*					p 	stands for property of object
*					m 	stands for "my" and means local to a method
*					Obj stands for an object from this package
*
*/

package cscie247.asn3;

import java.util.*;
import java.io.*;

public class ScheduleDriver {

    // Private vars
	private static FileReader		pObjFileReader; 				// File reader Object
	private static CourseData		pObjHolidays;					// Holiday data [type CourseData]
	private static ArrayList		pCoursesList;					// Courses data [type CourseData list]
	
    
	/**
    *
    *	Constructor: ScheduleDriver 
    *
    *	  Creates a FileReader object, and passes to it the two file names
    *
	*	  Once the files have been read, resultant course objects
	*	  are then printed. 
	*
    */
	public ScheduleDriver(String mFileHolidays, String mFileCourses) {
	
		// create the file reader
        pObjFileReader = new FileReader();
		
		// pass to the file reader the two files to be read
		pObjHolidays = pObjFileReader.readHolidays(mFileHolidays);
		pCoursesList = pObjFileReader.readCourses(mFileCourses);
			
	}
	
	/**
    *
    *	Main
    *
    *	  Main expects two arguments containing file names,
    *     one for [fileHolidays] and one for [fileCoursesTable]
    *
    *	  Creates a ScheduleDriver object, and passes to it the filenames
    *
    */	
	static public void main(String args[]) {
		
		if (args.length == 2) {
			ScheduleDriver mScheduler = new ScheduleDriver( args[0], args[1] );
			
			// print any errors encountered by the FileReader
			System.out.println(pObjFileReader.getReadErrors());
			
			// print the holidays, if any				(not required, but works well)
			// System.out.println(pObjHolidays);
					
			// print the courses, if any
			Iterator e = pCoursesList.iterator();
			while(e.hasNext())
				System.out.println((CourseData) e.next());
			
		} else
			System.out.println("Error: Two parameters required. [fileHolidays] [fileCoursesTable]");
    }
}