/*******************************************************************************
 
 
 	    FILE:	CourseScheduleFactory.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
	 PURPOSE:	Abstract Factory to create instances which target various 
	 			platforms.
			
 	RELATION:	CourseScheduleFactory is a base abstract factory that is 
 				extended by SFScheduleFactory, XMLScheduleFactory, and 
 				DBMSScheduleFactory classes 
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.asn3;

import java.util.*;


public abstract class CourseScheduleFactory {
	
	public ScheduleData		pHolidayData;			// extracted Holiday data
	public ArrayList		pCoursesData;			// extracted Courses data 
	
	public String			pErrors;				// All errors encountered 
													// while reading files.
	
	
	/***************************************************************************
    *
    *	Constructor: CourseScheduleFactory
	*
	*	Initializes the pErrors string of nonfatal errors
	*
    */
	public CourseScheduleFactory() {
		pErrors	= "";								// init error string empty
	}
	
	
	/***************************************************************************
    *
    *	Empty method declarations
	*
    */
	public abstract void processSchedule() throws CriticalSDError;
	
	
	/***************************************************************************
    * 
    *	Accessor for Holidays
	*
    *	@return		ScheduleData object of Holidays.
    */
	public ScheduleData getHolidays() {
		return pHolidayData;
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for Course data list
	*
    *	@return		ArrayList of courses data.
    */
	public ArrayList getCoursesData() {
		return pCoursesData;
	}
	
	
	/***************************************************************************
    * 
    *	Accessor for pErrors String
	*
    *	@return		all read errors encountered by Factory instance.
    */
	public String getErrors() {
		return pErrors;
	}
}