/*******************************************************************************
 
 
 	    FILE:	DBMSCourseReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete CourseReader for Relational Database Management System (DBMS)
 				Specifically handles course input for the schedule in DBMS format.	
			
 	RELATION:	DBMSCourseReader is a concrete factory that extends CourseReader,
 				it is created by DBMSScheduleFactory().
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;
import java.io.*;


public class DBMSCourseReader extends CourseReader {

	private String	pServer;							// name of the server
	private String	pDatabase;							// where database lives
	private String	pLogin;								// user login
	private String	pPassword;							// user password
	
	/***************************************************************************
    *
    *	Constructor: DBMSCourseReader 
	*
	*	@param		mServer		name of the server
	*	@param		mDatabase	server where database lives
	*	@param		mLogin		user login
	*	@param		mPassword	user password
    */
	public DBMSCourseReader(String mServer, String mDatabase, String mLogin, String mPassword) {
		super();
		
		// set DBMS info
		pServer		= mServer;
		pDatabase	= mDatabase;
		pLogin		= mLogin;
		pPassword	= mPassword;
	}
	
	
	/***************************************************************************
    *
    *	readCourses
    *
	*		STUB version.  Intentioned to read DBMS server for a students 
	*		course information
    *
    *	@return		An ArrayList [pCoursesList] of ScheduleData items for 
    *				each course.
	*
	*	Note: Method declaration in CourseReader
    */
    protected ArrayList readCourses(String pServer) throws CriticalSDError {
		pCourseReadErrors += "STUB ENCOUNTERED: Read DBMS Courses\n";	// stub
		return pCoursesList;
    }

	
	/***************************************************************************
    *
    *	parseLine
  	*
  	*		STUB version. Intentioned to parse strings recieved from  
	*		readHolidays [mLine] into ScheduleData types.
 	*
	*	@param mLine	String of content passed from readHolidays.
	*
	*	Note: Method declaration in CourseReader
    */
	protected void parseLine(String mLine) throws ReadFileError {
		// stub.
    }
}