/*******************************************************************************
 
 
 	    FILE:	DBMSHolidayReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete HolidayReader for Relational Database Management 
 				System (DBMS) extends the abstract Class HolidayReader
 				Specifically handles holiday input for the schedule in 
 				DBMS format.
		
 	RELATION:	DBMSHolidayReader is a concrete factory that extends 
 				HolidayReader, which is created by DBMSScheduleFactory()
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;


public class DBMSHolidayReader extends HolidayReader {

	private String	pServer;					// name of the server
	private String	pDatabase;					// server where database lives
	private String	pLogin;						// user login
	private String	pPassword;					// user password
	
	
	/***************************************************************************
    *
    *	Constructor: DBMSHolidayReader 
	*
	*	@param		mServer		name of the server
	*	@param		mDatabase	server where database lives
	*	@param		mLogin		user login
	*	@param		mPassword	user password
    */
	public DBMSHolidayReader(String mServer, String mDatabase, 
							 String mLogin, String mPassword) {
		super();
		
		// set DBMS info
		pServer		= mServer;
		pDatabase	= mDatabase;
		pLogin		= mLogin;
		pPassword	= mPassword;
	}
	
	
	/***************************************************************************
    *
    *	readHolidays
    *
    *		STUB version.  Intentioned to access server for a students 
    		holiday information
    *
    *	@return		A ScheduleData type [pObjHolidayData] that contains 
    				holiday information
	*
	*	Note: Method declaration in HolidayReader
    */
    protected ScheduleData readHolidays(String pServer) throws CriticalSDError {
		pHolidayReadErrors += "STUB ENCOUNTERED: Read DBMS Holidays\n";	// stub
		return pObjHolidayData;
    }
	
	
	/***************************************************************************
    *
    *	parseLine
  	*
  	*		STUB version. Intentioned to convert strings recieved from  
	*		readHolidays [mLine] into Date types.
	*
	*	@param mLine	String of content passed from readHolidays.
	*
	*	Note: Method declaration in HolidayReader
    */
	protected void parseLine(String mLine) throws ReadFileError {
		// stub.
    }
}