/*******************************************************************************
 
 
 	    FILE:	Reflection.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	To simplify and handle the process of reflection

 	RELATION:	Reflection is created by ScheduleDriver.main()
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn3;

import java.util.*;
import java.lang.reflect.*;

public class Reflection {
	
	private String 	pClassName;
	private Class	pClassDef = null;			// class deffinition of pObject
	private Object	pObject	  = null;			// object instance
	
	
	/***************************************************************************
    *
    *	Constructor: Reflection
	*
	*		Deals with reflection, simplifying implimentation of creating
	*		Classes and calling Methods via reflection.
	*		This constructor actually creates an instance of the desired
	*		class, which is stored in pObject, and referenced for
	*		all furure method calls.
	*
	*	@param mClassName	the name of the class to be instantiated
	*	@param ArrayList	list of args to be sent to that Classes constructor
	*
	*	Note: All args must be Strings (for now)
    */
	public Reflection(String mClassName, ArrayList mArgsLst) throws Exception {
		
		pClassName = mClassName;				// store the class name
		
		// ArrayList must be converted into two arrays.
		Object[]	mArgsArray		= getObjectArray(mArgsLst);
		Class[]		mArgTypesArray	= getTypesArray(mArgsLst);
		
		Constructor	mConstructor	= null;		// variable to hold Constructor
		
		// get the Class called [pClassName]
		pClassDef    = Class.forName(pClassName);
		
		// get constructor for the Class			
		mConstructor = pClassDef.getConstructor(mArgTypesArray);
		
		// create instance of object
		pObject	     = mConstructor.newInstance(mArgsArray);
		
		// NOTE:		forName() throws	InstantiationException,
		//									IllegalAccessException,
		//									ClassNotFoundException
		//
		//	 	 getConstructor() throws	NoSuchMethodException,
		//									SecurityException
		//
		//			newInstance() throws	InstantiationException, 
		//									IllegalAccessException,
		//									IllegalArgumentException,
		//									InvocationTargetException
		//
		// Exceptions are delt with in main()

	}
	
	
	/***************************************************************************
    *
    *	invokeMethod
	*
	*		Invokes the Method [mMethodName] in Class [pClassName] using the
	*		args in ArrayList [mArgsLst]
	*
	*	Note:	All args in ArrayList currently must be Strings, and the Method
	*			mMethodName must accept only Strings as arguments.
	*
	*	@param mMethodName	the name of the class to be instanciated
	*	@param mArgsLst		list of args to be sent to that Classes constructor
	*
	*	Note: All args must be Strings (for now)
    */
	public void invokeMethod(String mMethodName, ArrayList mArgsLst) throws Exception, CriticalSDError {
	
		// ArrayList must be converted into two arrays.
		Object[] mArgsArray		= getObjectArray(mArgsLst);
		Class[]	 mArgTypesArray	= getTypesArray(mArgsLst);
		
		Method	 mMethod		= null;				// variable to hold Method
		
		// get the Method
		mMethod = pClassDef.getMethod(mMethodName, mArgTypesArray);	
		
		// invoke that Method
		// NOTE: Method might throw a CriticalSDError, amoungst other errors
		mMethod.invoke(pObject, mArgsArray);						
																	
			
		// NOTE:	  getMethod() throws	NoSuchMethodException,
		//									SecurityException
		//
		//				 invoke() throws	IllegalAccessException,
		//									IllegalArgumentException,
		//									InvocationTargetException
		//
		// Exceptions are delt with in main()
	}
	
	
	/***************************************************************************
    *
    *	getObjectArray
	*
	*		Both newInstance() and invoke() require an Object Array rather
	*		than an ArrayList, so getObjectArray takes an ArrayList and converts
	*		it to an Object Array.
	*
	*	@param mArgsLst		ArrayList to be converted to an Object Array
	*	@return				Object[] Array of Strings
	*
    */
	private Object[] getObjectArray(ArrayList mArgsLst) {
	
		int mNumArgs = mArgsLst.size();
		
		Object[] mReturn = new Object[ mNumArgs ];		// Array of parameters
		
		// put arguments in [mArgsLst] ArrayList into [mReturn] Object Array
		for(int i = 0; i < mNumArgs; i++)
			mReturn[i] = mArgsLst.get(i);
		
		return mReturn;
	}
	
	
	/***************************************************************************
    *
    *	getTypesArray
	*
	*		Both getConstructor() and getMethod() require a Class Array
	*		the same size as mArgsLst, so getTypesArray returns such a beast.
	*
	*	@param mArgsLst		ArrayList to be converted to an Class Array
	*	@return				Class[] Array filled with String.class
	*
	*	Note: All args in mArgsLst are currently assumed to be Strings
    */
	private Class[]	getTypesArray(ArrayList mArgsLst) {
	
		int mNumArgs = mArgsLst.size();
		
		Class[] mReturn	= new Class[ mNumArgs ];		// Array of types
		
		// set [mReturn] Object Array to String.class
		for(int i = 0; i < mNumArgs; i++)
			mReturn[i] = String.class;

		return mReturn;
	}
}