/*******************************************************************************
 
 
 	    FILE:	SFCourseReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete CourseReader for Simple File (SF)
				extends the abstract Class CourseReader
 				Specifically handles course input for the schedule in SF format.	
				
 	RELATION:	SFCourseReader is a concrete factory that extends CourseReader,
 				it is created by SFScheduleFactory()
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;
import java.io.*;


public class SFCourseReader extends CourseReader {
	
	
	/***************************************************************************
    *
    *	Constructor: SFCourseReader 
	*
    */
	public SFCourseReader() {
		super();
	}
	
	
	/***************************************************************************
    *
    *	readCourses
    *
    *		Opens the file [mFile] file and parses its content into lines that 
	*		are sent to parseLine() to be evaluated fully.  Uses parseLine() to 
	*		construct a ScheduleData object of the holidays Dates and information,
	*		 which is then returned
    *
    *	@return		ArrayList [pCoursesList] of ScheduleData items for each course.
	*
	*	Note: Method declaration in CourseReader
    */
    protected ArrayList readCourses(String mFile) throws CriticalSDError {
		
        try {
			
			// open file for reading
			FileInputStream   mFIS = new FileInputStream(mFile);
			InputStreamReader mISR = new InputStreamReader(mFIS);
			BufferedReader    mBR  = new BufferedReader(mISR);
			
			String mLine = mBR.readLine();				// get first char (or EOF)
			while (mLine != null) {
				
				try {
					parseLine(mLine);					// parse to CourseParser
				} catch(ReadFileError e) {
					pCourseReadErrors += "ERROR: IN FILE \"" + mFile + 
										"\" [COURSES]: " + e.getMessage() + "\n";
				}
					
				mLine = mBR.readLine();					// get next line
			}
			
			mFIS.close();								// close reader
            
		} catch (FileNotFoundException e) {
			throw new CriticalSDError("CRITICAL ERROR: FILE \"" + 
										mFile + "\" NOT FOUND\n");
		} catch (Exception e) {
			throw new CriticalSDError("CRITICAL ERROR: UNABLE TO LOAD FROM \"" +
										mFile + "\"\n");
		}

		return pCoursesList;
    }

	
	/***************************************************************************
    *
    *	parseLine
  	*
  	*		Strings recieved from readHolidays [mLine] will be in a table format, 
  	*		which is parsed into five forms, an 'On Days List', a course number,  
  	*		a start date, an end date, and a course title string. Once parsed,  
  	*		table data is stored as parammeters for this object and buildCourse()
  	*		is used to create a new ScheduleData objects which are appended to 
	*		reference [pCoursesList]. If parser finds non-whitespace data that 
 	*		does not have five (5) tab delimited items then a ReadFileError
 	*		exception is thrown.
	*
	*	@param mLine	String of content passed from readHolidays.
	*
	*	Note: Method declaration in CoursesReader
    */
	protected void parseLine(String mLine) throws ReadFileError {
    
		mLine = mLine.trim();							// trim whitespace/junk
	
		// check for five tab delimited values
		StringTokenizer tabItems = new StringTokenizer(mLine, "\t");
		int tokenCount = tabItems.countTokens();
		
		if (mLine.length() == 0) {			// ignore trimmed junk or "title" line
			// skip
		} else if (tokenCount == 5) {

			// get table elements
			String		mOnDaysStr	= tabItems.nextToken();	 
						pCNumStr	= tabItems.nextToken();
			String		mStartStr	= tabItems.nextToken();
			String		mEndStr		= tabItems.nextToken();
						pTitleStr	= tabItems.nextToken();
	
			pOnDaysLst	= parseDaysToList(mOnDaysStr);
			pStartDate	= parseStrToDate(mStartStr);
			pEndDate	= parseStrToDate(mEndStr);
	
			if (! pTitleStr.equals("title")) {			// ignore label text
			
				// check for valid data
				if (pStartDate == null)
					throw new ReadFileError(mLine + " [BAD START DATE]");
					
				if (pEndDate == null)
					throw new ReadFileError(mLine + " [BAD END DATE]");
				
				if (pOnDaysLst.size() < 1 || pOnDaysLst.size() > 7)
					throw new ReadFileError(mLine + " [BAD DAYS]");
				
				if (pEndDate.before(pStartDate))
					throw new ReadFileError(mLine + " [START DATE after END DATE]");
				
				pCoursesList.add(buildCourse());		// add course to list
										
			}
		} else
			throw new ReadFileError(mLine + " [TOO MANY OR TOO FEW ARGS]");
    }
}