/*******************************************************************************
 
 
 	    FILE:	SFHolidayReader.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete HolidayReader for Simple File (SF)
				extends the abstract Class HolidayReader
 				Specifically handles holiday input for the schedule in SF format.	
		
 	RELATION:	SFHolidayReader is a concrete factory that extends HolidayReader,
 				it is created by SFScheduleFactory
 	
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

import java.util.*;
import java.io.*;


public class SFHolidayReader extends HolidayReader {
	
	/***************************************************************************
    *
    *	Constructor: SFHolidayReader 
	*
    */
	public SFHolidayReader() {
		super();
	}
	
	
	/***************************************************************************
    *
    *	readHolidays
    *
    *		Opens the file [mFile] file and parses its content into lines that 
	*		are sent to parseLine() to be evaluated fully.  Uses parseLine() to 
	*		construct a ScheduleData object of the holidays Dates and information,
	*		which is then returned
    *
    *	@return		ScheduleData type [pObjHolidayData] with holiday information
	*
	*	Note: Method declaration in HolidayReader
    */
    protected ScheduleData readHolidays(String mFile) throws CriticalSDError {
        
        try {
			
			// open file for reading
			FileInputStream   mFIS = new FileInputStream(mFile);
			InputStreamReader mISR = new InputStreamReader(mFIS);
			BufferedReader    mBR  = new BufferedReader(mISR);
			
			String mLine = mBR.readLine();			// get first char (or EOF)
			while (mLine != null) {
			
				try {
					parseLine(mLine);				// parseline
				} catch(ReadFileError e) {
					pHolidayReadErrors += "ERROR: IN FILE \"" + mFile 
									+ "\" [HOLIDAYS]: " + e.getMessage() + "\n";
				}
				
				mLine = mBR.readLine();				// get next line
			}
			
			mFIS.close();							// close reader
            
		} catch (FileNotFoundException e) {
			throw new CriticalSDError("CRITICAL ERROR: FILE \"" 
									+ mFile + "\" NOT FOUND\n");
		} catch (Exception e) {
			throw new CriticalSDError("CRITICAL ERROR: UNABLE TO LOAD FROM \"" 
									+ mFile + "\"\n");
		}
	
		return pObjHolidayData;
    }
	
	
	/***************************************************************************
    *
    *	parseLine
  	*
  	*		Uses parseStrToDate() to convert strings recieved from readHolidays 
  	*		[mLine] into Date types. If sucessful those Dates are appended to 
  	*		a referenced ScheduleData object.
	*
	*	@param mLine	String of content passed from readHolidays.
	*
	*	Note: Method declaration in HolidayReader
    */
	protected void parseLine(String mLine) throws ReadFileError {
        
		mLine = mLine.trim();						// remove whitespace/junk
		
		if (mLine.length() > 0) {
		
			Date mDate = parseStrToDate(mLine);		// try to parse string into Date
			
			if (mDate == null)
				throw new ReadFileError(mLine + " [BAD DATA]");
			
			pObjHolidayData.addDate(mDate);			// add holiday date
		}
    }
}