/*******************************************************************************
 
 
 	    FILE:	SFScheduleFactory.java
  
 	  AUTHOR:	Russell Lowke
 
 	   CLASS:	CSCIE-247, Assignment Three, 

 
 	 PURPOSE:	Concrete ScheduleFactory for SimpleFile (SF)
				extends the abstract Class CourseScheduleFactory
 				Factory to create three instances of HolidayReader, CourseReader
 				and ScheduleWriter that target dealing with SF format and 
 				contains a processSchedule() method specific to handling 
 				processing SF.
			
 	RELATION:	SFScheduleFactory is a concrete factory that extends 
 				CourseScheduleFactory, it is created by ScheduleDriver.main() 
 				using refection via the Reflection class 
 				
	MODIFIED:	November 9th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

 
package cscie247.asn3;

public class SFScheduleFactory extends CourseScheduleFactory {
	
	private static SFHolidayReader   pObjHolidayReader;		// obj reference
	private static SFCourseReader    pObjCourseReader;		// obj reference
	private static SFScheduleWriter  pObjScheduleWriter;	// obj reference
	
	private String					 pHolidayFile;		// name of holiday file
	private String					 pCoursesFile;		// name of courses file
	
	
	/***************************************************************************
    *
    *	Constructor: SFScheduleFactory 
	*
	*	@param		mHolidayFile	name of holiday file to read
	*	@param		mCoursesFile	name of courses file to read
    */
	public SFScheduleFactory(String mHolidayFile, String mCoursesFile) {
	
		super();
		
		// create core objects
		pObjHolidayReader  = new SFHolidayReader();
		pObjCourseReader   = new SFCourseReader();
		pObjScheduleWriter = new SFScheduleWriter();
		
		// set file names
		pHolidayFile	   = mHolidayFile;
		pCoursesFile	   = mCoursesFile;
	}
	
	
	/***************************************************************************
    *
    *	processSchedule
	*
    * 		Actively processes the users schedule in SF format, using
	*		the SF versions of HolidayReader, CourseReader and ScheduleWriter.
    *
	*	Note: Method Declaration in CourseScheduleFactory
    */
	public void processSchedule() throws CriticalSDError {
	
		// read holiday file
		pHolidayData = pObjHolidayReader.readHolidays(pHolidayFile);
		pErrors		+= pObjHolidayReader.getHolidayReadErrors();
		
		// tell the courses reader what the holidays are
		pObjCourseReader.setHolidays(pHolidayData);
		
		// read courses file
		pCoursesData = pObjCourseReader.readCourses(pCoursesFile);
		pErrors		+= pObjCourseReader.getCourseReadErrors();
		
		// print result
		pObjScheduleWriter.writeSchedule(pHolidayData, pCoursesData, pErrors);
		
	}
}