/*******************************************************************************
 
 
 	    FILE:	DistressWriter.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Assignment Four

 
 	 PURPOSE:	A text output class that "dumps" Distress objects to the screen.
                Has a method for displaying any of the problem domain objects
				
ASSOCIATIONS:	Is instatiated and called by DistressDriver
 	
	MODIFIED:	November 30th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn4;

import java.util.*;
import cscie247.distress.*;


public class DistressWriter {

    public static Distress pObjDistress;

    /**																		   |
    *
    *	DistressWriter Constructor.
    *
    */
	public DistressWriter(Distress mObjDistress) {
        pObjDistress = mObjDistress;
    }
    
    
    
	/**																		   |
    *
    *	Methods to "dump" each Distress object to the screen.
    *
    */
    public static void dumpAll() {
    
		System.out.println("\n>>>>>>>>>>>>>>>>>> dumpAll \n");
		
		dumpUser();
        dumpPropertyProfile();
		dumpProperty();
		dumpMortgageOffering();
		
    }
	public static void dumpUser() {
    
        System.out.println("-=-=-=-= dumpUsers \n");
        
		Iterator mItr = pObjDistress.list();
		while(mItr.hasNext())
            prtUser((User) mItr.next());
    }
	public static void dumpConsumer() {
    
        System.out.println("-=-=-=-= dumpConsumers \n");
    
		Iterator mItr = pObjDistress.list();
		User	 mUser;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			if (mUser.getUserType() == TypeUser.CONSUMER)
				prtConsumer((Consumer) mUser);
		}
	}
	public static void dumpRealtor() {
    
        System.out.println("-=-=-=-= dumpRealtors \n");
        
		Iterator mItr = pObjDistress.list();
		User	 mUser;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			if (mUser.getUserType() == TypeUser.REALTOR)
				prtRealtor((Realtor) mUser);
		}
	}
	public static void dumpLender() {
    
        System.out.println("-=-=-=-= dumpLenders \n");
        
		Iterator mItr = pObjDistress.list();
		User	 mUser;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			if (mUser.getUserType() == TypeUser.LENDER)
				prtLender((Lender) mUser);
		}
	}
	public static void dumpProperty() {
    
        System.out.println("-=-=-=-= dumpProperties \n");
        
		Iterator mItr = pObjDistress.listProperties();
		while(mItr.hasNext())
			prtProperty((Property) mItr.next());
	}
	public static void dumpMultiFamily() {
    
        System.out.println("-=-=-=-= dumpMultiFamilies \n");
        
		Iterator mItr = pObjDistress.listProperties();
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.MULTIFAMILY)
				prtMultiFamily((MultiFamily) mProperty);		
		}
	}
	public static void dumpSingleFamily() {
    
        System.out.println("-=-=-=-= dumpSingleFamilies \n");
        
		Iterator mItr = pObjDistress.listProperties();
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.SINGLEFAMILY)
				prtSingleFamily((SingleFamily) mProperty);
		}
	}
	public static void dumpCondominium() {
    
        System.out.println("-=-=-=-= dumpCondominiums \n");
        
		Iterator mItr = pObjDistress.listProperties();
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.CONDOMINIUM)
				prtCondominium((Condominium) mProperty);
		}
	}
	public static void dumpMortgageOffering() {
    
        System.out.println("-=-=-=-= dumpMortgageOfferings \n");
		Iterator mItr = pObjDistress.listMortgageOfferings();
		
		while(mItr.hasNext())
			prtMortgageOffering((MortgageOffering) mItr.next());
	}
	public static void dumpPropertyProfile() {
    
        System.out.println("-=-=-=-= dumpPropertyProfiles \n");
		Iterator mItr = pObjDistress.listPropertyProfiles();
        
		while(mItr.hasNext())
			prtPropertyProfile((PropertyProfile) mItr.next());
	}
	public static void dumpMFUnit() {
    
        System.out.println("-=-=-=-= dumpMFUnits \n");
        
		Iterator mItr = pObjDistress.listProperties();
		Iterator mItr2;
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.MULTIFAMILY) {
				mItr2 = ((MultiFamily) mProperty).list();
				while(mItr.hasNext())
					prtMFUnit( (MFUnit) mItr2.next());
			}
		}
	}
    
    
	/**																		   |
    *
    *	Methods to print a Distress object to the screen.
    *	toString() is not used here, so as to keep a separation between
    *	the concerns of the object data, and the platform specific requirements
    *	of displaying information.
    */
	public static void prtUser(User mUser) {
    
        // detect what type of User is to be printed,
        //   and redirect appropriately
        if (mUser.getUserType() == TypeUser.CONSUMER)
            prtConsumer((Consumer) mUser);
        else if (mUser.getUserType() == TypeUser.REALTOR)
            prtRealtor((Realtor) mUser);
        else if (mUser.getUserType() == TypeUser.LENDER)
            prtLender((Lender) mUser);
            
	}
	public static void prtConsumer(Consumer mObj) {
		System.out.println("            Consumer : [ " + mObj + " ]");
		System.out.println("    pPropertyProfile = " + mObj.getPropertyProfile());
		System.out.println("pPurchasingTimeFrame = " + mObj.getPurchasingTimeFrame());
		prtUserAbst((User) mObj);
	}
	public static void prtRealtor(Realtor mObj) {
		System.out.println("             Realtor : [ " + mObj + " ]");
		System.out.println("             pAgency = " + mObj.getAgency());
		System.out.println("         pProperties = " + mObj.getProperties());
		prtUserAbst((User) mObj);
	}
	public static void prtLender(Lender mObj) {
		System.out.println("              Lender : [ " + mObj + " ]");
		System.out.println("        pCompanyName = " + mObj.getCompanyName());
		System.out.println("  pMortgageOfferings = " + mObj.getMortgageOfferings());
		prtUserAbst((User) mObj);
	}
    	public static void prtUserAbst(User mObj) {
		System.out.println(" << abstract >> User : [ " + mObj + " ]");
		System.out.println("           pUserType = " + mObj.getUserType());
		System.out.println("               pName = " + mObj.getName());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("          pLoginName = " + mObj.getLoginName());
		System.out.println("                pPin = " + mObj.getPin());
		System.out.println("              pEmail = " + mObj.getEmail());
		System.out.println("       pCredotCardCo = " + mObj.getCredotCardCo());
		System.out.println("   pCreditCardNumber = " + mObj.getCreditCardNumber());
		System.out.println();
	}
    public static void prtProperty(Property mProperty) {
    
        // detect what type of Property is to be printed, 
        //   and redirect appropriately
        if (mProperty.getPropertyType() == TypeProperty.MULTIFAMILY)
				prtMultiFamily((MultiFamily) mProperty);
			else if (mProperty.getPropertyType() == TypeProperty.SINGLEFAMILY)
				prtSingleFamily((SingleFamily) mProperty);
			else if (mProperty.getPropertyType() == TypeProperty.CONDOMINIUM)
				prtCondominium((Condominium) mProperty);
                
    }
	public static void prtPropertyAbst(Property mObj) {
		System.out.println("            Property : [ " + mObj + " ]");
		System.out.println("       pPropertyType = " + mObj.getPropertyType());
		System.out.println("       pObjContainer = " + mObj.getObjContainer());
		System.out.println("         pPropertyID = " + mObj.getPropertyID());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("              pRooms = " + mObj.getRooms());
		System.out.println("              pBaths = " + mObj.getBaths());
		System.out.println("            pParking = " + mObj.getParking());
        System.out.println("          pYearBuilt = " + justYear(mObj.getYearBuilt()));
		System.out.println("        pParkingType = " + mObj.getParkingType());
		System.out.println("       pListingPrice = " + mObj.getListingPrice());
		System.out.println("        pLivingSpace = " + mObj.getLivingSpace());
		System.out.println("    pSpecialFeatures = " + mObj.getSpecialFeatures());
		System.out.println("        pDescription = " + mObj.getDescription());
		System.out.println("         pDateListed = " + dateFormat(mObj.getDateListed()));
		System.out.println("     pNumberOfFloors = " + mObj.getNumberOfFloors());
		System.out.println();
	}
	public static void prtMultiFamily(MultiFamily mObj) {
		System.out.println("         MultiFamily : [ " + mObj + " ]");
		System.out.println("      pNumberOfUnits = " + mObj.getNumberOfUnits());
		System.out.println("            pMFUnits = " + mObj.getMFUnits());
        
        // ...and print MFUnits in MultiFamily also.
        Iterator mItr = mObj.getMFUnits().iterator();
        while(mItr.hasNext())
            prtMFUnit( (MFUnit) mItr.next() );
            
		prtPropertyAbst((Property) mObj);
	}
	public static void prtMFUnit(MFUnit mObj) {
		System.out.println("              MFUnit : [ " + mObj + " ]");
		System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("         pUnitNumber = " + mObj.getUnitNumber());
		System.out.println("      pNumberOfRooms = " + mObj.getNumberOfRooms());
		System.out.println("      pNumberOfBaths = " + mObj.getNumberOfBaths());
		System.out.println("        pMonthlyRent = " + mObj.getMonthlyRent());
		System.out.println("     pUnitSquareFeet = " + mObj.getUnitSquareFeet());
		System.out.println("     pNumberOfFloors = " + mObj.getNumberOfFloors());
	}
	public static void prtSingleFamily(SingleFamily mObj) {
		System.out.println("        SingleFamily : [ " + mObj + " ]");
		System.out.println("            pLotSize = " + mObj.getLotSize());
		prtPropertyAbst((Property) mObj);
		
	}
	public static void prtCondominium(Condominium mObj) {
		System.out.println("         Condominium : [ " + mObj + " ]");
		System.out.println("           pCondoFee = " + mObj.getCondoFee());
		System.out.println("      pNumberOfUnits = " + mObj.getNumberOfUnits());
		prtPropertyAbst((Property) mObj);
	}
	public static void prtMortgageOffering(MortgageOffering mObj) {
		System.out.println("    MortgageOffering : [ " + mObj + " ]");
		System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("         pOfferingID = " + mObj.getOfferingID());
		System.out.println("    pDateEstablished = " + dateFormat(mObj.getDateEstablished()));
		System.out.println("       pMortgageType = " + mObj.getMortgageType());
		System.out.println("               pTerm = " + mObj.getTerm());
		System.out.println("       pInterestRate = " + mObj.getInterestRate());
		System.out.println("        pDownPayment = " + mObj.getDownPayment());
		System.out.println("             pPoints = " + mObj.getPoints());
		System.out.println();
	}
	public static void prtPropertyProfile(PropertyProfile mObj) {
		System.out.println("     PropertyProfile : [ " + mObj + " ]");
		System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("             pSearch = " + mObj.getSearchCritera());
        
        // print its SearchCriteria too.
        SearchCriteria mSearchCriteria = mObj.getSearchCritera();
        prtSearchCriteria(mSearchCriteria);
        
        // ...and print SearchItems in SearchCriteria also.
        Iterator mItr = mSearchCriteria.getSearchItems().iterator();
        while(mItr.hasNext())
            prtSearchItem( (SearchItem) mItr.next() );
                
        System.out.println();
	}
	public static void prtSearchCriteria(SearchCriteria mObj) {
        System.out.println("       SearchCritera : [ " + mObj + " ]");
        System.out.println("        pSearchItems = " + mObj.getSearchItems());
        System.out.println("    pHasPropertyType = " + mObj.getHasPropertyType());
        System.out.println("    pHasCityStateZip = " + mObj.getHasCityStateZip());
    }
    public static void prtSearchItem(SearchItem mObj) {
        System.out.println("          SearchItem : [ " + mObj + " ]");
        System.out.println("            pVarName = " + mObj.getVarName());
        System.out.println("                pMin = " + mObj.getMin());
        System.out.println("                pMax = " + mObj.getMax());
    }
    
    
	/**
    *
    *	The toString of Date is tokenized and formatted ("fixed"), so not to print
    *	HH:MM:SS and EST values.
    *
    *	@param	Date		Date to read toString() from
    *	@return	String		the resulting string in form "Mon Jan 05, 2004"
    */
	public static String dateFormat(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
		String mReturn = mTokenDate.nextToken() + " " +				// DAY
                         mTokenDate.nextToken() + " " +				// MONTH
                         mTokenDate.nextToken() + ", ";				// DATE
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		mReturn += mTokenDate.nextToken() + " ";					// YEAR
		
		return mReturn;
	}
    
	/**
    *
    *	Extract the year from the Date toString.
    *
    *	@param	Date		Date to read toString() from
    *	@return	String		just the year
    */
	public static String justYear(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
        mTokenDate.nextToken();										// DAY		(skip)
        mTokenDate.nextToken();										// MONTH	(skip)
        mTokenDate.nextToken();										// DATE		(skip)
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		String mReturn = mTokenDate.nextToken();					// YEAR
		
		return mReturn;
	}
}


