/*******************************************************************************
 
 
 	    FILE:	Condominium.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four

 
 	 PURPOSE:	This extends the class Property.  It describes a condo.
				
ASSOCIATIONS:	None
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;

public class Condominium extends Property {

    private double		pCondoFee;		// The monthly fee expected from the 
                                        //  condo association
    private int			pNumberOfUnits;	// The number of units in the condo
                                        //  complex
    
    /**																		   |
    *
    *	Condominium Constructor.
    *
    */
	public Condominium (User		mObjContainer,
                        String		mAddress,
                        String		mCityStateZip,
                        int			mRooms,
                        double		mBaths,
                        int			mParking,
                        Date		mYearBuilt,
                        TypeParking	mParkingType,
                        double		mListingPrice,
                        int			mLivingSpace,
                        String		mSpecialFeatures,
                        String		mDescription,
                        int			mNumberOfFloors,
                        double		mCondoFee, 
                        int			mNumberOfUnits) {
        
        super(TypeProperty.CONDOMINIUM, mObjContainer, mAddress, mCityStateZip, 
			  mRooms, mBaths, mParking, mYearBuilt, mParkingType, mListingPrice,
              mLivingSpace, mSpecialFeatures, mDescription, mNumberOfFloors);
    
        pCondoFee	   = mCondoFee;
        pNumberOfUnits = mNumberOfUnits;
    }
    
    
	/**																		   |
    *
    *	This method takes the name-value SearchCriteria and determines if the
    *	Property object matches them.  It ignores any names that it cannot 
    *	understand, and throws an exception if the value associated with a name
    *	does not belong to an appropriate type.  This returns false if the 
    *	Property object doesn't match any of the values given.
    *	match() throws an exception if the property and city are not given.
    *	Note: the search criteria generally describe bounds on an attribute
    *	(e.g., one criterion might be that the minimum number of rooms is 6).
    *
    *	Note: Overrides match in Property
    */
    public boolean match(SearchCriteria mCriteria) throws ErrDistress {
		
        if ( ! mCriteria.getHasPropertyTypeAndCity() )
            throw new ErrDistress("SearchCriteria does not have both"
                                    + "property type and city");
        
        boolean mReturn = true;						// assume match
        
		Iterator	mItr = mCriteria.list();
		SearchItem	mSearchItem;
		
		while(mItr.hasNext() && mReturn == true) {
			mSearchItem = (SearchItem) mItr.next();
		
            mReturn = compare(mSearchItem.getVarName(),
                              mSearchItem.getMin(),
                              mSearchItem.getMax());
		}
        
		return mReturn;
	}
    
    
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
		
		boolean mReturn = true;						// assume match
		
		if (mVarName.equals("condoFee")) {
			if (! testDouble(pCondoFee, mMin, mMax) )
				mReturn = false;
		} else if (mVarName.equals("numberOfUnits")) {
			if (! testInteger(pNumberOfUnits, mMin, mMax) )
				mReturn = false;
		}
		
        if ( ! super.compare(mVarName, mMin, mMax) ) {
            mReturn = false;
        }
        
		return mReturn;
	}
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public double  getCondoFee() {
        return pCondoFee; }
	public void setCondoFee(double mCondoFee) {
        pCondoFee = mCondoFee;
    }
    
	public int  getNumberOfUnits() {
        return pNumberOfUnits;
    }
	public void setNumberOfUnits(int mNumberOfUnits) {
        pNumberOfUnits = mNumberOfUnits;
    }
}