/*******************************************************************************
 
 
 	    FILE:	MortgageOffering.java
  
 	  AUTHOR:	Russell Lowke
					built to a specification by Bill Robinson 
					(see "Solution for Assignment 2", November 7th 2003)
					
 	   CLASS:	CSCIE-247, Assignment Four

 
 	 PURPOSE:	This is a set of data characterizing a Mortgage offering.
				
ASSOCIATIONS:	This is owned by a Lender, and it should be possible to find
                the Lender from the MortgageOffering object.
 	
	MODIFIED:	November 28th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					p 	= "property" of object, indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;

public class MortgageOffering {

	private static int pMortgageIDCounter = 0;

	private User		pObjContainer;		// reference to Container object
	private String		pOfferingID;		// The unique identifier of this
                                            //  offering.  It is a read-only
                                            //  attribute. This has the form
                                            //  "Mtgnnnnnn", where "nnnnnn"
                                            //  is a sequence of digits
	private Date		pDateEstablished;	// The date on which this offer
                                            //  was added to Distress.
                                            //  This is set by the system
                                            //  and is read-only
	private TypeMortgage pMortgageType;		// One of "fixed-rate" or "variable"
	private int			pTerm;				// The number of months to
                                            //  make payments
	private double		pInterestRate;		// The stated annual interest rate
                                            //  (not the ARM).  This is given in
                                            //  the form nn.mm (e.g., 5.3)
	private double		pDownPayment;		// The percentage down-payment
                                            //  expected (e.g., 5, 10, 20)
	private double		pPoints;			// The upfront percentage of the 
                                            //  mortgage value due at closing
                                            //  (e.g. 1 point, 1.5 points)
        
    /**																		   |
    *
    *	MortgageOffering Constructor.
    *
    */
	public MortgageOffering(User		 mObjContainer,
                            Date   		 mDateEstablished,
                            TypeMortgage mMortgageType, 
                            int			 mTerm,
                            double		 mInterestRate,
                            double		 mDownPayment,
                            double		 mPoints) {
		
		pObjContainer	 = mObjContainer;
        pOfferingID		 = generateId();
        pDateEstablished = mDateEstablished;
        pMortgageType	 = mMortgageType;
        pTerm			 = mTerm;
        pInterestRate	 = mInterestRate;
        pDownPayment	 = mDownPayment;
        pPoints			 = mPoints;
    }


	/**																		   |
    *
    *	the following small "test" methods are called by compare() to preform
    *	actual comparisons or range test of ints, doubles, strings, Dates
    *	and enumerated types
    *
    */
	protected boolean testInteger(int mVar, Object mMin, Object mMax) {
	
		Integer mTest = new Integer(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );											  
		return ( (testMin == +1 && testMax == -1) || 
				   testMin == 0 || testMax == 0);
	}
	protected boolean testDouble(double mVar, Object mMin, Object mMax) {
	
		Double mTest = new Double(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );											  
		return ( (testMin == +1 && testMax == -1) || 
				   testMin == 0 || testMax == 0);
	}
	protected boolean testString(String mVar, Object mVal) {
		return ( mVar.compareTo(mVal) == 0 );
	}
	protected boolean testDate(Date mTestDate, Object mMin, Object mMax) {
		// typecast objects
		Date mDateStart = (Date) mMin;
		Date mDateEnd   = (Date) mMax;
		
		return (mTestDate.after(mDateStart) && mTestDate.before(mDateStart));
	}
	protected boolean testEnum(Object mVar, Object mVal) {
		return ( mVar == mVal );
	}
	
	
    /**																		   |
    * 
    *	This method takes all the name-value pairs given and determines if the
    *	MortgageOffering object matches them.  It ignores any names that it
    *	cannot understand, and throws an exception if the value associated with
    *	a name does not belong to an appropriate type.  This returns false if
    *	the MortgageOffering object doesn't match any of the values given.
    */
    public boolean match(SearchCriteria mCriteria) {
	
		boolean mReturn = true;						// assume match
		
		Iterator	mItr = mCriteria.list();
		SearchItem	mSearchItem;
		
		while(mItr.hasNext()) {
			mSearchItem = (SearchItem) mItr.next();
			
            mReturn = compare(mSearchItem.getVarName(),
                              mSearchItem.getMin(),
                              mSearchItem.getMax());
		}
			
        return mReturn;
    }
    
	
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    private boolean compare(String mVarName, Object mMin, Object mMax) {
    
        boolean mReturn = true;						// assume match
        
        if (mVarName.equals("dateEstablished")) {
            if (! testDate(pDateEstablished, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("mortgageType")) {
            if (! testEnum(pMortgageType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("term")) {
            if (! testInteger(pTerm, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("interestRate")) {
            if (! testDouble(pInterestRate, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("downPayment")) {
            if (! testDouble(pDownPayment, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("points")) {
            if (! testDouble(pPoints, mMin, mMax) )
                mReturn = false;
        }
        
        return mReturn;
    }
    
    
	/**																		   |
    * 
    *	Obtains the email of the owner of this listing.
	*
    *	@return	String		the email of the owner who listed this item
    */
    private String obtainEmail() {
        return pObjContainer.getEmail();
    }
    
    
	/**																		   |
    * 
    *	generate a unique ID for Property objects
	*
    *	@return	String		ID in string format
    */
	private static String generateId() {
		
		pMortgageIDCounter++;
		
		String mID = "" + pMortgageIDCounter;
		
		for (int i = 6 - mID.length(); i > 0; i--)
			mID = "0" + mID;
		
		mID = "Mtg" + mID;
		
		return (mID);
	}
	
	
	/* 																		   |
	*			Accessors & Mutators
	*/

	public User getContainer() {
		return pObjContainer;										// read only
                             // reference to the Lender who listed this Property
	}
	
	public String  getOfferingID() {
        return pOfferingID;											// read only
    }
    
	public Date  getDateEstablished() {
        return pDateEstablished;									// read only
    }
    
    
	public TypeMortgage  getMortgageType() {
        return pMortgageType;
    }
	public void setMortgageType(TypeMortgage mMortgageType) {
        pMortgageType = mMortgageType;
    }
    
    
	public int  getTerm() {
        return pTerm;
    }
	public void setTerm(int mTerm) {
        pTerm = mTerm;
    }
    
    
	public double  getInterestRate() {
        return pInterestRate;
    }
	public void setInterestRate(double mInterestRate) {
        pInterestRate = mInterestRate;
    }
    
    
	public double  getDownPayment() {
        return pDownPayment;
    }
	public void setDownPayment(double mDownPayment) {
        pDownPayment = mDownPayment;
    }
    
    
	public double  getPoints() {
        return pPoints;
    }
	public void setPoints(double mPoints) {
        pPoints = mPoints;
    }
}