/*******************************************************************************
 
 
 	    FILE:	DistressClient.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Assignment Five

 
       USAGE:	java DistressClient hostName portNumber
                java cscie247.asn5.client.DistressClient localhost 8888
       
 	 PURPOSE:	Used to test drive and simulate the application.
                Objects are fabricated and manipulated within Distress, then
                their values checked using DistressWriter
				
ASSOCIATIONS:	Entry point to running and testing Distress
                Instantiates Distress and DistressWriter
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					mDistress = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.client;

import cscie247.asn5.*;
import java.util.*;
import java.rmi.*;

public class DistressClient {

    public static void main(String argv[])	{
        
        System.out.println("---------------------------+");
        System.out.println("            START CLIENT   |");
        System.out.println("---------------------------+");
                
        // defaults
        String   mHostName   = "localhost";
        int      mPort       = 1099;
        String   mObjectName = "DISTRESSService";
        
        if(argv.length != 2) {
            System.out.println("Usage: java DistressClient hostName portNumber");
            System.out.println("Using defaults, hostname = \"localhost\"  portNumber = 1099");
        } else {
            
            // get host name
            mHostName = argv[0];
            
            // get the port
            try {
                mPort = Integer.parseInt(argv[1]);
            } catch (NumberFormatException e) {
                System.out.println("Usage: java DistressServer hostname portNumber");
                System.out.println("Could not read port number.");
                System.exit(1);
            }
        }
        
        User mUser = null;
        
        try {
        
            // client first gets a reference to DISTRESSService
            //    from the Server's registry.
            Distress mDistress = (Distress) Naming.lookup("//" + mHostName + ":" + mPort + "/" + mObjectName);
            
            System.out.println(">>>> Login: ma_pa_kettle  Password: ma_and_pa");
            try {
                mUser = mDistress.login("ma_pa_kettle", "ma_and_pa");
            } catch (ErrRegistration e) {
                System.out.println("ERR_REG: " + e.getMessage());
            }
            System.out.println("Login sucessful.");
            
            // construct empty search criteria
            System.out.println(">>>> Ma & Pa constructing their search criteria.");
            
			// add search criteria looking for property
            SearchCriteria mSearch = new SearchCriteria();
            mSearch.addObject("cityStateZip", "Arlington, MA 02474");
            mSearch.addObject("propertyType", TypeProperty.SINGLEFAMILY);
            mSearch.addInteger("parking", 2, 4);
            mSearch.addDouble("listingPrice", 300000, 600000);
            System.out.println("SearchCriteria created.");
            
			// add Ma & Pa search criteria to mUser (not required, but anyway...)
			try {
                System.out.println(">>>> Adding search criteria to Ma & Pa.");
				mUser.add(mSearch);
			} catch (ErrDistress e) {
                System.out.println("ERRDISTRESS:" + e.getMessage());
            } catch (Exception e) {
                System.out.println("ERR:" + e.getMessage());
            }
            System.out.println("SearchCriteria added. [not required]");
            
            Iterator mItr;
            ArrayList mSearchResult;
            
			System.out.println(">>>> Do search, should find J.Snopes property");
            mSearchResult = mDistress.findProperty(mSearch);
            System.out.println("Search done.");
            
            System.out.println(">>>> Iterate through searches found");
            mItr = mSearchResult.iterator();
            while(mItr.hasNext())
                prtProperty( (Property) mItr.next() );
                
            System.out.println(">>>> Unregister J.Snopes");
            mDistress.unregister("jsnopes");
            System.out.println("jsnopes unregistered.");
            
            System.out.println(">>>> Do search.");
            mSearchResult = mDistress.findProperty(mSearch);
            System.out.println("Search done.");
            System.out.println(">>>> Iterate through searches found");
            mItr = mSearchResult.iterator();
            while(mItr.hasNext())
                prtProperty( (Property) mItr.next() );
            
            
        } catch (Exception e) {
            System.out.println("ERR_OTHER:" + e.getMessage());
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
	}

	///																		   |
    //
    //	Methods to print a Distress object to the screen.
    //	toString() is not used here, so as to keep a separation between
    //	the concerns of the object data, and the platform specific requirements
    //	of displaying information.
    //
    // these methods are identical to those used by DistressWriter.java
    // EXCEPT that when comparing enumerated types the .getIndex() method
    // is used.
    //
	public static void prtUser(User mUser) throws java.rmi.RemoteException {
    
        // detect what type of User is to be printed,
        //   and redirect appropriately
        if (mUser.getUserType().getIndex() == TypeUser.CONSUMER.getIndex())
            prtConsumer((Consumer) mUser);
        else if (mUser.getUserType().getIndex() == TypeUser.REALTOR.getIndex())
            prtRealtor((Realtor) mUser);
	}
	public static void prtConsumer(Consumer mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Consumer");
		// System.out.println("    pPropertyProfile = " + mObj.getPropertyProfile());
		System.out.println("pPurchasingTimeFrame = " + mObj.getPurchasingTimeFrame());
		prtUserAbst((User) mObj);
	}
	public static void prtRealtor(Realtor mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Realtor");
		System.out.println("             pAgency = " + mObj.getAgency());
		// System.out.println("         pProperties = " + mObj.getProperties());
		prtUserAbst((User) mObj);
	}
    public static void prtUserAbst(User mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: User << abstract >>");
		System.out.println("           pUserType = " + mObj.getUserType());
		System.out.println("               pName = " + mObj.getName());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("          pLoginName = " + mObj.getLoginName());
		System.out.println("                pPin = " + mObj.getPin());
		System.out.println("              pEmail = " + mObj.getEmail());
		System.out.println("       pCredotCardCo = " + mObj.getCredotCardCo());
		System.out.println("   pCreditCardNumber = " + mObj.getCreditCardNumber());
		System.out.println();
	}
    public static void prtProperty(Property mProperty) throws java.rmi.RemoteException {
    
        // detect what type of Property is to be printed, 
        //   and redirect appropriately
        if (mProperty.getPropertyType().getIndex() == TypeProperty.SINGLEFAMILY.getIndex() )
            prtSingleFamily((SingleFamily) mProperty);
        else if (mProperty.getPropertyType().getIndex() == TypeProperty.CONDOMINIUM.getIndex() )
            prtCondominium((Condominium) mProperty);
    }
	public static void prtPropertyAbst(Property mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Property << abstract >>");
		System.out.println("       pPropertyType = " + mObj.getPropertyType());
		// System.out.println("       pObjContainer = " + mObj.getObjContainer());
		System.out.println("         pPropertyID = " + mObj.getPropertyID());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("              pRooms = " + mObj.getRooms());
		System.out.println("              pBaths = " + mObj.getBaths());
		System.out.println("            pParking = " + mObj.getParking());
        System.out.println("          pYearBuilt = " + justYear(mObj.getYearBuilt()));
		System.out.println("        pParkingType = " + mObj.getParkingType());
		System.out.println("       pListingPrice = " + mObj.getListingPrice());
		System.out.println("        pLivingSpace = " + mObj.getLivingSpace());
		System.out.println("    pSpecialFeatures = " + mObj.getSpecialFeatures());
		System.out.println("        pDescription = " + mObj.getDescription());
		System.out.println("         pDateListed = " + dateFormat(mObj.getDateListed()));
		System.out.println("     pNumberOfFloors = " + mObj.getNumberOfFloors());
		System.out.println();
	}
	public static void prtSingleFamily(SingleFamily mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: SingleFamily");
		System.out.println("            pLotSize = " + mObj.getLotSize());
		prtPropertyAbst((Property) mObj);
		
	}
	public static void prtCondominium(Condominium mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Condominium");
		System.out.println("           pCondoFee = " + mObj.getCondoFee());
		System.out.println("      pNumberOfUnits = " + mObj.getNumberOfUnits());
		prtPropertyAbst((Property) mObj);
	}
	public static void prtPropertyProfile(PropertyProfile mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: PropertyProfile");
		// System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("             pSearch = " + mObj.getSearchCritera());
        
        // print its SearchCriteria too.
        SearchCriteria mSearchCriteria = mObj.getSearchCritera();
        prtSearchCriteria(mSearchCriteria);
        
        // ...and print SearchItems in SearchCriteria also.
        Iterator mItr = mSearchCriteria.getSearchItems().iterator();
        while(mItr.hasNext())
            prtSearchItem( (SearchItem) mItr.next() );
                
        System.out.println();
	}
	public static void prtSearchCriteria(SearchCriteria mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchCritera");
        System.out.println("        pSearchItems = " + mObj.getSearchItems());
        System.out.println("    pHasPropertyType = " + mObj.getHasPropertyType());
        System.out.println("    pHasCityStateZip = " + mObj.getHasCityStateZip());
    }
    public static void prtSearchItem(SearchItem mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchItem");
        System.out.println("            pVarName = " + mObj.getVarName());
        System.out.println("                pMin = " + mObj.getMin());
        System.out.println("                pMax = " + mObj.getMax());
    }
    
	///
    //
    //	The toString of Date is tokenized and formatted ("fixed"), so not to print
    //	HH:MM:SS and EST values.
    //
    //	@param	Date		Date to read toString() from
    //	@return	String		the resulting string in form "Mon Jan 05, 2004"
    //
	public static String dateFormat(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
		String mReturn = mTokenDate.nextToken() + " " +				// DAY
                         mTokenDate.nextToken() + " " +				// MONTH
                         mTokenDate.nextToken() + ", ";				// DATE
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		mReturn += mTokenDate.nextToken() + " ";					// YEAR
		
		return mReturn;
	}
    
	///
    //
    //	Extract the year from the Date toString.
    //
    //	@param	Date		Date to read toString() from
    //	@return	String		just the year
    //
	public static String justYear(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
        mTokenDate.nextToken();										// DAY		(skip)
        mTokenDate.nextToken();										// MONTH	(skip)
        mTokenDate.nextToken();										// DATE		(skip)
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		String mReturn = mTokenDate.nextToken();					// YEAR
		
		return mReturn;
	}
}