/*******************************************************************************
 
 
 	    FILE:	PropertyImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	This is a compendium of information about a particular Property.
				
ASSOCIATIONS:	This is owned by a Realtor, and it should be possible to find
                the Realtor from the PropertyImpl object.
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public abstract class PropertyImpl extends UnicastRemoteObject
                                    implements Property, java.io.Serializable {
	
	private static int   fPropertyIDCounter = 0;
	
	private TypeProperty fPropertyType;
	private User		 fObjContainer;		// reference to Container object
    private String		 fPropertyID;		// Assigned by system, read only
                                            //  (called 'tracking number' in 
                                            //  the assignment statement.
    private String		 fAddress;			// number and street, unit number
                                            //  if needed
    private String		 fCityStateZip;		// Town, State 5-digit zipcode
    private int			 fRooms;			// Bedrooms, kitchen, living room,
                                            //  dining room, den, other
                                            // non-bathroom/non-closet
    private double		 fBaths;			// Have to allow 1.5 baths
    private int			 fParking;			// Number of spaces available
    private	Date		 fYearBuilt;		// Month/Day/Year
    private	TypeParking	 fParkingType;		// Off-street, not available, garage
    private double		 fListingPrice;		// Dollars and cents
    private int			 fLivingSpace;		// Integral number of square feet in
                                            //  the house
    private	String		 fSpecialFeatures;	// Wax poetic in 25 words or less
    private	String		 fDescription;		// even more lyrical in 256 chars
    private	Date		 fDateListed;		// 4-digit year
    private	int			 fNumberOfFloors;	// Number of floors in property

    /**																		   |
    *
    *	PropertyImpl Constructor.
    *
    */
	public PropertyImpl(TypeProperty mPropertyType,
					User		 mObjContainer,
                    String		 mAddress,
                    String		 mCityStateZip,
                    int			 mRooms,
                    double		 mBaths,
                    int			 mParking,
                    Date		 mYearBuilt,
                    TypeParking	 mParkingType,
                    double		 mListingPrice,
                    int			 mLivingSpace,
                    String		 mSpecialFeatures,
                    String		 mDescription,
                    int			 mNumberOfFloors) throws java.rmi.RemoteException {
    
		fPropertyType	 = mPropertyType;
		fObjContainer	 = mObjContainer;
        fPropertyID		 = generateId();
        fAddress		 = mAddress;
        fCityStateZip	 = mCityStateZip;
        fRooms			 = mRooms;
        fBaths			 = mBaths;
        fParking		 = mParking;
        fYearBuilt		 = mYearBuilt;
        fParkingType	 = mParkingType;
        fListingPrice	 = mListingPrice;
        fLivingSpace	 = mLivingSpace;
        fSpecialFeatures = mSpecialFeatures;
        fDescription	 = mDescription;
        fNumberOfFloors	 = mNumberOfFloors;
        
		/// Get when item was listed (now).
		GregorianCalendar mCalendar = new GregorianCalendar();
		fDateListed		 = mCalendar.getTime();    	// today	
    }
    
    
	/**																		   |
    * 
    *	Empty method declarations
    *	
    */
	public abstract boolean match(SearchCriteria mCriteria) throws ErrDistress;
    
    
	/**																		   |
    * 
    *	generate a unique ID for PropertyImpl objects
	*
    *	@return	String		ID in string format
    */
	private static String generateId() {
		
		fPropertyIDCounter++;
		
		String mID = "" + fPropertyIDCounter;
		
		for (int i = 6 - mID.length(); i > 0; i--)
			mID = "0" + mID;
		
		mID = "Prop" + mID;
		
		return (mID);
	}
	
    
	/**																		   |
    *
    *	the following small "test" methods are called by compare() to preform
    *	actual comparisons or range test of ints, doubles, strings, Dates
    *	and enumerated types
    *
    */
	protected boolean testInteger(int mVar, Object mMin, Object mMax) {
	
		Integer mTest = new Integer(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );	
        
        boolean mReturn = ( (testMin == +1 && testMax == -1) || 
				              testMin == 0 || testMax == 0 );
		return mReturn;
	}
	protected boolean testDouble(double mVar, Object mMin, Object mMax) {
	
		Double mTest = new Double(mVar);
		
		// test for range
		int testMin = ( mTest.compareTo(mMin) );		
		int testMax = ( mTest.compareTo(mMax) );
        
        boolean mReturn = ( (testMin == +1 && testMax == -1) || 
                              testMin == 0 || testMax == 0);
		return mReturn;
	}
	protected boolean testString(String mVar, Object mVal) {
    
        boolean mReturn = ( mVar.compareTo(mVal) == 0 );
		return mReturn;
	}
	protected boolean testDate(Date mTestDate, Object mMin, Object mMax) {
		// typecast objects
		Date mDateStart = (Date) mMin;
		Date mDateEnd   = (Date) mMax;
		
        boolean mReturn = (mTestDate.after(mDateStart) && mTestDate.before(mDateStart));
		return mReturn;
	}
	protected boolean testEnum(Object mVar, Object mVal) {
        TypeEnum mFirst  = (TypeEnum) mVar;
        TypeEnum mSecond = (TypeEnum) mVal;
        
        boolean mReturn = ( mFirst.getIndex() == mSecond.getIndex() );
		return mReturn;
	}
    
    
	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
        
        boolean mReturn = true;						// assume match
        
        if (mVarName.equals("propertyType")) {
            if (! testEnum(fPropertyType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("address")) {
            if (! testString(fAddress, mMin) )
                mReturn = false;
        } else if (mVarName.equals("cityStateZip")) {
            if (! testString(fCityStateZip, mMin) )
                mReturn = false;
        } else if (mVarName.equals("rooms")) {
            if (! testInteger(fRooms, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("baths")) {
            if (! testDouble(fBaths, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parking")) {
            if (! testInteger(fParking, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("yearBuilt")) {
            if (! testDate(fYearBuilt, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("parkingType")) {
            if (! testEnum(fParkingType, mMin) )
                mReturn = false;
        } else if (mVarName.equals("listingPrice")) {
            if (! testDouble(fListingPrice, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("livingSpace")) {
            if (! testInteger(fLivingSpace, mMin, mMax) )
                mReturn = false;
        } else if (mVarName.equals("specialFeatures")) {
            if (! testString(fSpecialFeatures, mMin) )
                mReturn = false;
        } else if (mVarName.equals("description")) {
            if (! testString(fDescription, mMin) )
                mReturn = false;
        } else if (mVarName.equals("numberOfFloors")) {
            if (! testInteger(fNumberOfFloors, mMin, mMax) )
                mReturn = false;
        }
        
        return mReturn;
    }
    
    
    
	/**																		   |
    * 
    *	Obtains the email of the owner of this listing.
	*
    *	@return	String		the email of the owner who listed this item
    */
    private String obtainEmail() throws java.rmi.RemoteException {
        return fObjContainer.getEmail();
    }
    
    
	/* 																		   |
	*			Accessors & Mutators
	*/
    
	// avoids instanceof and allows quick dynamic referencing
	public TypeProperty getPropertyType() {							// read only
		return fPropertyType;
	}
	
	public User getObjContainer() {							
		return fObjContainer;										// read only
                            // reference to the Realtor who listed this Property
	}
	
	public String getPropertyID() {
        return fPropertyID;											// read only
    }
    
	public String getAddress() {
        return fAddress;
    }
	public void setAddress(String mAddress) {
        fAddress = mAddress;
    }
    
	public String getCityStateZip() {
        return fCityStateZip;
    }
	public void setCityStateZip(String mCityStateZip) {
        fCityStateZip = mCityStateZip;
    }
    
	public int getRooms() {
        return fRooms;
    }
	public void setRooms(int mRooms) {
        fRooms = mRooms;
    }
    
	public double getBaths() {
        return fBaths;
    }
	public void setBaths(double mBaths) {
        fBaths = mBaths;
    }
    
	public int getParking() {
        return fParking;
    }
	public void setParking(int mParking) {
        fParking = mParking;
    }
    
	public Date getYearBuilt() {
        return fYearBuilt;
    }
	public void setYearBuilt(Date mYearBuilt) {
        fYearBuilt = mYearBuilt;
    }
    
	public TypeParking getParkingType() {
        return fParkingType;
    }
	public void setParkingType(TypeParking mParkingType) {
        fParkingType = mParkingType;
    }
    
	public double getListingPrice() {
        return fListingPrice;
    }
	public void setListingPrice(double mListingPrice) {
        fListingPrice = mListingPrice;
    }
    
	public int getLivingSpace() {
        return fLivingSpace;
    }
	public void setLivingSpace(int mLivingSpace) {
        fLivingSpace = mLivingSpace;
    }
    
	public String getSpecialFeatures() {
        return fSpecialFeatures;
    }
	public void setSpecialFeatures(String mSpecialFeatures) {
        fSpecialFeatures = mSpecialFeatures;
    }
    
	public String getDescription() {
        return fDescription;
    }
	public void setDescription(String mDescription) {
        fDescription = mDescription;
    }
    
	public Date getDateListed() {
        return fDateListed;
    }
	public void setDateListed(Date mDateListed) {
        fDateListed = mDateListed;
    }
    
	public int getNumberOfFloors() {
        return fNumberOfFloors;
    }
	public void setNumberOfFloors(int mNumberOfFloors) {
        fNumberOfFloors = mNumberOfFloors;
    } 
 }