/*******************************************************************************
 
 
 	    FILE:	SingleFamilyImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	This class extends Property.  It describes a single family
                home, typically with its own lot and other features.
				
ASSOCIATIONS:	None. 
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

public class SingleFamilyImpl extends PropertyImpl implements SingleFamily, java.io.Serializable {

    private	int		fLotSize;		// 1 acre = 43,560 sq ft
    
    /**																		   |
    *
    *	SingleFamilyImpl Constructor.
    *
    */
	public SingleFamilyImpl(User		mObjContainer,
                        String		mAddress,
                        String		mCityStateZip,
                        int			mRooms,
                        double		mBaths,
                        int			mParking,
                        Date		mYearBuilt,
                        TypeParking	mParkingType,
                        double		mListingPrice,
                        int			mLivingSpace,
                        String		mSpecialFeatures,
                        String		mDescription,
                        int			mNumberOfFloors,
                        int 		mLotSize) throws java.rmi.RemoteException {
        
        super(TypeProperty.SINGLEFAMILY, mObjContainer, mAddress, mCityStateZip,
			  mRooms, mBaths, mParking, mYearBuilt, mParkingType, mListingPrice,
              mLivingSpace, mSpecialFeatures, mDescription, mNumberOfFloors);
        
        fLotSize = mLotSize;
    }
    
  
	/**																		   |
    *
    *	This method takes the name-value SearchCriteria and determines if the
    *	Property object matches them.  It ignores any names that it cannot 
    *	understand, and throws an exception if the value associated with a name
    *	does not belong to an appropriate type.  This returns false if the 
    *	Property object doesn't match any of the values given.
    *	match() throws an exception if the property and city are not given.
    *	Note: the search criteria generally describe bounds on an attribute
    *	(e.g., one criterion might be that the minimum number of rooms is 6).
    *
    *	Note: Overrides match in Property
    */
    public boolean match(SearchCriteria mCriteria) throws ErrDistress {
		
        if ( ! mCriteria.getHasPropertyTypeAndCity() )
            throw new ErrDistress("SearchCriteria does not have both"
                                    + "property type and city");
                                    
        boolean mReturn = true;						// assume match
        
		Iterator	mItr = mCriteria.list();
		SearchItem	mSearchItem;
		
		while(mItr.hasNext() && mReturn == true) {
			mSearchItem = (SearchItem) mItr.next();
		
            mReturn = compare(mSearchItem.getVarName(),
                              mSearchItem.getMin(),
                              mSearchItem.getMax());
		}
        
		return mReturn;
	}
      

	/**																		   |
    * 
    *	Called by match().  takes a string and preforms the appropriate 
    *						comparison
    *	
    *	@param	String		being looked for
	*	@param	Object		Min value in range
	*	@param	Object		Max value in range
    */
    public boolean compare(String mVarName, Object mMin, Object mMax) {
		
		boolean mReturn = true;						// assume match
		
		if (mVarName.equals("lotSize")) {
			if (! testInteger(fLotSize, mMin, mMax) )
				mReturn = false;
		}
		
        if ( ! super.compare(mVarName, mMin, mMax) ) {
            mReturn = false;
        }
        
		return mReturn;
	}
	
	
	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public int  getLotSize() {
        return fLotSize;
    }
	public void setLotSize(int mLotSize) {
        fLotSize = mLotSize;
    }
}