/*******************************************************************************
 
 
 	    FILE:	UserImpl.java
  
 	  AUTHOR:	Russell Lowke
					
 	   CLASS:	CSCIE-247, Assignment Five

 
 	 PURPOSE:	User is an abstract class that defines all the common attributes
                of the known system users.
				
ASSOCIATIONS:	Owned by Distress object, no direct association back to that.
 	
	MODIFIED:	December 14th 2003
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					f 	= "field", indicates member variable/field
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.asn5.server;

import cscie247.asn5.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public abstract class UserImpl extends UnicastRemoteObject
                                        implements User, java.io.Serializable {
    
	private TypeUser	fUserType;
    private String		fName;				// Full name of user 
                                            // (Should match name on creditcard)
    private String		fAddress;			// Number and street of user
                                            // (Should match the billing 
                                            //  address for the credit card)
    private String		fCityStateZip;		// City, State 5-digit zip
    private String		fLoginName;			// This is the user's id
    private String		fPin;				// Any 8-character string will do
    private String		fEmail;				// somename@someaddress
    private String		fCredotCardCo;		// We take Visa, MasterCard, 
                                            // and Amex
    private String		fCreditCardNumber;	// up to 25 digits, no hyphens


    /**																		   |
    *
    *	UserImpl Constructor.
    *
    */
	public UserImpl(TypeUser mUserType,
				String	 mName,
                String	 mAddress,
                String	 mCityStateZip,
                String	 mLoginName,
                String	 mPin, 
                String	 mEmail, 
                String	 mCredotCardCo, 
                String	 mCreditCardNumber) throws java.rmi.RemoteException {
		
		fUserType		  = mUserType;
        fName			  = mName;
        fAddress		  = mAddress;
        fCityStateZip	  = mCityStateZip;
        fLoginName		  = mLoginName;
        fPin 			  = mPin;
        fEmail 			  = mEmail;
        fCredotCardCo 	  = mCredotCardCo;
        fCreditCardNumber = mCreditCardNumber;
    }
	
	
	/**																		   |
    * 
    *	Empty method declarations
    *	
    */
    public abstract Iterator list();
	public abstract void add(Object mObject) throws ErrDistress, java.rmi.RemoteException;
	public abstract void remove(Object mObject) throws ErrDistress, java.rmi.RemoteException;
	

	/* 																		   |
	*			Accessors & Mutators
	*/
	
	public TypeUser getUserType() {									// read only
		return fUserType; 									// avoids instanceof
                                             // allows quick dynamic referencing
	}
	
	public String getName() {
        return fName;
    }
	public void setName(String mName) {
        fName = mName;
    }
    
	public String getAddress() {
        return fAddress;
    }
	public void setAddress(String mAddress) {
        fAddress = mAddress;
    }
    
	public String getCityStateZip() {
        return fCityStateZip;
    }
	public void setCityStateZip(String mCityStateZip) {
        fCityStateZip = mCityStateZip;
    }
    
	public String getLoginName() {
        return fLoginName;
    }
	public void setLoginName(String mLoginName) {
        fLoginName = mLoginName;
    }
    
	public String getPin() {
        return fPin;
    }
	public void setPin(String mPin) {
        fPin = mPin;
    }
    
	public String getEmail() {
        return fEmail;
    }
	public void setEmail(String mEmail) {
        fEmail = mEmail;
    }
    
	public String getCredotCardCo() {
        return fCredotCardCo;
    }
	public void setCredotCardCo(String mCredotCardCo) {
        fCredotCardCo = mCredotCardCo;
    }
    
	public String getCreditCardNumber() {
        return fCreditCardNumber;
    }
	public void setCreditCardNumber(String mCreditCardNumber) {
        fCreditCardNumber = mCreditCardNumber;
    }
}