/*******************************************************************************
 
 
 	    FILE:	Distress.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project

 
 	 PURPOSE:	Distress is the focal point of interaction with external 
                objects.  It has three main functions: providing the ability for
                new users to gain access to the system, allowing existing users 
                to login to the system, and supporting ad hoc searches for 
                properties or mortgages.   There should be only one Distress 
                object instance.
				
ASSOCIATIONS:	The Distress object keeps a collection of User objects.
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress;

import java.util.*;


// remote interface must be public, otherwise client will get an error
// remote interface must extend the interface java.rmi.Remote

public interface Distress extends java.rmi.Remote {

    // each method must delare java.rmi.RemoteException in its throws clause
	public User register (TypeUser  mUserType,
						  String    mName,
						  String	mAddress,
						  String	mCityStateZip,
						  String	mLoginName,
						  String	mPin,
						  String	mEmail,
						  String	mCreditCardCo,
						  String	mCreditCardNumber,
						  Object	mObject) throws DistressException, RegistrationException, java.rmi.RemoteException;

    public void			unregister(String mLoginName) 
                            throws DistressException, RegistrationException, java.rmi.RemoteException;
    public User			login(String mLoginName, String mPassword) 
                            throws DistressException, RegistrationException, java.rmi.RemoteException;
    public ArrayList	findProperty(SearchCriteria mSearchCriteria)
                            throws DistressException, java.rmi.RemoteException;
	public User			getUser(String mLoginID)
                            throws java.rmi.RemoteException;
    public void			add(User mUser)
                            throws DistressException, RegistrationException, java.rmi.RemoteException;
    public void			remove(User mUser)
                            throws RegistrationException, java.rmi.RemoteException;
    public Iterator		list()
                            throws java.rmi.RemoteException;
    public Iterator		listProperties()
                            throws java.rmi.RemoteException;
	public Iterator		listMortgageOfferings()
                            throws java.rmi.RemoteException;
	public Iterator		listPropertyProfiles()
                            throws java.rmi.RemoteException;
	public Iterator		listHeldByUser(TypeUser mUserType)
                            throws java.rmi.RemoteException;
	public ArrayList	getUsers()
                            throws java.rmi.RemoteException;
	public void			setUsers(ArrayList mUsers)
                            throws java.rmi.RemoteException;
}
