/*******************************************************************************
 
 
 	    FILE:	DistressScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	Abstract parent class for all Distress GUI screens
				that utilizes template pattern to create consistent display and
				functionality. Contains standard methods comon to
				all Distress screens.
				
ASSOCIATIONS:	Abstract Parent of all screens used in Distress Client
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public abstract class DistressScreen extends JFrame {

    public static Distress distress;

    protected String   name;

	protected JPanel infoPanel;
	protected JPanel workingPanel;
	protected JPanel commandPanel;

    // constant used for padding the view and edit screens
    protected static String LEFTTXTPADDING = "      ";

    public static void setDistress(Distress mDistress) {
        distress = mDistress;
    }


    /**
	*
	*   DistressScreen constructor 
	*   always sets up three panels (template pattern)
	*   and adds them to the ContentPane
	*/
	public DistressScreen (String mName) {

        super(mName);

        name = mName;

        setBackground(Color.lightGray);
        Container mContainer = getContentPane();
        mContainer.setLayout(new BorderLayout());

        /*
        *	now set up and add the three Panels
        */
        infoPanel    = setUpInfoPanel();
        workingPanel = setUpWorkingPanel();
        commandPanel = setUpCommandPanel();

        mContainer.add(infoPanel,BorderLayout.NORTH);
        mContainer.add(workingPanel,BorderLayout.CENTER);
        mContainer.add(commandPanel,BorderLayout.SOUTH);

	}


	/**																		   |
    *
    *	position a window relative to screen size
    */
    public static void positionWindow(DistressScreen mWindow) {

        // Get the size of the screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();

        // Determine the new location of the window
        int w = mWindow.getSize().width;
        int h = mWindow.getSize().height;
        int x = (dim.width-w)/2;
        int y = (dim.height-h)/3;

        // Move the window
        mWindow.setLocation(x, y);
    }


	/**																		   |
    *
    *	Ensure all Distress screens have setUpInfoPanel, setUpWorkingPanel and 
    *	setUpCommandPanel creating a template pattern.
    */
	public abstract JPanel setUpInfoPanel();
	public abstract JPanel setUpWorkingPanel();
	public abstract JPanel setUpCommandPanel();


	/**																		   |
    *
    *   Standard info box.
    */
    public static void info(String mAlertError) {
        JFrame mframe = new JFrame("Info");

        JOptionPane.showMessageDialog(mframe, mAlertError + "  ",
                                    "Info", JOptionPane.INFORMATION_MESSAGE);

    }
    
    
	/**																		   |
    *
    *   Standard alert box, error!
    */
    public static void alert(String mAlertError) {
        JFrame mframe = new JFrame("Alert");

        JOptionPane.showMessageDialog(mframe, mAlertError + "  ",
                                    "Alert", JOptionPane.ERROR_MESSAGE);
    }


    /**
	*
    *   Quit distress alert
	*
	*   Always asks "Are you sure..."
    */
    public static void quitDistress() {
        JFrame mframe = new JFrame("Quit?");

        int n = JOptionPane.showConfirmDialog(mframe, "Are you sure you wish to quit Distress?  ",
                "Quit?", JOptionPane.YES_NO_OPTION);
        if (n == JOptionPane.YES_OPTION) {
            System.exit(0);
        } else {
            MainDistressScreen.openWindow();
        }
    }


    /**
	*
	*   The WindowListenerQuit extends WindowAdapter
	*   and overrides the windowClosing method to
	*   do a quitDistress call
	*   mMainScreen.addWindowListener(mMainScreen.new MDSWindowListener());
	*/
	protected class MDSWindowListener extends WindowAdapter {
        public void windowClosing (WindowEvent e) {
            quitDistress();
        }
	}
    

	/**
	*
	*   this is a helper method for adding Components to a
	*   JPanel using a GridBagLayout.
	*   The caller specifies the parent JPanel, the Component
	*   to be added, the grid location (row and column), plus the
	*   width and height of each entry (rows by column)
	*/
	protected void addComponent(JPanel parent, Component child, GridBagLayout gridbag,
                                int gridx, int gridy, int gridwidth, int gridheight) {
        GridBagConstraints gbc = new GridBagConstraints(gridx, gridy, gridwidth, gridheight,
                        1.0, 1.0, GridBagConstraints.CENTER,
                        GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 1, 1);
        gridbag.setConstraints(child, gbc);
        parent.add(child);
	}


	/**																		   |
    *
    *	Convert a string to a double
    */
	protected static double stringToDouble(String stringObj) {
        return Double.parseDouble(stringObj.trim());
    }

	/**																		   |
    *
    *	Convert a string to an int
    */
    protected static int stringToInt(String stringObj) {
        return Integer.parseInt(stringObj.trim());
    }
    
    
    /**
    *
    *	Methods to extract the city, state or zip from a cityStateZip String
    */
    public static String getCity(String mCityStateZip) {
        StringTokenizer mTokenizer = new StringTokenizer(mCityStateZip, ",");
        return mTokenizer.nextToken();		// get city
    }
    public static String getState(String mCityStateZip) {
        StringTokenizer mTokenizer = new StringTokenizer(mCityStateZip, " ");
        mTokenizer.nextToken();				// skip city
        return mTokenizer.nextToken();		// get state
    }
    public static String getZip(String mCityStateZip) {
        StringTokenizer mTokenizer = new StringTokenizer(mCityStateZip, " ");
        mTokenizer.nextToken();				// skip city
        mTokenizer.nextToken();				// skip state
        return mTokenizer.nextToken();		// get zip
    }
    
	/**
    *
    *	The toString of Date is tokenized and formatted ("fixed"), so not to print
    *	HH:MM:SS and EST values.
    *
    *	@param	Date		Date to read toString() from
    *	@return	String		the resulting string in form "Mon Jan 05, 2004"
    */
	public static String dateFormatmmddyyyy(Date mDate) {

		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");

		mTokenDate.nextToken();										// DAY	    (skip)
        String mMonth = mTokenDate.nextToken();						// MONTH
        String mReturn = "";
        
        if (mMonth.equals("Jan"))
            mReturn = "01";
        else if (mMonth.equals("Feb"))
            mReturn = "02";
        else if (mMonth.equals("Mar"))
            mReturn = "03";
        else if (mMonth.equals("Apr"))
            mReturn = "04";
        else if (mMonth.equals("May"))
            mReturn = "05";
        else if (mMonth.equals("Jun"))
            mReturn = "06";
        else if (mMonth.equals("Jul"))
            mReturn = "07";
        else if (mMonth.equals("Aug"))
            mReturn = "08";
        else if (mMonth.equals("Sep"))
            mReturn = "09";
        else if (mMonth.equals("Oct"))
            mReturn = "10";
        else if (mMonth.equals("Nov"))
            mReturn = "11";
        else if (mMonth.equals("Dec"))
            mReturn = "12";
        else
            mReturn = mMonth;
            
        mReturn += "/" + mTokenDate.nextToken() + "/";				// DATE
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		mReturn += mTokenDate.nextToken();							// YEAR

		return mReturn;
	}
    
    
	///																		   |
  	//
  	//	Converts a string of form "mm/dd/yy" into a Date object.
  	//	Unsucessful attempts result in null.
  	//
  	//	@param String	The date string in neeed of conversion.
  	//
  	//	@return			A Date object, or null
    //
	public static Date mmddyyyyToDate(String mDateStr) throws DateException {

		Date mReturn = null;							// default return
		
		// ensure three values
		StringTokenizer mmddyyyy = new StringTokenizer(mDateStr, "/");
		if (mmddyyyy.countTokens() == 3) {
		
			int mMonth = stringToInt(mmddyyyy.nextToken());		
			int mDay   = stringToInt(mmddyyyy.nextToken());
			int mYear  = stringToInt(mmddyyyy.nextToken());
	
			// ensure valid month and day values
			if (mMonth >= 1 && mMonth <= 12 &&
				mDay   >= 1 && mDay   <= 31) {
				
				// month is 0 to 11
				Calendar mCal = new GregorianCalendar(mYear, mMonth - 1, mDay);		
				mReturn = mCal.getTime();
			}
		}
	
        if (mReturn == null) {
            throw new DateException("Invalid date \"" + mDateStr + "\"");
        }
        
		return mReturn;
	}
}