/*******************************************************************************
 
 
 	    FILE:	MainDistressScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	This is the first screen shown when the application starts.
                Its infoPanel welcomes the user with a message, the workingPanel 
                has text fields for entering the login and password, and the 
                commandPanel has buttons "Exit" and "Login." Clicking "Exit" 
                causes the program to terminate.  Clicking "Login" attempts to 
                log the user in.  A successful login will take us either to the
                RealtorScreen or ConsumerScreen
				
ASSOCIATIONS:	Opened by the DistressGUIClient
				"Exit" btn attempts to quit
				"Login" btn will try to open the RealtorScreen or ConsumerScreen
				Child of DistressScreen
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/

package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class MainDistressScreen extends DistressScreen {
    
    public static MainDistressScreen	thisWindow = null;
        
    private JTextField					loginFld;
    private JPasswordField				passwordFld;
    private JButton						loginBtn;
	private JButton						exitBtn;
  
	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow();     
	}
    
    
	/**																		   |
    * 
    *	Static method that goes about opening this window and adding 
    *	a window listener to conclude Distress if this window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    */
    public static void openWindow() {
    
        // create instance of this window
        thisWindow = new MainDistressScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        
        thisWindow.setSize(400, 200);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
    }
    
    
   	/**	
    *																   |
    *	static method to hide this window and dispose of it.
    *
    */
    public static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }


	public MainDistressScreen () {
        super("Distress Login");
	}
    
    
	/**																		   |
    *
    *	Sets up the information panel (template pattern)
    *
    *	@return	JPanel		JPanel using a BorderLayout
    */
	public JPanel setUpInfoPanel() {
    
        JPanel mainInfoPanel = new JPanel();
        mainInfoPanel.setLayout(new BorderLayout());
        mainInfoPanel.add(new JLabel("Welcome to DISTRESS",
                                 JLabel.CENTER), BorderLayout.NORTH); 
        mainInfoPanel.add(new JLabel("Please log in by entering your login name and password",
                                 JLabel.CENTER),BorderLayout.CENTER); 
        mainInfoPanel.add(new JLabel("Then click the 'Login' Button below",
                                 JLabel.CENTER), BorderLayout.SOUTH);
                                 
        return mainInfoPanel;
	}


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
    *
    *	@return	JPanel		JPanel using a GridBagLayout
    */
	public JPanel setUpWorkingPanel() {
        GridBagLayout gbLayout = new GridBagLayout();
        JPanel mainWorkingPanel = new JPanel();
        mainWorkingPanel.setLayout(gbLayout);
        
        loginFld     = new JTextField();
        passwordFld  = new JPasswordField();
        
        int col1 = 1;  // column for the labels
        int col2 = 3;  // column for the text field

        int row = 0; // a 'spacer' row
        addComponent(mainWorkingPanel,new Label(" "),
                        gbLayout,0,row,2,1);  
        row = 1;
        addComponent(mainWorkingPanel, new Label(" "),
                        gbLayout, 0, row, 1, 1);  // a spacer cell
        addComponent(mainWorkingPanel, new JLabel("Login Name "),
                        gbLayout, col1, row, 2, 1); 
        addComponent(mainWorkingPanel, loginFld,
                        gbLayout, col2, row, 3, 1); 
        row = 2;
        addComponent(mainWorkingPanel, new JLabel("Password"),
                        gbLayout, col1, row, 2, 1); 
        addComponent(mainWorkingPanel, passwordFld,
                        gbLayout, col2, row, 3, 1);
        addComponent(mainWorkingPanel, new JLabel(" "),
                        gbLayout, col2+3, row, 1, 1);   // a spacer cell
        row = 3; // a 'spacer' row
        addComponent(mainWorkingPanel, new Label(" "),
                        gbLayout, 0, row, 3, 1);   // a spacer cell

        // make KeyListener class that enables or disables Login button
        KeyListener mKeyListener = new KeyListener() {
            // must deff keyPressed & keyReleased, though not used
            public void keyPressed(KeyEvent e) {}
            public void keyTyped(KeyEvent e) {}
            public void keyReleased(KeyEvent e) {
                if (loginFld.getText().equals("") || passwordFld.getPassword().length == 0)
                    loginBtn.setEnabled(false);
                else
                    loginBtn.setEnabled(true);
            }
        };
        
        loginFld.addKeyListener(mKeyListener);
        passwordFld.addKeyListener(mKeyListener);
        
        return mainWorkingPanel;
	}
    

	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*		two buttons with a spacer between them
    *
    *	@return	JPanel		JPanel using a default layout
    */
	public JPanel setUpCommandPanel() {
    
        JPanel mReturn = new JPanel();
        
        /*
        * Set up the JButton for Exit and Login
        */    
        exitBtn =  new JButton("Exit");
        
        /*
        *   The ActionEvent handler for the exitBtn 
        */
        exitBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                quitDistress();
            }
        }); 
   
   
        /*
        *   Set up the JButton for Login
        *   The Login button is disabled and should
        *   be enabled when the user has entered a
        *   login name and password
        */   
        loginBtn = new JButton("Login");
        loginBtn.setEnabled(false);
        loginBtn.addActionListener(new ActionListener() {
        
            public void actionPerformed(ActionEvent ae) {
            
                User mUser = null;

                try {
                
                    if (distress == null)
                        throw new DistressException("Distress not found");
                    
                    mUser = distress.login(loginFld.getText(), 
                                    decipherPassword(passwordFld.getPassword()) );
                    
                    // get next screen
                    MainDistressScreen.closeWindow();
                    
                    if (mUser.getUserType().toString().equals("REALTOR")) {
                        RealtorScreen.openWindow( (Realtor) mUser);
                    } else if (mUser.getUserType().toString().equals("CONSUMER")) {
                        ConsumerScreen.openWindow( (Consumer) mUser);
                    }
        
                } catch (RegistrationException e) {
                    alert(e.getMessage());
                } catch (DistressException e) {
                    alert(e.getMessage());
                } catch (java.rmi.RemoteException e) {
                    alert(e.getMessage());
                }
                
            }
        });
        
        mReturn.add(loginBtn);
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(exitBtn);
        
        return mReturn;
	}


	/**																		   |
    *
    *	Converts a passsword char array into a String
    */
    private String decipherPassword(char[] mInput) {
        String mReturn = "";
        
        int mInputLength = mInput.length;
        for (int i = 0; i < mInputLength; i++)
            mReturn += mInput[i];
        
        return mReturn;
    }
}