/*******************************************************************************
 
 
 	    FILE:	SingleFamilyViewScreen.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project
 
 
 	 PURPOSE:	SingleFamilyViewScreen. The infoPanel has the non-editable
				attributes of the  property.  The workingPanel has all other
				attributes in a non-editable form. The  commandPanel has only a
				'Close' button.  Clicking on 'Close' closes the
				SingleFamilyViewScreen and returns to the SearchResultsScreen.
				
ASSOCIATIONS:	Opened from SearchResultsScreen
				"Close" btn opens SearchResultsScreen
				Child of abstract class DistressScreen
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.distress.client;

import cscie247.distress.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class SingleFamilyViewScreen extends DistressScreen {
    
    public static SingleFamilyViewScreen thisWindow = null;
    public static SingleFamily singleFamily;

    private JButton closeBtn;

	/**																		   |
    *
    *	main allows this screen to be displayed independent of Distress,
    *	which is useful for testing, and saves me having to fireup
    *	the Distress server if I'm not on a network
    */
	public static void main(String args[]) {
    
        distress = null;			// Distress not loaded
        openWindow(null);     
	}
    

    public SingleFamilyViewScreen() {
        super("View Single Family");
    }
    
	/**																		   |
    * 
    *	Static method that goes about opening this window and adding 
    *	a window listener to conclude Distress if this window is closed
    *	The MDSWindowListener is found in the abstract class DistressScreen.java
    *	Note: thisWindow is a static field that contains this window's object
    *	
    *	@param	SingleFamily		The SingleFamily object being viewed
    */
    public static void openWindow(SingleFamily mSingleFamily) {
    
        singleFamily = mSingleFamily;
        
        // instance of this window
        thisWindow = new SingleFamilyViewScreen();
        thisWindow.addWindowListener(thisWindow.new MDSWindowListener());
        thisWindow.setSize(400, 500);
        positionWindow( (DistressScreen) thisWindow );
        thisWindow.setVisible(true);
    }
    
    
   	/**	
    *																   |
    *	static method to hide this window and dispose of it.
    *
    */
    private static void closeWindow() {
        thisWindow.setVisible(false);
        thisWindow = null;
    }

	/**																		   |
    *
    *	Sets up the information panel (template pattern)
    *
    *	@return	JPanel		JPanel using a GridLayout
    */
    public JPanel setUpInfoPanel() {
    
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new GridLayout(0, 2, 2, 2));

        // initialize values to "" (useful if Distress not loaded)
        String mPropertyType = "";
        String mPropertyID   = "";
        
        if (singleFamily != null) {
            try {
                // load properties of singleFamily into vars
                mPropertyType = singleFamily.getPropertyType().toString();
                mPropertyID   = singleFamily.getPropertyID();
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
                             
        // array of labels
        String[] mLabels = { "",
                             "Property Type:", 
                             "Property ID:",
                             "" };
        
        // array of corresponding values
        String[] mValues = { "",
                             mPropertyType, 
                             mPropertyID,
                             "" };
        
        
        // place labels and values into grid layout
        int mItemCount = mLabels.length;
        for (int i = 0; i < mItemCount; i++) {
            mReturn.add(new JLabel(LEFTTXTPADDING + mLabels[i]));
            mReturn.add(new JLabel(mValues[i]));
        }
        
        return mReturn;
    }


	/**																		   |
    *
    *	Sets up the working panel (template pattern)
    *
    *	@return	JPanel		JPanel using a GridLayout
    */
    public JPanel setUpWorkingPanel() {
    
        JPanel mReturn = new JPanel();
        mReturn.setLayout(new GridLayout(0, 2, 4, 4));
        
        // initialize values to "" (useful if Distress not loaded, i.e. singleFamily = null)
        String mDateListed		= "";
        String mAddress			= "";
        String mCity			= "";
        String mState			= "";
        String mZip				= "";
        String mYearBuilt		= "";
        String mRooms			= "";
        String mBaths			= "";
        String mParkingType		= "";
        String mParking			= "";
        String mListingPrice	= "";
        String mLivingSpace		= "";
        String mSpecialFeatures	= "";
        String mDescription		= "";
        String mNumberOfFloors	= "";
        String mLotSize			= "";

        if (singleFamily != null) {
            // load properties of singleFamily into vars
            try {
            
                mDateListed         = dateFormatmmddyyyy(singleFamily.getDateListed());
                mAddress            = singleFamily.getAddress();
                mCity				= getCity(singleFamily.getCityStateZip());
                mState				= getState(singleFamily.getCityStateZip());
                mZip				= getZip(singleFamily.getCityStateZip());
                mYearBuilt          = dateFormatmmddyyyy(singleFamily.getYearBuilt());
                mRooms              = "" + singleFamily.getRooms();
                mBaths              = "" + singleFamily.getBaths();
                mParkingType        = singleFamily.getParkingType().toString();
                mParking            = "" + singleFamily.getParking();
                mListingPrice       = "$" + singleFamily.getListingPrice();
                mLivingSpace        = "" + singleFamily.getLivingSpace();
                mSpecialFeatures    = singleFamily.getSpecialFeatures();
                mDescription        = singleFamily.getDescription();
                mNumberOfFloors     = "" + singleFamily.getNumberOfFloors();
                mLotSize            = "" + singleFamily.getLotSize();
                
            } catch (java.rmi.RemoteException e) {
                alert(e.getMessage());
            }
        }
        
        // array of labels   
        String[] mLabels = { "Date Listed:",
                                "Address:",
                                "City:",
                                "State:",
                                "Zip:",
                                "Year Built:",
                                "Listing Price:",
                                "Rooms:", 
                                "Baths:",
                                "Parking Type:",
                                "Parking:",
                                "Living Space:",
                                "Special Features:",
                                "Description:",
                                "Number of Floors:",
                                "Lot Size:" };
                                
        // array of corresponding values
        String[] mValues = { mDateListed,
                             mAddress, 
                             mCity,
                             mState,
                             mZip,
                             mYearBuilt,
                             mListingPrice,
                             mRooms,
                             mBaths,
                             mParkingType,
                             mParking,
                             mLivingSpace,
                             mSpecialFeatures,
                             mDescription,
                             mNumberOfFloors,
                             mLotSize };
        
        // place labels and values into grid layout
        int mItemCount = mLabels.length;
        for (int i = 0; i < mItemCount; i++) {
            mReturn.add(new JLabel(LEFTTXTPADDING + mLabels[i]));
            mReturn.add(new JLabel(mValues[i]));
        }

        return mReturn;
    }


	/**																		   |
    *
    *	Sets up the command panel (template pattern)
	*   The  commandPanel has only a 'Close' button.
	*   Clicking on 'Close' closes the SingleFamilyViewScreen and 
	*   returns to the SearchResultsScreen.
    *
    *	@return	JPanel		JPanel using a default layout
    */
    public JPanel setUpCommandPanel() {

        /*
        * Set up the JButton for Close
        */
        closeBtn = new JButton("Close");
        closeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeWindow();
                SearchResultsScreen.openWindow();
            }
        });
        
        
        JPanel mReturn = new JPanel();
        
        mReturn.add(new JLabel("     ")); // a spacer
        mReturn.add(closeBtn); 
        
        return mReturn;
    }
}