/*******************************************************************************
 
 
 	    FILE:	DistressWriter.java
  
 	  AUTHOR:	Russell Lowke
     
 	   CLASS:	CSCIE-247, Final Project

 
 	 PURPOSE:	A text output class that "dumps" Distress objects to the screen.
                Has a method for displaying any of the problem domain objects
				
ASSOCIATIONS:	Is instatiated and called by DistressDriver
 	
	MODIFIED:	February 28th 2004
 	  	
	   NOTES:	Tab Width = 4
 
 				Variable naming prefix conventions:
 					m 	= "my", indicates a local variable to a method
 					obj = indicates an object from this package
  	
 
*******************************************************************************/


package cscie247.distress.server;

import cscie247.distress.*;
import java.util.*;


public class DistressWriter {

    public static Distress objDistress;

    ///																		   |
    //
    //	DistressWriter Constructor.
    //
    //
	public DistressWriter(Distress mObjDistress) {
        objDistress = mObjDistress;
    }
    
    
    
	///																		   |
    //
    //	Methods to "dump" each Distress object to the screen.
    //
    //
    public static void dumpAll() throws java.rmi.RemoteException {
    
		System.out.println("\n>>>>>>>>>>>>>>>>>> dumpAll \n");
		
		dumpUser();
        dumpPropertyProfile();
		dumpProperty();		
    }
	public static void dumpUser() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpUsers \n");
        
		Iterator mItr = objDistress.list();
		while(mItr.hasNext())
            prtUser((User) mItr.next());
    }
	public static void dumpConsumer() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpConsumers \n");
    
		Iterator mItr = objDistress.list();
		User	 mUser;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			if (mUser.getUserType() == TypeUser.CONSUMER)
				prtConsumer((Consumer) mUser);
		}
	}
	public static void dumpRealtor() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpRealtors \n");
        
		Iterator mItr = objDistress.list();
		User	 mUser;
		
		while(mItr.hasNext()) {
			mUser = (User) mItr.next();
			if (mUser.getUserType() == TypeUser.REALTOR)
				prtRealtor((Realtor) mUser);
		}
	}
	public static void dumpProperty() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpProperties \n");
        
		Iterator mItr = objDistress.listProperties();
		while(mItr.hasNext())
			prtProperty((Property) mItr.next());
	}
	public static void dumpSingleFamily() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpSingleFamilies \n");
        
		Iterator mItr = objDistress.listProperties();
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.SINGLEFAMILY)
				prtSingleFamily((SingleFamily) mProperty);
		}
	}
	public static void dumpCondominium() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpCondominiums \n");
        
		Iterator mItr = objDistress.listProperties();
		Property mProperty;
		
		while(mItr.hasNext()) {
			mProperty = (Property) mItr.next();
			if (mProperty.getPropertyType() == TypeProperty.CONDOMINIUM)
				prtCondominium((Condominium) mProperty);
		}
	}
	public static void dumpPropertyProfile() throws java.rmi.RemoteException {
    
        System.out.println("-=-=-=-= dumpPropertyProfiles \n");
		Iterator mItr = objDistress.listPropertyProfiles();
        
		while(mItr.hasNext())
			prtPropertyProfile((PropertyProfile) mItr.next());
	}
    
    
	///																		   |
    //
    //	Methods to print a Distress object to the screen.
    //	toString() is not used here, so as to keep a separation between
    //	the concerns of the object data, and the platform specific requirements
    //	of displaying information.
    //
	public static void prtUser(User mUser) throws java.rmi.RemoteException {
    
        // detect what type of User is to be printed,
        //   and redirect appropriately
        if (mUser.getUserType() == TypeUser.CONSUMER)
            prtConsumer((Consumer) mUser);
        else if (mUser.getUserType() == TypeUser.REALTOR)
            prtRealtor((Realtor) mUser);
	}
	public static void prtConsumer(Consumer mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Consumer");
		// System.out.println("    pPropertyProfile = " + mObj.getPropertyProfile());
		System.out.println("pPurchasingTimeFrame = " + mObj.getPurchasingTimeFrame());
		prtUserAbst((User) mObj);
	}
	public static void prtRealtor(Realtor mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Realtor");
		System.out.println("             pAgency = " + mObj.getAgency());
		// System.out.println("         pProperties = " + mObj.getProperties());
		prtUserAbst((User) mObj);
	}
    public static void prtUserAbst(User mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: User << abstract >>");
		System.out.println("           pUserType = " + mObj.getUserType());
		System.out.println("               pName = " + mObj.getName());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("          pLoginName = " + mObj.getLoginName());
		System.out.println("                pPin = " + mObj.getPin());
		System.out.println("              pEmail = " + mObj.getEmail());
		System.out.println("       pCredotCardCo = " + mObj.getCredotCardCo());
		System.out.println("   pCreditCardNumber = " + mObj.getCreditCardNumber());
		System.out.println();
	}
    public static void prtProperty(Property mProperty) throws java.rmi.RemoteException {
    
        // detect what type of Property is to be printed, 
        //   and redirect appropriately
        if (mProperty.getPropertyType() == TypeProperty.SINGLEFAMILY)
            prtSingleFamily((SingleFamily) mProperty);
        else if (mProperty.getPropertyType() == TypeProperty.CONDOMINIUM)
            prtCondominium((Condominium) mProperty);
    }
	public static void prtPropertyAbst(Property mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: Property << abstract >>");
		System.out.println("       pPropertyType = " + mObj.getPropertyType());
		// System.out.println("       pObjContainer = " + mObj.getObjContainer());
		System.out.println("         pPropertyID = " + mObj.getPropertyID());
		System.out.println("            pAddress = " + mObj.getAddress());
		System.out.println("       pCityStateZip = " + mObj.getCityStateZip());
		System.out.println("              pRooms = " + mObj.getRooms());
		System.out.println("              pBaths = " + mObj.getBaths());
		System.out.println("            pParking = " + mObj.getParking());
        System.out.println("          pYearBuilt = " + justYear(mObj.getYearBuilt()));
		System.out.println("        pParkingType = " + mObj.getParkingType());
		System.out.println("       pListingPrice = " + mObj.getListingPrice());
		System.out.println("        pLivingSpace = " + mObj.getLivingSpace());
		System.out.println("    pSpecialFeatures = " + mObj.getSpecialFeatures());
		System.out.println("        pDescription = " + mObj.getDescription());
		System.out.println("         pDateListed = " + dateFormat(mObj.getDateListed()));
		System.out.println("     pNumberOfFloors = " + mObj.getNumberOfFloors());
		System.out.println();
	}
	public static void prtSingleFamily(SingleFamily mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: SingleFamily");
		System.out.println("            pLotSize = " + mObj.getLotSize());
		prtPropertyAbst((Property) mObj);
		
	}
	public static void prtCondominium(Condominium mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS Condominium");
		System.out.println("           pCondoFee = " + mObj.getCondoFee());
		System.out.println("      pNumberOfUnits = " + mObj.getNumberOfUnits());
		prtPropertyAbst((Property) mObj);
	}
	public static void prtPropertyProfile(PropertyProfile mObj) throws java.rmi.RemoteException {
		System.out.println("CLASS: PropertyProfile");
		// System.out.println("       pObjContainer = " + mObj.getContainer());
		System.out.println("             pSearch = " + mObj.getSearchCritera());
        
        // print its SearchCriteria too.
        SearchCriteria mSearchCriteria = mObj.getSearchCritera();
        prtSearchCriteria(mSearchCriteria);
        
        // ...and print SearchItems in SearchCriteria also.
        Iterator mItr = mSearchCriteria.getSearchItems().iterator();
        while(mItr.hasNext())
            prtSearchItem( (SearchItem) mItr.next() );
                
        System.out.println();
	}
	public static void prtSearchCriteria(SearchCriteria mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchCritera");
        System.out.println("        pSearchItems = " + mObj.getSearchItems());
        System.out.println("    pHasPropertyType = " + mObj.getHasPropertyType());
        System.out.println("    pHasCityStateZip = " + mObj.getHasCityStateZip());
    }
    public static void prtSearchItem(SearchItem mObj) throws java.rmi.RemoteException {
        System.out.println("CLASS: SearchItem");
        System.out.println("            pVarName = " + mObj.getVarName());
        System.out.println("                pMin = " + mObj.getMin());
        System.out.println("                pMax = " + mObj.getMax());
    }
    
    
	///
    //
    //	The toString of Date is tokenized and formatted ("fixed"), so not to print
    //	HH:MM:SS and EST values.
    //
    //	@param	Date		Date to read toString() from
    //	@return	String		the resulting string in form "Mon Jan 05, 2004"
    //
	public static String dateFormat(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
		String mReturn = mTokenDate.nextToken() + " " +				// DAY
                         mTokenDate.nextToken() + " " +				// MONTH
                         mTokenDate.nextToken() + ", ";				// DATE
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		mReturn += mTokenDate.nextToken() + " ";					// YEAR
		
		return mReturn;
	}
    
	///
    //
    //	Extract the year from the Date toString.
    //
    //	@param	Date		Date to read toString() from
    //	@return	String		just the year
    //
	public static String justYear(Date mDate) {
	
		StringTokenizer mTokenDate = new StringTokenizer(mDate.toString(), " ");
        
        mTokenDate.nextToken();										// DAY		(skip)
        mTokenDate.nextToken();										// MONTH	(skip)
        mTokenDate.nextToken();										// DATE		(skip)
		mTokenDate.nextToken();										// HH:MM:SS (skip)
		mTokenDate.nextToken();										// EST		(skip)
		String mReturn = mTokenDate.nextToken();					// YEAR
		
		return mReturn;
	}
}


